#!/bin/sh
#@***h* GPCPcdr_d1_GetInputFiles.sh
#       Global Precipitation Climatology Project (GPCP) 
#
# NAME
#   GPCPcdr_d1_GetInputFiles.sh
#
# PURPOSE
#   This code gets file listings for each input dataset 
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 1/2017 Initial Version prepared for NCEI
#
#@*****

# rename inputs
yyyy=$1
mm=$2
sat=$3


# Read location of files from params.h
InputPATH=$(echo `grep "InputPATH" OptionsD.h` | awk -F\"  '{print $2}')
InterimPATHmonthly=$(echo `grep "InterimPATH" ../monthly/OptionsM.h` | awk -F\"  '{print $2}')


# read RSS v6 SSMI Tb files and output to file listing file
# Need to read file from last day of previous month 
# and first file of next month
yyyy_m1=`date -d "${yyyy}-${mm}-01 -1day" "+%Y"`
yyyy_p1=`date -d "${yyyy}-${mm}-01 +1month" "+%Y"`
mm_m1=`date -d "${yyyy}-${mm}-01 -1day" "+%m"`
mm_p1=`date -d "${yyyy}-${mm}-01 +1month" "+%m"`
dd_m1=`date -d "${yyyy}-${mm}-01 -1day" "+%d"`
dd_p1=`date -d "${yyyy}-${mm}-01 +1month" "+%d"`

if [ $yyyy$mm -ge 198707 ] && [ $yyyy$mm -le 200812 ] ; then
  ls -1 ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/${sat}/${yyyy_m1}/${mm_m1}/*D${yyyy_m1}${mm_m1}${dd_m1}_*.nc > file_listings/RSS_Tb_${sat}_${yyyy}${mm}.list
  ls -1 ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/${sat}/${yyyy}/${mm}/*.nc >> file_listings/RSS_Tb_${sat}_${yyyy}${mm}.list
  ls -1 ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/${sat}/${yyyy_p1}/${mm_p1}/*D${yyyy_p1}${mm_p1}${dd_p1}_*.nc >> file_listings/RSS_Tb_${sat}_${yyyy}${mm}.list
elif [ $yyyy$mm -ge 200901 ] ; then
  ls -1 ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/${sat}/${yyyy_m1}/${mm_m1}/*D${yyyy_m1}${mm_m1}${dd_m1}_*.nc > file_listings/RSS_Tb_${sat}_${yyyy}${mm}.list
  ls -1 ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/${sat}/${yyyy}/${mm}/*.nc >> file_listings/RSS_Tb_${sat}_${yyyy}${mm}.list
  ls -1 ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/${sat}/${yyyy_p1}/${mm_p1}/*D${yyyy_p1}${mm_p1}${dd_p1}_*.nc >> file_listings/RSS_Tb_${sat}_${yyyy}${mm}.list
else
  echo "Time period is before SSMI/SSMIS period: file listing not produced."
fi

# get listing of Merged IR files for corr_g
if [ $yyyy$mm -lt 199610 ] ; then
  echo "Time period is before 199610; mergIR 3hrly file listing not produced."
else
  ls -1 ${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}/hr* > file_listings/mergIR_3hr_1deg_${yyyy}${mm}.list
fi



# get list of TOVS files
if [ $yyyy -le 2002 ]; then
    ls -1 ${InputPATH}/TOVS_1DD/${yyyy}/i${yyyy}${mm}*.utime.* > file_listings/TOVS_1DD_${yyyy}${mm}.list
# if AIRS data is required, read fomr 
else
    rm -f file_listings/TOVS_1DD_${yyyy}${mm}.list
    Ndays=`date -d "${yyyy}-${mm}-01 +1 month -1 day" +%d`
    for dd in `seq -w 01 $Ndays`
    do
	file=`find "${InterimPATHmonthly}/AIRS_V6_1DD/${yyyy}/${mm}/" -maxdepth 1 -size +0 -type f -name "*${yyyy}${mm}${dd}.avg"`
	# if no AIRS file was found, try to replace with one of Bolvin's IR files...
	# this is done to be compatible with the historical period of 1DD data
	if [ -z $file ]
	then
	    file=`find "${InputPATH}/TOVS_1DD/${yyyy}/" -maxdepth 1 -size +0 -type f -name "i${yyyy}${mm}${dd}.utime.*"`
	fi
	# if a filename was retrieved, output to file
        if [ -n $file ]
        then
	    echo $file >> file_listings/TOVS_1DD_${yyyy}${mm}.list 
	fi
    done
fi

