      ! Write retrieved quantities to output file for analysis.

      subroutine output_halfdeg(nwork, ipos, more, lat, lon, stdtime,
     $               pixel_status, rain_status, sfcrain,
     $               start_time, sum_sfcrain, sum_ncount, 
     $               sum_nambig, sum_nrpix, grid_time) 





      implicit none

      include 'parameters.inc'


      integer     nwork, ipos, jlin
      integer     testval
      logical     more
      real        lat(NPIX, MAX_SCANS)
      real        lon(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)
      real        sfcrain(NPIX, MAX_SCANS)

      ! Area averaged output variables
      integer     start_time(6)
      real        sum_sfcrain(360,720)
      integer     sum_ncount(360,720)
      integer     sum_nambig(360,720)
      integer     sum_nrpix(360,720)
      integer*1   grid_time(360,720)

      integer     lin, pix, i, ilat, ilon
      integer     check_ymd, check_hms
      integer     scan_minutes, start_minutes
      real        longitude




c
c     Start time is the first time in the orbit.
c     7/11/2002 EJN - To avoid a bogus "19870101000000" start_time
c                     for the first file of a run, search for the
c                     first valid time to load into start_time.  If
c                     the .gdmp file contains only 19870101 times,
c                     load this value into start_time and proceed.
c     5/20/2011 EJN - In all likelihood, jlin will retain the value
c                     of ipos that was passed in, but this check is
c                     kept just in case.

      testval=1
      jlin = ipos - 1
      jlin = jlin + 1
      do while (testval.eq.1)
         check_ymd = stdtime(jlin,1)*10000 + stdtime(jlin,2)*100 +
     &            stdtime(jlin,3)
         check_hms = stdtime(jlin,4)*10000 + stdtime(jlin,5)*100 +
     &            stdtime(jlin,6)
         if (check_ymd .eq. 19870101 .and. check_hms .eq. 0 .and.
     &      jlin.lt.nwork) then
            jlin = jlin + 1
         else
            testval=0
         end if
      end do

c
      do i = 1, 6
        start_time(i) = stdtime(jlin,i)
      end do



c     Start minutes is the time in minutes of the first scan
      start_minutes = (start_time(4) * 60) + start_time(5)

c     Accumulate parameters into 0.25 deg. grid elements
      do lin = jlin, nwork
         ! scan minutes is the time in minutes of the current scan
         scan_minutes = stdtime(lin,4)*60 + stdtime(lin,5)
         IF ( (scan_minutes - start_minutes) .GT. 108 ) THEN
            more = .TRUE.
            ipos = lin
            RETURN
         ENDIF
         do pix = 1, NPIX

            ! skip to end if pixel_status < 0; however, allow -21
            ! to pass through, as set at end of rain-land-noaa.f.

            ! if sfcrain = -999., then also skip to end; this
            ! will also eliminate any pixels whose pixel_status was set
            ! to -21 in qualctrl-v1 due to invalid sfctype < 0.
            ! Note that rain-ctrl set some sfcrain to -999. instead
            ! of MISS_FLT (= -9999.9).  The test below is designed to
            ! catch both of these possible values.

            if ( ( pixel_status(pix,lin) .ge. 0 .OR.
     &           pixel_status(pix,lin) .eq. -21 ) .AND.
     &           sfcrain(pix,lin) .GE. -998. ) then


               ilat = int(180. - 2.*lat(pix,lin)) + 1
               if ( ilat .gt. 360 ) ilat = 360
               if ( lon(pix,lin) .lt. 0.) then
                  longitude = 360. + lon(pix,lin)
               else
                  longitude = lon(pix,lin)
               endif
               
               ilon = int(2.*longitude) + 1
               if ( ilon .gt. 720) ilon = 720
               

               ! Trap for eventual assignment to byte variable
               ! Do not accumulate more than 255 estimates
               if ( sum_ncount(ilat,ilon) .ne. 255 ) then

                  sum_ncount(ilat,ilon) = sum_ncount(ilat,ilon) + 1
                  ! sfcrain can be negative where a pixel is ambiguous, so take
                  ! absolute value when adding.
                  if ( ABS(sfcrain(pix,lin)).gt.0 )
     $                 sum_nrpix(ilat,ilon) = sum_nrpix(ilat,ilon) + 1


c                 Keep track of the time offset in minutes from the starting
c                 scan time for each grid box.  This grid will always contain the
c                 last "time" that a data point was accumulated.
                  grid_time(ilat,ilon) = scan_minutes - start_minutes
 
c                 If day was about to change, grid time can be negative.
                  if ( scan_minutes .lt. start_minutes ) 
     $                 grid_time(ilat,ilon) = 24 * 60 + scan_minutes - 
     $                 start_minutes
                  
                  if ((rain_status(pix,lin) .gt.   0) .or.
     $                 (rain_status(pix,lin) .eq. -31))  
     $                 sum_nambig(ilat,ilon) = sum_nambig(ilat,ilon) + 1


                  ! sfcrain can be negative where a pixel is ambiguous, with
                  ! rain_status = 13 or 14, so take absolute value when adding.
                  if ( rain_status(pix,lin) .ge. 0 ) then
                     sum_sfcrain(ilat,ilon) = sum_sfcrain(ilat,ilon) +
     $                    ABS(sfcrain(pix,lin))
                  endif
               endif
            endif
         end do
      end do 

      more = .FALSE.
      ipos = nwork
      return
      end

