      ! Screening subroutine for land surfaces (sfctype = 1)

      ! Ye Hong          02/09/2000  Modified channel number for AMSR
      ! Eric Nelkin/SSAI 10/29/2003  TRMM Version 6-equivalent, obtained
      !                         from Jeff McCollum & Wanchun Chen (NOAA)
      ! Eric Nelkin/SSAI 11/26/2003  Restore arid soil contamination check

      subroutine screen_land(nwork, Tc, sfctype, sfckind, 
     &           stdtime, pixel_status, rain_status, lat, lon, elev)

      implicit none

      include 'parameters.inc'

      integer     nwork
      real        Tc(NPIX, MAX_SCANS, NCHAN)
      integer*1   sfctype(NPIX, MAX_SCANS)
      integer*1   sfckind(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS,6)   ! YYYY MM DD HH MM SS
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   pixel_status2(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)
      integer*2   rain_status_gscat(NPIX, MAX_SCANS)
      integer*2   screen_status(NPIX, MAX_SCANS)
      real        lat(NPIX, MAX_SCANS)         ! [-90, +90]
      real        lon(NPIX, MAX_SCANS)         ! [-180, +180]
      integer*1   elev(4320,2160)

      integer     lin, pix, ilon, ilat
      integer     i, j, n_ok
      integer     n_dim, n_half, N2, N2N21
      real        sumx, sumxx, sig85h

      logical     app_sfc
      real        DT19, T22V, T85H, T85V

      real        WARM85_LN
      real        AMB_HI, AMB_LO, AMB_INT
      real        POL

      integer     MASKS, MASKP, MASKI, MASKO, MASKC

      integer     step, linc, ypix, lonc, xpix

c     Set values for screens that will be used for rain_status.
c     Masks imply that no rain is possible so mask values < 0.

      MASKS = -21
      MASKP = -41
      MASKI = -31
      MASKO = -51
      MASKC = -61

c     Set threshold values needed by the screening algorithm.
c     The added values are for Tbs.

      WARM85_LN = 257.
      AMB_HI    = 262. + 7.1
      AMB_LO    = 255. + 6.9
      AMB_INT   = 158. + 5.3
      POL       =  20. + 0.25

      do lin = 1, nwork
         do pix = 1, NPIX
		pixel_status2(pix,lin)=pixel_status(pix,lin)
	 enddo
      enddo

      do lin = 1, nwork
         do pix = 1, NPIX
            if ( pixel_status2(pix,lin) .lt. 0 ) goto 10

            ! Identify applicable surfaces.  This screen is
            ! for land surfaces only.

            app_sfc = .FALSE.
            if (sfctype(pix,lin) .EQ. 1) app_sfc = .TRUE.
            ! but skip pixels that might have ice.  They are
            ! screened separately under the ice screen.
            if ( sfckind(pix,lin) .eq. 13 ) app_sfc = .FALSE.

            if ( app_sfc ) then

               ! The following screens come from GSCAT.
               ! (except for the arid soil addition)

               DT19 = Tc(pix,lin,2) - Tc(pix,lin,1)
               T22V = Tc(pix,lin,3)
               T85H = Tc(pix,lin,6)
               T85V = Tc(pix,lin,7)


               !  Warm Tb at 85 but unusually low 22 GHz is defined
               !  as not raining.

               if ((T85H .gt. WARM85_LN).and.(T22V .lt. AMB_HI)) then
                  rain_status(pix,lin) = MASKO

               !  If 85H is not as warm, then a series of other tests
               !  are needed to determine if pixel must be screened.

               elseif (T22V  .lt. AMB_HI) then

                  !  Use Grody criterion to screen for ice surfaces.

                  if ((T22V .lt. AMB_LO) .and.
     &                    (T22V .lt. (AMB_INT + 0.49*T85H))) then
                     rain_status(pix,lin) = MASKI
 
                  !  Check for large polarization at 19 GHz.
 
                  elseif (DT19 .gt. POL ) then
                     rain_status(pix,lin) = MASKP

                  !  Check a second polarization criterion for
                  !  possible ice background contamination.

                  elseif ((DT19 .gt. ((55.+POL)-0.25*T85H)) .and.
     &                      (DT19 .gt. (POL-10.))) then
                     rain_status(pix,lin) = MASKP

                  !  Check a third polarization criterion for
                  !  possible arid soil contamination.

                  elseif ((DT19 .gt. 7) .and. T85V .gt. 253) then
                     rain_status(pix,lin) = MASKP

                  else

                  !  The next tests require the 5x5 variability
                  !  of the 85 GHz.  If sigma cannot be computed
                  !  due to edges or bad values in the 5x5, then
                  !  the whole pixel is thrown out as bad data
                  !  in order not to bias results.

                     n_dim  = 5
                     n_half = 2
                     N2     = n_dim*n_dim
                     N2N21  = n_dim*n_dim*(n_dim*n_dim-1)

                     n_ok = 0
                     do j = lin-n_half, lin+n_half
                        do i  = pix-n_half, pix+n_half
                           if (i .lt. 1 ) goto 30
                           if (i .gt. NPIX) goto 30
                           if (j .lt. 1) goto 30
                           if (j .gt. nwork) goto 30
                           if (pixel_status2(i,j) .lt. 0 ) goto 30
                           n_ok = n_ok + 1
                        end do
                     end do

   30                if (n_ok .ne. N2) then
                     ! temporarily set pixel status to +99
                     pixel_status2(pix,lin) = 99
                     rain_status(pix,lin) = 13
                     !  The 5x5 matrix contains good values

                     else

                     !  Accumulate sums and sums of squares

                        sumx  = 0.
                        sumxx = 0.
                        do j  = lin-n_half, lin+n_half
                           do i = pix-n_half, pix+n_half
                              sumx = sumx + Tc(i,j,6)
                              sumxx = sumxx + Tc(i,j,6)*Tc(i,j,6)
                           end do
                        end do
                        sig85h = sqrt((N2*sumxx - sumx*sumx)/N2N21)

                     !  Assignments for cold 22V, Grody cold ice and
                     !  Grody polarization already cut out.  Low sig85
                     !  and 22V < 249 are probably cold surface.  With
                     !  22V < 262 and high sig85, it is probably rain.
                     !  Cannot discriminate precip from cold surface for
                     !  the remaining in-between sig85 and 22V < 262.

                        if (sig85h .le. 30. ) then

                           !  First case is for ambiguous T22V

                           if ( T22V .ge. AMB_LO ) then
                              rain_status(pix,lin) = 13

                           !  Unambiguous cold surface

                           elseif (sig85h .le. 20. ) then
                              rain_status(pix,lin) = MASKI
 
                           !  The rest is ambiguous

                           else
                              rain_status(pix,lin) = 14
                           end if

                        !  The rest is probably raining.

                        else
                           rain_status(pix,lin) = 0
                        end if
                     end if
                  end if

               !  Grody "sand" polarization test when 22V > 262

               elseif (DT19 .gt. POL ) then
                  rain_status(pix,lin) = MASKP

               !  Check a second polarization criterion for
               !  possible ice background contamination.

               elseif ((DT19 .gt. ((55.+POL)-0.25*T85H)) .and.
     &                      (DT19 .gt. (POL-10.))) then
                  rain_status(pix,lin) = MASKP

               !  Check a third polarization criterion for
               !  possible arid soil contamination.

               elseif ((DT19 .gt. 7) .and. T85V .gt. 253) then
                  rain_status(pix,lin) = MASKP

               else

               !  No more tests.  Rain is possible

                  rain_status(pix,lin) = 0
               end if
            end if
  10     end do
      end do

      do lin = 1, nwork
         do pix = 1, NPIX
            rain_status_gscat(pix,lin)=rain_status(pix,lin)
         enddo
      enddo
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Now all the pixels have been given an initial screening that is in
c rain_status.  The first post-processing is a quality control check to
c search for surface types and check the screens as part of the heritage
c of the subroutine that averaged boxes.
c
c First, set this new grid (screen_status) to missing values
c
      do lin = 1,nwork
         do pix = 1,NPIX
            screen_status(pix,lin) = rain_status(pix,lin)
            if ( pixel_status2(pix,lin).lt.0.or.sfctype(pix,lin).NE. 1
     &      .or.sfckind(pix,lin).eq.13) then
               pixel_status2(pix,lin) = -15
            endif
         enddo
      enddo
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c This is some qc to find pixels with rainfall screens (in screen_status)
c isolated from the neighbors (removed for amsr 7/14/00).
c
      do lin = 1,nwork
         do pix = 1,NPIX

            if(pixel_status2(pix,lin).lt.0) goto 300

            rain_status(pix,lin)=screen_status(pix,lin)

 300    enddo
      enddo
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Now go back through the grid and deal with indeterminate pixels
c
      do lin = 1,nwork
         do pix = 1,NPIX

            if(pixel_status2(pix,lin).lt.0) goto 400

            screen_status(pix,lin)=rain_status(pix,lin)
c
c Check where there is ambiguous rainfall.
c
            if(rain_status(pix,lin).ge.13) then
               screen_status(pix,lin)=-9999
c
c March around the pixel to find pixels with conclusive scattering types, if they exist
c
               step=1
 325           do linc=max0(1,lin-step),min0(lin+step,nwork)
                  ypix=linc
                  do lonc=max0(1,pix-step),min0(NPIX,pix+step)
                     xpix=lonc
c
c Check and see if the pixel was designated as scattering
c
                     if(pixel_status2(xpix,ypix).lt.0) goto 350

                     if(rain_status(xpix,ypix).ge.-41
     &                    .and.rain_status(xpix,ypix).le.0 ) then
c
c If there hasn't been a new assignment of scattering screen, do it now
c
                        if(abs(screen_status(pix,lin)).gt.50) then
c
                           screen_status(pix,lin)=rain_status(xpix,ypix)
c
c If there already has been an assignment, make sure it's the same one as this
c
                        else if(screen_status(pix,lin)
     &                          .ne.rain_status(xpix,ypix)) then
c
c There are two different scattering screens, so make this pixel ambiguous
c
                           screen_status(pix,lin)=13
                           goto 400
                        endif
                     endif
 350              enddo
               enddo
c
c If there is a consensus on a new scattering screen, move to next pixel
c
               if(   screen_status(pix,lin).ge.-41
     &              .and. screen_status(pix,lin).le.0 ) then
                  goto 400
               endif

               step=step+1
c
c Put this check in to avoid trying to read past the end of the orbit.
c
               if(step.le.min0(pix-1,NPIX-pix).or.
     &              step.le.min0(lin-1,(nwork-lin))) then
                  goto 325
               else
                  screen_status(pix,lin)=13
                  goto 400
               endif
            endif
 400     enddo
      enddo
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Now go back through and fill in the individual pixels
c
      do lin = 1,nwork
         do pix = 1,NPIX
            rain_status(pix,lin) = screen_status(pix,lin)
         enddo
      enddo

      ! Finally, set pixels w. status = +99 to "bad"
      do lin = 1, nwork
         do pix = 1, NPIX
           if (pixel_status2(pix,lin) .eq. 99)
     &                       pixel_status(pix,lin) = -15
         end do
      end do

      do lin = 1, nwork
         do pix = 1, NPIX

            if(pixel_status2(pix,lin).lt.0) goto 500

            ! Identify applicable surfaces.  This screen is
            ! for land surfaces only.

           ilat = int((90-lat(pix,lin))*12 + 1)
           ilon = int((lon(pix,lin) + 180)*12 + 1)

           if (ilat.lt.1) ilat = 1
           if (ilat.gt.2160) ilat = 2160
           if (ilon.lt.1) ilon = 1
           if (ilon.gt.4320) ilon = 4320
 
c
c Don't fill in elevations 2 km or higher
c
           if(elev(ilon,ilat).ge.2) then
              app_sfc = .TRUE.
           else
              app_sfc = .FALSE.
c
c Don't fill in for those times of year where we expect snow
c
              if(lat(pix,lin).gt.60) then
                 app_sfc = .TRUE.
              else if(stdtime(lin,2).eq.6. and. lat(pix,lin).gt.55) then
                 app_sfc = .TRUE.
              else if((stdtime(lin,2).eq.5  .or. stdtime(lin,2).eq.10)
     &        .and. lat(pix,lin).gt.50) then
                 app_sfc = .TRUE.
              else if(stdtime(lin,2).eq.4 .and. lat(pix,lin).gt.45) then
                 app_sfc = .TRUE.
              else if((stdtime(lin,2).eq.3  .or. stdtime(lin,2).eq.11)
     &        .and. lat(pix,lin).gt.40) then
                 app_sfc = .TRUE.
              else if((stdtime(lin,2).eq.2  .or. stdtime(lin,2).eq.12)
     &        .and. lat(pix,lin).gt.37.5) then
                 app_sfc = .TRUE.
              else if(stdtime(lin,2).eq.1 .and. lat(pix,lin).gt.35) then
                 app_sfc = .TRUE.
              endif
           endif

           if(pixel_status2(pix,lin).lt. 0 .or.app_sfc) then
              rain_status(pix,lin)=rain_status_gscat(pix,lin)
           endif
 500    end do
      end do

      return
      end
