      SUBROUTINE TA_TO_TB_FULL(TA_LO, TA_HI, NUMSCAN, USESCN, TB_LO,
     &                         TB_HI)
C***********************************************************************
C* SUBROUTINE TA_TO_TB_FULL(TA_LO, TA_HI, NUMSCAN, USESCN, TB_LO,TB_HI)*
C*                                                                     *
C* Called by:  retrieval.F                                             *
C*                                                                     *
C* This subroutine is a revision of the RSS subroutine TA_TO_TB, which *
C* converted a single 7-channel set of SSM/I antenna temperatures to   *
C* brightness temperatures.  This version passes in the complete arrays*
C* of antenna temperatures - all pixels, all A and B scans - for an RSS*
C* orbit file, and returns the complete arrays of brightness           *
C* temperatures.                                                       *
C*                                                                     *
C* Note that the GPROF calling program will need to re-order the       *
C* resulting Tb's to get the desired channel order 19H,19V,22V,37H,37V,*
C* 85H,85V.                                                            *
C*                                                                     *
C* Input Parameters:                                                   *
C*                                                                     *
C*  TA_LO    REAL    SSM/I low-resolution antenna temperature array    *
C*                   (channels 1-5, as 19V,19H,22V,37V,37H)            *
C*  TA_HI    REAL    SSM/I high-resolution antenna temperature array   *
C*                   (channels 6-7, as 85V,85H)                        *
C*  NUMSCAN  INT     number of low-resolution scans contained in this  *
C*                   RSS orbit file                                    *
C*  USESCN   LOGICAL true = scan OK; false = scan not OK               *
C*                                                                     *
C* Output Parameters:                                                  *
C*                                                                     *
C*  TB_LO    REAL    SSM/I low-resolution brightness temperature array *
C*                   (channels 1-5, as 19H,19V,22V,37H,37V)            *
C*  TA_HI    REAL    SSM/I high-resolution brightness temperature array*
C*                   (channels 6-7, as 85H,85V)                        *
C*                                                                     *
C* Written by:    Eric Nelkin/SSAI                                     *
C* Date Written:  September 16, 2011                                   *
C* Modifications: 9/19/2011 EJN   Pass in USESCN to set TB values to   *
C*                                missing for bad-quality scans        *
c                 4/28/2014 SAPIANO  ISTART was defined in a DATA
c                                 statement, but this was causing an
c                                 issue in the CDR code (reasons not
c                                 fully understood). It is now defined
c                                 without a DATA statement
C***********************************************************************
C
      IMPLICIT NONE

      REAL*4 AVV(4),AHV(4),AOV(4),AHH(4),AVH(4),AOH(4)

      INTEGER*4 ISTART,IFREQ
      INTEGER*4 IPIX, ISCAN, I, NUMSCAN
C     REAL*4 TA(7),TB(7)
      REAL*4 TA_LO(5,64,1700), TB_LO(5,64,1700)
      REAL*4 TA_HI(6:7,128,2,1700), TB_HI(6:7,128,2,1700)
      LOGICAL USESCN(1700)
      REAL*4 DELTA(4),CHI(2,4),XFAC,TCOS

C     DATA INITIALIZATION

c      DATA ISTART/1/
      DATA DELTA/0.03199,0.02685,0.01434,0.01186/
      DATA CHI/.00379,.00525,.00983,0.0,.02136,.02664,.01387,.01967/

c     mrpsapiano: define ISTART without DATA statement
      ISTART=1

C     BEGIN EXECUTION
      IF(ISTART.EQ.1) THEN
        ISTART=0
        DO 30 IFREQ=1,4
           IF(IFREQ.EQ.2) GO TO 30
           TCOS=2.7
           IF(IFREQ.EQ.4) TCOS=3.2
           XFAC=(1.-CHI(1,IFREQ)*CHI(2,IFREQ))*(1.-DELTA(IFREQ))
           AVV(IFREQ)=(1.+CHI(1,IFREQ))/XFAC
           AHV(IFREQ)=-CHI(1,IFREQ)*(1.+CHI(2,IFREQ))/XFAC
           AOV(IFREQ)=(1.-AVV(IFREQ)-AHV(IFREQ))*TCOS
           AHH(IFREQ)=(1.+CHI(2,IFREQ))/XFAC
           AVH(IFREQ)=-CHI(2,IFREQ)*(1.+CHI(1,IFREQ))/XFAC
           AOH(IFREQ)=(1.-AHH(IFREQ)-AVH(IFREQ))*TCOS
   30   END DO
      ENDIF

C EJN 9/16/2011
C Adapt the following formulas for use in full low- and high-resolution
C arrays.
C
C     TB(1)=AVV(1)*TA(1) + AHV(1)*TA(2) + AOV(1)
C     TB(2)=AHH(1)*TA(2) + AVH(1)*TA(1) + AOH(1)
C
C     TB(3)=1.01993*TA(3) + 1.994
C
C     TB(4)=AVV(3)*TA(4) + AHV(3)*TA(5) + AOV(3)
C     TB(5)=AHH(3)*TA(5) + AVH(3)*TA(4) + AOH(3)
C
C     TB(6)=AVV(4)*TA(6) + AHV(4)*TA(7) + AOV(4)
C     TB(7)=AHH(4)*TA(7) + AVH(4)*TA(6) + AOH(4)
C

      DO ISCAN = 1, NUMSCAN
         IF ( USESCN(ISCAN) ) THEN
            DO IPIX = 1, 64
               TB_LO(1,IPIX,ISCAN) = AVV(1)*TA_LO(1,IPIX,ISCAN) +
     &                               AHV(1)*TA_LO(2,IPIX,ISCAN) + AOV(1)
               TB_LO(2,IPIX,ISCAN) = AHH(1)*TA_LO(2,IPIX,ISCAN) +
     &                               AVH(1)*TA_LO(1,IPIX,ISCAN) + AOH(1)
C
               TB_LO(3,IPIX,ISCAN) = 1.01993*TA_LO(3,IPIX,ISCAN) + 1.994
C
               TB_LO(4,IPIX,ISCAN) = AVV(3)*TA_LO(4,IPIX,ISCAN) +
     &                               AHV(3)*TA_LO(5,IPIX,ISCAN) + AOV(3)
               TB_LO(5,IPIX,ISCAN) = AHH(3)*TA_LO(5,IPIX,ISCAN) +
     &                               AVH(3)*TA_LO(4,IPIX,ISCAN) + AOH(3)
            END DO
         ELSE
            DO IPIX = 1, 64
               TB_LO(1,IPIX,ISCAN) = -999.
               TB_LO(2,IPIX,ISCAN) = -999.
               TB_LO(3,IPIX,ISCAN) = -999.
               TB_LO(4,IPIX,ISCAN) = -999.
               TB_LO(5,IPIX,ISCAN) = -999.
            END DO
         ENDIF
      END DO
C
      DO ISCAN = 1, NUMSCAN
         IF ( USESCN(ISCAN) ) THEN
            DO I = 1, 2
            DO IPIX = 1, 128
               TB_HI(6,IPIX,I,ISCAN) = AVV(4)*TA_HI(6,IPIX,I,ISCAN) +
     &                             AHV(4)*TA_HI(7,IPIX,I,ISCAN) + AOV(4)
               TB_HI(7,IPIX,I,ISCAN) = AHH(4)*TA_HI(7,IPIX,I,ISCAN) +
     &                             AVH(4)*TA_HI(6,IPIX,I,ISCAN) + AOH(4)
            END DO
            END DO
         ELSE
            DO I = 1, 2
            DO IPIX = 1, 128
               TB_HI(6,IPIX,I,ISCAN) = -999.
               TB_HI(7,IPIX,I,ISCAN) = -999.
            END DO
            END DO
         ENDIF
      END DO
C
C EJN 9/16/2011
C It is likely unnecessary to set the remaining A-scans to missing
C values, since the calling program, retrieval.F, will do this by
C calling clear_scans on the transformed "Tb" array (see there).
C Nevertheless, this is done here to be safe.
C
      DO ISCAN = NUMSCAN+1, 1700
      DO IPIX = 1, 64
         TB_LO(1,IPIX,ISCAN) = -999.
         TB_LO(2,IPIX,ISCAN) = -999.
         TB_LO(3,IPIX,ISCAN) = -999.
         TB_LO(4,IPIX,ISCAN) = -999.
         TB_LO(5,IPIX,ISCAN) = -999.
      END DO
      END DO
C
      DO ISCAN = NUMSCAN+1, 1700
      DO I = 1, 2
      DO IPIX = 1, 128
         TB_HI(6,IPIX,I,ISCAN) = -999.
         TB_HI(7,IPIX,I,ISCAN) = -999.
      END DO
      END DO
      END DO
C
      RETURN 
      END
