      subroutine retrieval(input_file, GPROFPath)
c     Modified version of GPROF V7
c     Obtained from E. Nelkin 

      implicit none


      include 'parameters.inc'

      integer i, j, k, ii, ipos, jpos, chan
      integer npad
      integer nwork, nwrite   
      integer lyr
      integer istrt, istop
      integer start_line, nread, nfound, nrec
      integer first_line, last_line
      character*256 input_file, GPROFPATH
      logical more
      integer inunit, ios, strlen
      logical opn
      character*256 oname
      character*8 cstdate, cendate
      character*6 csttime, centime


c     Define output variables (formerly passed via "common" command)
      integer*4 nscan_lores
      integer*4 nscan_hires
      integer      LOPIX, HIPIX
      parameter   (LOPIX=64, HIPIX=128)
      character*3  sat_name     ! currently 'F13', 'F14', or 'F15'
      integer*4    itime87_ARR(1700)
      integer*2    stdtime_A(1700, 6)  ! YYYY MM DD HH MM SS
      integer*1    iflags(12, 1700)
      real*4       frcrev(1700)
      real*4       xlat(HIPIX,2, 1700)
      real*4       xlon(HIPIX,2, 1700)
      real*4       ta_lo(5,LOPIX, 1700)
      real*4       ta_hi(2,HIPIX,2, 1700)

c     Other former "common" variables
      logical      usescn(1700)
      integer*4    numscan
      real*4       tb_lo(5,LOPIX, 1700)
      real*4       tb_hi(6:7,HIPIX,2, 1700)

      ! Work arrays reserved for satellite input data
      real        Tb (NPIX, MAX_SCANS, NCHAN)  ! K
      real        lat(NPIX, MAX_SCANS)         ! [-90, +90]
      real        lon(NPIX, MAX_SCANS)         ! [-180, +180]
      integer     wentz_time(MAX_SCANS)        ! Wentz times (sec)
      integer*2   wentz_sfctype(NPIX, MAX_SCANS)

      ! For Petty background
      real        T_bkgrd_clear(NPIX, MAX_SCANS, NCHAN) ! K

      ! Arrays used for internal (std.) surface types and times;
      ! Tc is used for modified or corrected brightness temperatures.
      real        Tc(NPIX, MAX_SCANS, NCHAN) ! K
      integer*1   sfctype(NPIX, MAX_SCANS)
      integer*1   sfckind(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)      ! YYYY MM DD HH MM SS

      ! Internal variables
      real        T_bkgrd(NPIX, MAX_SCANS, NCHAN) ! K
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)

      ! Retrieved variables
      real        frzheight(NPIX, MAX_SCANS)    ! km
      real        sfcrain(NPIX, MAX_SCANS)  ! mm/hr
      real        convect(NPIX, MAX_SCANS)  ! mm/hr
      real        quality(NPIX, MAX_SCANS)
      real        clw (NPIX, MAX_SCANS, NLYR)  ! kg/m^2
      real        rain(NPIX, MAX_SCANS, NLYR)  ! kg/m^2
      real        ciw (NPIX, MAX_SCANS, NLYR)
      real        ice (NPIX, MAX_SCANS, NLYR)
      real        Q1(NPIX, MAX_SCANS, 0:NLYR)

      ! Area averaged output variables
      integer     start_time(6)
      real        sum_sfcrain(720,1440)
      integer     sum_ncount(720,1440)
      integer     sum_nambig(720,1440)
      integer     sum_nrpix(720,1440)
      real        sum1_sfcrain(180,360)
      integer     sum1_ncount(180,360)
      integer     sum1_nambig(180,360)
      integer     sum1_nrpix(180,360)
      real        sumH_sfcrain(360,720)
      integer     sumH_ncount(360,720)
      integer     sumH_nambig(360,720)
      integer     sumH_nrpix(360,720)

      ! Variables used for output
      integer*1   gridH_time(360,720)
      integer*2   gridH_sfcrain(360,720)
      integer*1   gridH_ncount(360,720)
      integer*1   gridH_nambig(360,720)
      integer*1   gridH_nrpix(360,720)

      ! Housekeeping variables
      real         conv_obs(NPIX, MAX_SCANS)
      real         rain_obs(NPIX, MAX_SCANS)
      real         emisIndex(NPIX, MAX_SCANS, NEMIS)
      real         scatIndex(NPIX, MAX_SCANS, NSCAT)
      real         chanWgt(NPIX, MAX_SCANS, 7)

      ! Database variables
      integer*1    sfmask1(1440, 720)
      integer*1    sfmask2(1440, 720)
      integer*2    sstdata(0:143,0:90,12)
      real         flttw_tab(131, 131)
      integer*1    elev(4320,2160)

      ! Rainfall profile database variables
      integer      n, ilat, ilon
      logical structure

      integer      dbw_nprofiles(DB_WATER)
      integer      dbw_sfc_code(DB_WATER)
      real         dbw_lyrhgt(DB_WATER,NLYR)
      real         dbw_occur(DB_WATER,DBW_MXPROFILE)
      real         dbw_frzlevel(DB_WATER, DBW_MXPROFILE)
      real         dbw_sfcrain(DB_WATER,DBW_MXPROFILE)
      real         dbw_convect(DB_WATER,DBW_MXPROFILE) 
      real         dbw_faconvmax(DB_WATER,DBW_MXPROFILE,NRING) 
      real         dbw_faconvmean(DB_WATER,DBW_MXPROFILE,NRING) 
      real         dbw_rain_cover(DB_WATER,DBW_MXPROFILE) 
      real         dbw_conv_cover(DB_WATER,DBW_MXPROFILE)
      real         dbw_clw(DB_WATER,DBW_MXPROFILE,NLYR)
      real         dbw_rain(DB_WATER,DBW_MXPROFILE,NLYR)
      real         dbw_ciw(DB_WATER,DBW_MXPROFILE,NLYR) 
      real         dbw_ice(DB_WATER,DBW_MXPROFILE,NLYR)
      real         dbw_Q1(DB_WATER,DBW_MXPROFILE,0:NLYR)
      real         dbw_Tb(DB_WATER,DBW_MXPROFILE,DB_NCHAN)
      real         dbw_Tbclr(DB_WATER,DBW_MXPROFILE,DB_NCHAN)
      real         dbw_Tb_cover(DB_WATER,DBW_MXPROFILE,DB_NCHAN)
      real         dbw_emisIndex(DB_WATER,DBW_MXPROFILE,NEMIS)
      real         dbw_scatIndex(DB_WATER,DBW_MXPROFILE,NSCAT)

      integer      dbl_nprofiles(DB_LAND)
      integer      dbl_sfc_code(DB_LAND)
      real         dbl_lyrhgt(DB_LAND,NLYR)
      integer      dbl_cstype(DB_LAND,DBL_MXPROFILE)
      real         dbl_occur(DB_LAND,DBL_MXPROFILE)
      real         dbl_sfcrain(DB_LAND,DBL_MXPROFILE)
      real         dbl_convect(DB_LAND,DBL_MXPROFILE)
      real         dbl_rain_cover(DB_LAND,DBL_MXPROFILE)
      real         dbl_conv_cover(DB_LAND,DBL_MXPROFILE)
      real         dbl_clw(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbl_rain(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbl_ciw(DB_LAND,DBL_MXPROFILE,NLYR) 
      real         dbl_ice(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbl_Q1(DB_LAND,DBL_MXPROFILE,0:NLYR)
      real         dbl_Tb(DB_LAND,DBL_MXPROFILE,DB_NCHAN)
      real         dbl_Tb_cover(DB_LAND,DBL_MXPROFILE,DB_NCHAN)

      integer      dbc_nprofiles(DB_LAND)
      integer      dbc_sfc_code(DB_LAND)
      real         dbc_lyrhgt(DB_LAND,NLYR)
      integer      dbc_cstype(DB_LAND,DBL_MXPROFILE)
      real         dbc_occur(DB_LAND,DBL_MXPROFILE)
      real         dbc_sfcrain(DB_LAND,DBL_MXPROFILE)
      real         dbc_convect(DB_LAND,DBL_MXPROFILE)
      real         dbc_rain_cover(DB_LAND,DBL_MXPROFILE)
      real         dbc_conv_cover(DB_LAND,DBL_MXPROFILE)
      real         dbc_clw(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbc_rain(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbc_ciw(DB_LAND,DBL_MXPROFILE,NLYR) 
      real         dbc_ice(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbc_Q1(DB_LAND,DBL_MXPROFILE,0:NLYR)
      real         dbc_Tb(DB_LAND,DBL_MXPROFILE,DB_NCHAN)
      real         dbc_Tb_cover(DB_LAND,DBL_MXPROFILE,DB_NCHAN)




C ---------- Initialization ----------


      ! Set variable "structure" to .TRUE. if structure is needed
      structure = .TRUE.

      ! Initialize area averaged output variables
      do i = 1, 6
         start_time(i) = 0
      end do
      do ilat = 1, 720
         do ilon = 1, 1440
            sum_sfcrain(ilat,ilon) = 0.
            sum_ncount(ilat,ilon)  = 0
            sum_nambig(ilat,ilon)  = 0
            sum_nrpix(ilat,ilon)   = 0
         end do
      end do
      do ilat = 1, 180
         do ilon = 1, 360
            sum1_sfcrain(ilat,ilon) = 0.
            sum1_ncount(ilat,ilon)  = 0
            sum1_nambig(ilat,ilon)  = 0
            sum1_nrpix(ilat,ilon)   = 0
         end do
      end do

      do ilat = 1, 360
         do ilon = 1, 720
            sumH_sfcrain(ilat,ilon) = 0.
            sumH_ncount(ilat,ilon)  = 0
            sumH_nambig(ilat,ilon)  = 0
            sumH_nrpix(ilat,ilon)   = 0
         end do
      end do


C  Read databases into memory the first time this subroutine is called.
      call read_dbase(sfmask1, sfmask2, sstdata, 
     &  flttw_tab, dbw_nprofiles, dbw_sfc_code, dbw_lyrhgt,  
     &  dbw_occur, dbw_frzlevel, dbw_sfcrain, dbw_convect, 
     &  dbw_rain_cover, dbw_conv_cover, dbw_faconvmax,  
     &  dbw_faconvmean,  
     &  dbw_clw, dbw_rain, dbw_ciw, dbw_ice, dbw_Q1, dbw_Tb, 
     &  dbw_Tbclr, dbw_Tb_cover, elev, dbl_nprofiles, 
     &  dbl_sfc_code, dbl_lyrhgt, dbl_cstype, dbl_occur, 
     &  dbl_sfcrain, dbl_convect, dbl_rain_cover, dbl_conv_cover, 
     &  dbl_clw, dbl_rain, dbl_ciw, dbl_ice, dbl_Q1, 
     &  dbl_Tb, dbl_Tb_cover, dbc_nprofiles, 
     &  dbc_sfc_code, dbc_lyrhgt, dbc_cstype, dbc_occur, 
     &  dbc_sfcrain, dbc_convect, dbc_rain_cover, dbc_conv_cover, 
     &  dbc_clw, dbc_rain, dbc_ciw, dbc_ice, dbc_Q1, 
     &  dbc_Tb, dbc_Tb_cover)
 




c-------------------------------------------------------
c     Read number of scans from NetCDF4 file containing RSS Tb 
      call ReadNC_RSS_SSMI_Tb_v6_nscans(input_file, 
     &     nscan_lores, nscan_hires)
      NUMSCAN=nscan_lores

c     Read NetCDF4 file containing RSS Tb 
      call ReadNC_RSS_SSMI_Tb_v6(input_file, nscan_lores, nscan_hires,
     &     sat_name, itime87_ARR, stdtime_A,
     &     iflags, frcrev, xlat, xlon, ta_lo, ta_hi)

C EJN: First check the Wentz quality flags to determine which scans are
C      valid.  When set to 1, the first four flag values indicate that
C      the scan should not be used at all, while the next seven flag
C      values indicate that the particular channel (19V, 19H, 22V, 37V,
C      37H, 85V, 85H) should not be used.  (Channel 12 is unused.)
C
      DO K = 1, NUMSCAN
         USESCN(K) = .TRUE.
         DO J = 1, 12
            IF (IFLAGS(J,K) .EQ. 1) THEN
               USESCN(K) = .FALSE.
            ENDIF
         END DO
      END DO


C Convert both the low- and high-resolution SSMI/I antenna temperature
C arrays to brightness temperatures.  Formerly, this was done within
C READ_SSMI_L1A, operating on one set of 7 channels at a time.

      CALL TA_TO_TB_FULL(TA_LO, TA_HI, NUMSCAN, USESCN, TB_LO, TB_HI)



C ---------- Main Retrieval Loop ----------


c     Fill all global variables with default values
c     this used to be a separate subroutine
      do j = 1, MAX_SCANS
         do i = 1, NPIX
            pixel_status(i,j)  = 0
            rain_status(i,j)   = 0
            frzheight(i,j)     = MISS_FLT
            sfcrain(i,j)       = MISS_FLT
            convect(i,j)       = MISS_FLT
            quality(i,j)       = MISS_FLT
            Q1(i,j,0)          = MISS_FLT
            
            if ( structure ) then
               do lyr = 1, NLYR
                  clw(i,j,lyr)  = MISS_FLT
                  rain(i,j,lyr) = MISS_FLT
                  ciw(i,j,lyr)  = MISS_FLT
                  ice(i,j,lyr)  = MISS_FLT
                  Q1(i,j,lyr)   = MISS_FLT
               end do
            endif
         end do
      end do
      
      

C  npad refers to the number of pad lines to expect at the beginning
C  and end of an orbit, whereas NBUFF refers to the number of buffer
C  lines for the interior-orbit segments of NCORE lines each.
      npad = 0


c     MRPS:
c     The original Nelkin code used time to remove the padded scans
c     and overlaps. The RSS CDR does not have overlaps, although it
c     does have padded scans which are removed here
c     Note: Nelkin code does not seem to remove padded scans, so 
c           comparisons of files obtained with his code should
c           have the extra padded scans... something to be mindful of

      do i=1,numscan
         if (frcrev(i) .LT. 0 .OR. frcrev(i) .GT. 1) then
            usescn=.FALSE.
         end if
      end do
      istrt=0
      do i=1,numscan
         if (frcrev(i) .GE. 0) then
            istrt=i
            exit
         end if
      end do
      istop=istrt
      do i=1,numscan
         if (frcrev(i) .GT. 1) then
            exit
         end if
         istop=i
      end do


      

C EJN: If no unique data were found, return to the main program to
C      advance to the next input file in the list.

       IF ( istrt .EQ. 0 .OR. istop .EQ. 0 ) RETURN


C EJN 9/13/2011:
C It is CRITICAL that I line up Tb, lat, lon and stdtime arrays so that
C their line 1 is actually "start_line" as defined by select_unique,
C except in "B-scan coordinates".
C
C EJN 9/19/2011
C Transfer the RSS latitude, longitude values to arrays having full
C A- and B-scans as the second array index, while also adjusting the
C indexing so that the XLAT/XLON values at line istrt are assigned to
C line 1 in the new arrays "lat" and "lon", etc.
C *** Note that READ_SSMI_L1A returns longitudes in range [0,360], so
C values above 180 are adjusted to achieve [-180,+180], as needed by
C this code.
C
C Do the same for stdtime.  Note that the identical time is assigned
C to both the A- and B-scan at each scan pair.
C
C For the brightness temperatures, the RSS channel order 19V,19H,22V,
C 37V,37H,85V,85H must be re-arranged to 19H,19V,22V,37H,37V,85H,85V.
C


      ii = -1
      DO J = istrt, istop
         ii = ii + 2
C
         DO K = 1, 6
            stdtime(ii,K)   = stdtime_A(J,K)
            stdtime(ii+1,K) = stdtime_A(J,K)
         END DO
C
         DO I = 1, NPIX
            lat(I,ii)   = XLAT(I,1,J)
            lat(I,ii+1) = XLAT(I,2,J)
C
            lon(I,ii)   = XLON(I,1,J)
            IF ( lon(I,ii) .GE. 180.) lon(I,ii) = lon(I,ii) - 360.
            lon(I,ii+1) = XLON(I,2,J)
            IF ( lon(I,ii+1) .GE. 180.) lon(I,ii+1) = lon(I,ii+1) - 360.
C 85H:
            Tb(I,ii,6)   = TB_HI(7,I,1,J)
            Tb(I,ii+1,6) = TB_HI(7,I,2,J)
C 85V:
            Tb(I,ii,7)   = TB_HI(6,I,1,J)
            Tb(I,ii+1,7) = TB_HI(6,I,2,J)
         END DO
C


         DO I = 1, NPIX/2
C 19H:
            Tb(2*I-1,  ii, 1) = TB_LO(2,I,J)
            Tb(2*I-1,ii+1, 1) = -999.
            Tb(  2*I,  ii, 1) = -999.
            Tb(  2*I,ii+1, 1) = -999.
C 19V:
            Tb(2*I-1,  ii, 2) = TB_LO(1,I,J)
            Tb(2*I-1,ii+1, 2) = -999.
            Tb(  2*I,  ii, 2) = -999.
            Tb(  2*I,ii+1, 2) = -999.
C 22V:
            Tb(2*I-1,  ii, 3) = TB_LO(3,I,J)
            Tb(2*I-1,ii+1, 3) = -999.
            Tb(  2*I,  ii, 3) = -999.
            Tb(  2*I,ii+1, 3) = -999.
C 37H:
            Tb(2*I-1,  ii, 4) = TB_LO(5,I,J)
            Tb(2*I-1,ii+1, 4) = -999.
            Tb(  2*I,  ii, 4) = -999.
            Tb(  2*I,ii+1, 4) = -999.
C 37V:
            Tb(2*I-1,  ii, 5) = TB_LO(4,I,J)
            Tb(2*I-1,ii+1, 5) = -999.
            Tb(  2*I,  ii, 5) = -999.
            Tb(  2*I,ii+1, 5) = -999.
         END DO
C
      END DO



C EJN 9/14/2011
C The remainder of the code uses nread/nfound/nwork/nwrite in terms
C of the high-resolution A- AND B-scans, whereas istrt and istop were
C computed in terms of A-scans only.  Therefore, the values for these
C "n" variables must be doubled for the rest of the code to work
C properly.  Note that "start_line" is not needed.

      nread = 2 * (istop - istrt + 1)


C EJN: For RSS files, force nfound = nread, to disturb the CLASS TDRR
C      code as little as possible.

      nfound = nread


      if (nfound .LE. nread) then
         nwork = min(nfound, MAX_SCANS)
         nwrite = nwork-npad
         
      ! Fill empty lines with missing values. 
         first_line = nwrite+1
         last_line = MAX_SCANS
               
C  The following two solutions give the same result.  As a group,
C  we should ultimately determine which of the two we prefer.
C
C  Solution 1: Do not call clear_scans if there is a pad.
C
C                 if ( .NOT. pad ) call clear_scans(nfound+1, nread, Tb,
C    &                   lat, lon, wentz_time, wentz_sfctype, stdtime,
C    &                   sfctype, sfckind)
C
C  Solution 2: Modify the argument for the first line to be cleared,
C              to account for a possible pad.
C
CC       call clear_scans(nfound+npad+1, nread, Tb, lat, lon,
CC   &                    wentz_time, wentz_sfctype, stdtime,
CC   &                    sfctype, sfckind)
C EJN 9/9/2011:
C Ordinarily, we use Solution 2 above.  However, it appears here
C that Vila's code never used "first_line" and "last_line" as defined
C above, but should have.  Note also that nfound+npad+1 = first_line
C in this code, so it is only the second argument that has a different
C value now, i.e., "last_line" instead of nread.
C
c         call clear_scans(first_line, last_line, Tb, lat, lon,
c     &                    wentz_time, wentz_sfctype, stdtime,
c     &                    sfctype, sfckind)
c
ccmrps following code was in clear_scans subroutine

         do j = first_line, last_line
            do i = 1, NPIX
               do chan = 1, NCHAN
                  Tb(i, j, chan) = -999.0
               end do
               lat(i, j) = -999.0
               lon(i, j) = -999.0
               wentz_sfctype(i, j) = -1
               sfctype(i, j) = -1
               sfckind(i, j) = -1
            end do
            do i = 1, 6
               stdtime(j,i) = -999
            end do
            wentz_time(j) = -999
         end do
         
         
      else
         nwork  = MAX_SCANS
         nwrite = MAX_SCANS
      end if
 


      ! Deconvolve temperatures to fill high resolution
      ! array.  In this code, low-res temperatures are merely
      ! copied.
      call Tb2Tc_intplte(nwork, Tb, Tc)

      ! Calculate emission and scattering indices for the ocean
      ! database profiles.
      call dbw_emis_scat_Index(dbw_nprofiles, dbw_Tb, dbw_Tbclr, 
     &  dbw_emisIndex, dbw_scatIndex)  

       ! bindmp/ascii use Wentz sfc.  Get standard sfc-type/kind
      call surface_db93_petty(nfound,
     &     sfmask2, lat, lon, sfctype, sfckind)

      ! Perform generic quality control
      call qualctrl(nwork, Tb, Tc, lat, lon, stdtime,
     &           sat_name, sfctype, pixel_status)

      ! Compute auxiliary parameters.
      call frzhgt_ttw(nwork, Tc, flttw_tab, sstdata,
     &     pixel_status, lat, lon, stdtime, sfctype,
     &     frzheight)


      ! Rainfall screening routines.  Each surface type is
      ! handled independently.  Other surface types are
      ! not considered.

      call screen_land(nwork, Tc, sfctype, sfckind, 
     &         stdtime, pixel_status, rain_status, lat, lon, elev)
      call screen_ocean(nwork, Tc, sfctype, sfckind, frzheight,
     &         pixel_status, rain_status)
      call screen_coast(nwork, Tc, sfctype, 
     &         pixel_status, rain_status)
      call screen_ice(nwork, Tc, sfctype, sfckind, 
     &         pixel_status, rain_status)
      
      ! Background Tb - rainfall routines sometimes need
      ! to know the background Tb (where it is not raining).
      call bkgrd_fill(nwork, Tc, sfctype, pixel_status,
     &     rain_status, T_bkgrd)

      ! Compute Petty Clear-air radiances
      call bkgrd_clear_ocean (nwork, lon, lat, stdtime,
     &     pixel_status, sfctype, 1, sstdata,
     &     Tc, T_bkgrd_clear)
      call T_bkgrd_ocean_ctrl(nwork, pixel_status, 
     &     sfctype, T_bkgrd_clear, lat, lon)

      ! Compute emission and scattering indeces over ocean
      call emis_scat_index(nwork, Tc, T_bkgrd, 
     &  1, T_bkgrd_clear, pixel_status, sfctype,
     &  emisIndex, scatIndex)

      ! Compute convective fraction over ocean
      call comp_frac(nwork, Tc, T_bkgrd, pixel_status,  
     &  sfctype, conv_obs, rain_obs, emisIndex, scatIndex)

      ! Rainfall retrieval.
      call rain_ocean(nwork, sfctype, sfckind, frzheight, 
     &      pixel_status, rain_status, dbw_nprofiles, 
     &      dbw_sfc_code, dbw_occur, dbw_frzlevel, dbw_rain_cover,
     &      dbw_sfcrain, dbw_convect, dbw_conv_cover, dbw_faconvmax,
     &      dbw_faconvmean, dbw_clw, dbw_rain, dbw_ciw,
     &      dbw_ice, dbw_Q1, dbw_Tb_cover, dbw_emisIndex,  
     &      dbw_scatIndex, conv_obs, rain_obs, emisIndex, scatIndex,  
     &      sfcrain, convect, quality, structure, clw, rain, ciw, ice, 
     &      Q1) 

      call rain_land(nwork, Tc, sfctype, pixel_status,
     &      rain_status, dbl_nprofiles, dbl_sfc_code,
     &      dbl_occur, dbl_sfcrain, dbl_convect,
     &      dbl_clw, dbl_rain, dbl_ciw, dbl_ice, dbl_Q1,
     &      dbl_Tb, sfcrain, convect, quality, structure,
     &      clw, rain, ciw, ice, Q1, dbl_cstype)

      call rain_coast(nwork, Tc, sfctype, sfckind, frzheight, 
     &      pixel_status, rain_status, dbc_nprofiles, 
     &      dbc_sfc_code, dbc_cstype, dbc_occur, 
     &      dbc_sfcrain, dbc_convect, dbc_clw, dbc_rain, dbc_ciw,  
     &      dbc_ice, dbc_Q1, dbc_Tb, dbc_Tb_cover, sfcrain, 
     &      convect, quality, structure, clw, rain, ciw, ice, Q1)
           


      do i=51,100
         print *, Tc(1,i,1), Tc(1,i,2), Tc(1,i,3), Tc(1,i,4)
         print *, sfcrain(1,i),sfcrain(2,i),sfcrain(3,i),sfcrain(4,i)
      end do




      !==============================================================

      ! Write retrieved parameters to output file.
      ! Only write lines 1 through nwork.  Ordinarily this will be
      ! the contents of the entire core area (i.e. nwork = NCORE),
      ! but on the last iteration of the main retrieval loop (i.e.
      ! nwork < NCORE) this will only be the portion of the core area
      ! that contains data.

      ipos = 1
      more = .FALSE.
      call output_halfdeg(nwrite, ipos, more, lat, lon,
     &     stdtime, pixel_status, rain_status, sfcrain,
     &     start_time, sumH_sfcrain, sumH_ncount, sumH_nambig,
     &     sumH_nrpix, gridH_time)



c     get averages
      do ilat = 1, 360
         do ilon = 1, 720
            if ( sumH_ncount(ilat,ilon) .gt. 0 ) then
               gridH_sfcrain(ilat,ilon) = nint(100.*
     &              sumH_sfcrain(ilat,ilon)/ sumH_ncount(ilat,ilon))
               gridH_ncount(ilat,ilon) = sumH_ncount(ilat,ilon)
               gridH_nambig(ilat,ilon) = sumH_nambig(ilat,ilon)
               gridH_nrpix(ilat,ilon) = sumH_nrpix(ilat,ilon)
            else
               gridH_sfcrain(ilat,ilon) = -9999
               gridH_ncount(ilat,ilon) = 0
               gridH_nambig(ilat,ilon) = 0
               gridH_nrpix(ilat,ilon) = 0
               gridH_time(ilat,ilon) = 0
            endif
         end do
      end do
      
c     construct output filename
      do strlen = 1,len_trim(GPROFPath)
         if (GPROFPath(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      
      IF ( more ) THEN
         jpos = ipos - 1
      ELSE
         jpos = ipos
      ENDIF

      write(cstdate, '(I4.4I2.2I2.2)' ) start_time(1:3)
      write(csttime, '(I2.2I2.2I2.2)' ) start_time(4:6)
      write(cendate, '(I4.4I2.2I2.2)' ) stdtime(jpos,1:3)
      write(centime, '(I2.2I2.2I2.2)' ) stdtime(jpos,4:6)
      oname=GPROFPath(1:strlen)//
     &     "/GPROF2004V."//sat_name//"."//cstdate//"_"//
     &     csttime//"_"//cendate//"_"//centime

      do strlen = 1,len_trim(oname)
         if (oname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1


C     check for a free iounit
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      
c     open output file
      open(inunit, FILE = oname(1:strlen), STATUS = 'replace',
     &     FORM='UNFORMATTED', iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to open output file'
         STOP
      end if
      
c     write data
      write(inunit, iostat=ios) start_time
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_time
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_sfcrain
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_ncount
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_nambig
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_nrpix
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to write output file'
         STOP
      end if
      
      
c     close output file
      close(inunit,iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004:'//
     &        'Failed to close output file'
         STOP
      end if
      
      

      
      end subroutine retrieval




