      subroutine bkgrd_clear_ocean (nwork, lon, lat, stdtime, 
     & pixel_status, sfctype, iinter, sstdata, Tc,
     & T_bkgrd_ocean)

*----------------------------------------------------------------------------
*  Modified:
*    05/02               -  Replaced NBUFF and NCORE with MAX_SCANS      
*
*  Input:
*    nwork         :  # of scans
*    lon           :  longitude
*    lat           :  latitude
*    stdtime       :  standard time
*    pixel_status  :  pixel status
*    sfctype       :  = 0; water
*    iinter        :  = 0 ; no interpolations for background Tbs
*                     = 1 ; interpolations are requested 
*    hitheta       :  hi-resolution satellite local zenith angle
*    sstdata       :  sea surface temperature for 2.5 degree lon/lat
*    Tc            :  hi-resolution brightness temperature

*  Output:
*    T_bkgrd_ocean : background brightness temperature with modifications 
*                     (pixels over the ocean) 
*---------------------------------------------------------------------------

      implicit none           ! Remove at own risk!

      include 'parameters.inc'
      include 'clear_tb_param.inc'

      integer     nwork
      real        lat(NPIX, MAX_SCANS)
      real        lon(NPIX, MAX_SCANS)
c      real 	  hitheta(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)
      integer*2   sstdata(0:143,0:90,12)
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*1   sfctype(NPIX, MAX_SCANS)

      real        Tc(NPIX, MAX_SCANS, NCHAN)
      real        T_bkgrd_ocean(NPIX, MAX_SCANS, NCHAN)
      real	  sst_bkgrd(NPIX, MAX_SCANS)
      real	  wv_petty(NPIX, MAX_SCANS)
      real	  u_petty(NPIX, MAX_SCANS)
      real        wv
      real        u

      integer*2   bkgrd_status(NPIX, MAX_SCANS)
      integer*2   u_status(NPIX, MAX_SCANS)
      integer*2   status

      integer     lin, pix, ic, iterations, ihalf, nch
      integer     ncountmin
      integer     month, row, col
      real        sst
      real        t10h, t10v, t19h, t19v, t23v, t37h, t37v, t85h, t85v
 
* Chiu
      integer     iinter           ! control if interpolations are done
      integer     iconti           ! control if iteration continues
      integer     ichange          ! determine if reducing min # of pixels
 
* initialization
      do lin = 1, nwork
         do pix = 1, NPIX
            sst_bkgrd(pix,lin)    = -100.
            wv_petty(pix,lin)     = -200.
            u_petty(pix,lin)      = -200.
            bkgrd_status(pix,lin) = -91
            u_status(pix,lin)     = -91
            do ic = 1, NCHAN
               T_bkgrd_ocean(pix,lin,ic) = -200. 
            enddo
         enddo
      enddo            

      do lin = 1, nwork

         do pix = 1, NPIX


* obatin sst
            ! Skip if pixel is not O.K.
            if ((pixel_status(pix,lin).ge.0).and.
     &          (sfctype(pix,lin).eq.0)) then  ! water only

              month = stdtime(lin,2)
              row = int((lon(pix,lin) + 180.0)/2.5 )
              col = int((lat(pix,lin) + 90.0)/2.0)

              if(row .gt. 143)then
                 row = 143
              else if(row .lt. 0)then
                 row = 0
              endif
              if(col .gt. 90)then
                 col = 90
              else if(col .lt. 0)then
                 col = 0
              endif

              month=11
              sst = 0.1*sstdata(row, col, month)

              t19h = Tc(pix,lin,1)
              t19v = Tc(pix,lin,2)
              t23v = Tc(pix,lin,3)
              t37h = Tc(pix,lin,4)
              t37v = Tc(pix,lin,5)
              t85h = Tc(pix,lin,6)
              t85v = Tc(pix,lin,7)

              call get_clear_tbs(t19v,t19h,t23v,
     &             t37v,t37h,t85v,t85h,sst,
     &             wv,u, status)

	      sst_bkgrd(pix,lin) = sst
              bkgrd_status(pix,lin) = status
              u_status(pix,lin) = status
              wv_petty(pix,lin) = wv
              u_petty(pix,lin) = u

            endif
         enddo
      enddo
      
c      call wvu_ctrl(nwork, pixel_status, sfctype, wv_petty, u_petty,
c     &              lat, lon, bkgrd_status, u_status)


* the following is to interpolate T_bkgrd if pixels are missing...
      if (iinter.eq.0) then 
          return
      else
          call miss_flag(nwork,ista2,wv_petty,bkgrd_status)
          call average(nwork,1,ista2,wv_petty,bkgrd_status)

          call miss_flag(nwork,ista1,u_petty,u_status)
          call average(nwork,1,ista1,u_petty,u_status)

          ihalf = 2
          ncountmin = ((2*ihalf+1)**2)/3
          ichange=0        !0=no pixel changes; 1=at leat one changes
*         print *,'interpolated domain and threshold: ',ihalf,ncountmin
          iterations = 0

* water vapor -------------------

 10       continue
          iterations = iterations +1
*         print *,'iterations: ', iterations
          if (iterations.gt.1000) go to 100

          call inter_cal(nwork,ihalf,ncountmin,ista_ave,
     &                   wv_petty,bkgrd_status,ichange,iconti)

* at least one pixel changed after this iteration (ichange=1)
          if (ichange.ne.0) go to 10  

* no pixels change: 
*     1) If some pixels are still not done yet (iconti=1), reduce 
*        ncountmin and iterate again.
*     2) If everything is done (iconti=0), leave the iteration loop.

          if (iconti.eq.0) go to 20  ! case(2)

          ncountmin=ncountmin-1
          ichange=0
*         print *,'reducing ncountmin to ', ncountmin

          if (ncountmin.ge.1) then
              go to 10
          else
              go to 20
          endif
 100      print *,'Number of interations exceeds 1000'
          STOP
 20       continue  !print *,'Iterations for water vapor is done.'
                    

* wind speed --------
          ncountmin = ((2*ihalf+1)**2)/3
          ichange=0        !0=no pixel changes; 1=at leat one changes
          iconti=0
          iterations = 0

 30       iterations = iterations +1
*         print *,'iterations: ', iterations
          if (iterations.gt.1000) go to 100

          call inter_cal(nwork,ihalf,ncountmin,ista_ave,
     &                   u_petty,u_status,ichange,iconti)
         
          if (ichange.ne.0) go to 30  
          if (iconti.eq.0) go to 40  
          ncountmin=ncountmin-1
          ichange=0
          if (ncountmin.ge.1) then
              go to 30
          else
              go to 40
          endif
 40       continue !print *,'Iterations for windspeed is done.'


* calculation background tbs using regression equations

          call reg_clear_tbs (nwork,wv_petty,u_petty,
     &         bkgrd_status,u_status,T_bkgrd_ocean)
c          print *,' completion of calculations of tbs'


* replace T_bkgrd of pixels over the ocean with updated T_bkgrd_ocean

    ! not necessary

          do lin = 1, nwork
             do pix = 1, NPIX
c                if ((bkgrd_status(pix,lin).ge.ista_ave).and.
c     &              (u_status(pix,lin).ge.ista_ave)) then  
                    do ic = 1, NCHAN
c                       T_bkgrd(pix,lin,ic) = T_bkgrd_ocean(pix,lin,ic) 
c               write(*,*) T_bkgrd_ocean(pix,lin,ic),pix,lin,ic
                    enddo
c                endif
             enddo
          enddo

    ! TEST
                   
      endif
   
      return
      end


*=========================================================================

      subroutine inter_cal(nwork,ihalf,ncountmin,ismin,
     .                     xarray,sarray,ichange,iconti)

*----------------------------------------------------------------------------------
* ... interpolate missing data (pixels over land/coast, with possible rain, and
*     those adjacent to the above pixels)

* Input
*     nwork    : # of scans
*     ihalf    : half # of grid points of the domain for interpolations
*                eg. ihalf=2 if 5x5 domain is desired.
*     ncountmin: threshold of # pixels for interpolations over the desired domain 
*                eg. ncountmin=17: do interpolations if there are 17 valid data
*                within the 5x5 domain. 
*     ismin    : threshold of status of valid data
*     xarray   : array of values
*     sarray   : array of status  

* Output
*     xarray   : array of values
*     sarray   : array of status (status=3 after the pixel is interpolated) 
*     ichange  : =0; none of pixles is changed in the criteria (ihalf&ncountmin)  
*                    it means that we need to reduce ncountmin...
*                =1; at least one pixel is changed in the iteration.
*     iconti   : =0; no more interpolations are needed. (stop)
*                =1; more interpolations are needed. (continue)
*----------------------------------------------------------------------------------

      implicit none           ! Remove at own risk!

      include 'parameters.inc'

      integer     nwork

      real	  xarray(NPIX, MAX_SCANS)
      real	  scratch(NPIX, MAX_SCANS)
      integer*2   sarray(NPIX, MAX_SCANS)
      integer*2   iter_status(NPIX, MAX_SCANS)
      integer*2   status
      integer     lin, pix, ncount, ncountmin
      integer     ichange          ! determine if reducing min # of pixels
      integer     iconti           ! 1 means some pixels are not done yet
      integer     isdone, ismin           
      integer     ii, i1, i2, jj, j1, j2, jmin, jmax, ihalf
      real        x0, x, sum


      JMIN = 1   
      JMAX = nwork 
      ichange=0
      iconti=0
      isdone = ismin+1

      do pix=1,NPIX
         i1 = pix -ihalf
         i2 = pix +ihalf
         if (i1 .lt. 1) i1 = 1
         if (i2 .gt. NPIX) i2 = NPIX

         do lin=JMIN,JMAX
            x0 = xarray(pix,lin)
            status = sarray(pix,lin)
            if ((status.ge.ismin).or.(status.eq.-91)) then
                scratch(pix,lin) = x0
                iter_status(pix,lin)=status
            else
                j1 = lin -ihalf
                j2 = lin +ihalf
                if (j1 .lt. JMIN) j1 = JMIN
                if (j2 .gt. JMAX) j2 = JMAX

                sum = 0.0
                ncount = 0
                do ii = i1,i2
                   do jj = j1,j2
                       x = xarray(ii,jj)
                       if (sarray(ii,jj). ge. ismin) then
                           sum = sum + x
                           ncount = ncount + 1
                       endif
                   enddo
                enddo

                if (ncount .ge. ncountmin) then
                    scratch(pix,lin) = sum/float(ncount)
                    iter_status(pix,lin) = isdone
                    ichange = 1     !at least one pixel changes
                else
                    scratch(pix,lin) = x0
                    iter_status(pix,lin)=status
                    iconti = 1
                endif
            endif
         enddo
      enddo
 
* end loop over pixels

      do pix=1,NPIX
         do lin=JMIN,JMAX
            xarray(pix,lin) = scratch(pix,lin)
            sarray(pix,lin) = iter_status(pix,lin)
         enddo
      enddo

      return 
      end


*=======================================================================

      subroutine miss_flag(nwork,ismin,xarray,sarray)

*-----------------------------------------------------------------------------
* ... flag pixels as missing data, which are over land/coast, or with possible 
*     rain, or adjacent the aforementioned pixels.
* ... All missing pixles will be flagged as value=-300. and status=-81 to be
*     distinguished.         

* Input:
*     nwork : # of scans
*     ismin : threshold of status of valid data
*     xarray: array of data (water vapor or wind speed)
*     sarray: array of status of data
* Output:
*     xarray: array of data (water vapor or wind speed)
*     sarray: array of status of data
*-----------------------------------------------------------------------------

      implicit none           ! Remove at own risk!
 
      include 'parameters.inc'

      integer     nwork

      real	  xarray(NPIX, MAX_SCANS)
      real	  scratch(NPIX, MAX_SCANS)
      integer*2   sarray(NPIX, MAX_SCANS)
      integer*2   iter_status(NPIX, MAX_SCANS)
      integer*2   status
      integer     lin, pix, ismin
      integer     ii, i1, i2, jj, j1, j2, jmax, jmin
      real        x0

      JMIN = 1
      JMAX = nwork

* The initialization is done to make sure each pixel has been through the 
* process of checking missing flag.  No pixel should be with -999. of scratch
* and -99 of iter_status. 


      do pix = 1, NPIX
         do lin = JMIN, JMAX
            scratch(pix,lin)=-999.
            iter_status(pix,lin) = -99
         enddo
      enddo

      do pix = 1, NPIX
         i1 = pix-1
         i2 = pix+1
         if (i1.lt.1) i1=1
         if (i2.gt.NPIX) i2=NPIX

         do lin = JMIN, JMAX
            x0 = xarray(pix,lin)
            status = sarray(pix,lin)
            if (status.lt.ismin) then
                scratch(pix,lin) = x0
                iter_status(pix,lin)=status
             
            else

               j1 = lin-1
               j2 = lin+1
               if (j1.lt.JMIN) j1=JMIN
               if (j2.gt.JMAX) j2=JMAX 
               do ii = i1, i2
                  do jj = j1, j2
                     if (sarray(ii,jj).lt.ismin) then
                        scratch(pix,lin)=-300.
                        iter_status(pix,lin)=-81
                        go to 10
                     endif
                  enddo
               enddo
               scratch(pix,lin) = x0
               iter_status(pix,lin) = status
            endif
 10         continue
	 enddo
      enddo

      do pix = 1, NPIX
         do lin = JMIN, JMAX
            xarray(pix,lin) = scratch(pix,lin)
            sarray(pix,lin) = iter_status(pix,lin)
         enddo
      enddo
      
      return
      end


*=======================================================================

      subroutine average(nwork,npoint,ismin,xarray,sarray)

*----------------------------------------------------------------------------------
* ... do 3x3 averaging
* ... Averaging is processed whenever there is at least one fine pixel 
*     within the 3x3 box.  The criteria may be modified if necessary.

* Input:
*     nwork : # of scan lines
*     npoint: # of grid points that you would like to do averaging.  For 
*             example:  if over 3x3 box, npoint=1; if over 5x5, npoint=2
*     ismin : threshold status of valid data (refer to setup in clear_tb_param.inc)
*     xarray: array of values
*     sarray: array of status
* Output
*     xarray: array of values
*     sarray: array of status (status=ista_ave after the pixel is averaged) 
*----------------------------------------------------------------------------------

      implicit none           ! Remove at own risk!

      include 'parameters.inc'

      integer     nwork, npoint

      real	  xarray(NPIX, MAX_SCANS)
      real	  scratch(NPIX, MAX_SCANS)
      integer*2   sarray(NPIX, MAX_SCANS)
      integer*2   iter_status(NPIX, MAX_SCANS)
c      integer*2   ista_ave
      integer     lin, pix, ncount, ismin
      integer     ii, i1, i2, jj, j1, j2, jmin, jmax
      real        x, sum

      include 'clear_tb_param.inc'

      JMIN = 1
      JMAX = nwork

      do pix = 1, NPIX
         i1 = pix - npoint
         i2 = pix + npoint
         if (i1.lt.1) i1=1
         if (i2.gt.NPIX) i2=NPIX

         do lin = JMIN, JMAX
            j1 = lin - npoint
	    j2 = lin + npoint
	    if (j1.lt.JMIN) j1=JMIN
	    if (j2.gt.JMAX) j2=JMAX
 
            sum=0.
            ncount=0
	    do ii = i1, i2
	       do jj = j1, j2
                  x = xarray(ii,jj)
	          if (sarray(ii,jj).ge.ismin) then
                      sum=sum+x
                      ncount=ncount+1
                  endif
               enddo
            enddo

            if (ncount .ne. 0) then
                scratch(pix,lin) = sum/float(ncount)
                iter_status(pix,lin) = ista_ave
            else
                scratch(pix,lin)=xarray(pix,lin)
                iter_status(pix,lin) = sarray(pix,lin)
            endif
	 enddo
      enddo

      do pix = 1, NPIX
         do lin = JMIN, JMAX
            xarray(pix,lin) = scratch(pix,lin)
            sarray(pix,lin) = iter_status(pix,lin)
         enddo
      enddo
      
      return
      end

*===========================================================================

      subroutine reg_clear_tbs (nwork,wvarray,uarray,wvs,us,
     &     T_bkgrd_ocean)

*-------------------------------------------------------------------------------
* ... Called after all calculations/interpolation for water vapor and wind speed
*     fields are done.   
*
* Input: 
*     nwork : # of scans
*     wvarray: array of retrieved water vapor
*     uarray: array of retrieved wind speed
*     wvs: array of status of water vapor
*     us : array of status of wind speed
* Output:
*     T_bkgrd_ocean: background brightness temperatures 
*-------------------------------------------------------------------------------

      implicit none           ! Remove at own risk!

      include 'parameters.inc'

      integer     nwork

      real        T_bkgrd_ocean(NPIX, MAX_SCANS, NCHAN)
      real	  wvarray(NPIX, MAX_SCANS)
      real	  uarray(NPIX, MAX_SCANS)
      real        anomt(NCHAN)
      real        wv
      real        u
      integer*2   wvs(NPIX, MAX_SCANS)
      integer*2   us(NPIX, MAX_SCANS)
      integer*2   is1,is2
      integer     pix,lin,ich

      include 'clear_tb_param.inc'


      do 30 pix = 1, NPIX
         do 20 lin = 1, nwork
            is1 = wvs(pix,lin)
            is2 = us(pix,lin)
            if (is1.lt.ista_ave) go to 20
            if (is2.lt.ista_ave) go to 20
            wv =wvarray(pix,lin)
            u = uarray(pix,lin)
            do 10 ich = 1, NCHAN
               anomt(ich) = anom0(ich) + wv*wvcor(ich) 
               T_bkgrd_ocean(pix,lin,ich) = ta - exp(anomt(ich)-
     +                                      ucor(ich)*u)
 10         continue
 20      continue
 30   continue

      return 
      end

