      ! Subroutine to interpolate low res. channels into hi res.
      ! Right now it is copying the low res. channels - this needs
      ! to be changed.
      ! Eric Nelkin/SSAI  3/08/2005  If an unphysical Tb (outside [50,
      !                              325K] is encountered in the A-scan,
      !                              set all NPIX values on the A-scan
      !                              to -9999.9, at that channel.  Allow 
      !                              B-scan interpolation to proceed, as
      !                              this will generate negative (but
      !                              not fixed) values that will be
      !                              caught by qualctrl-v1.

      subroutine Tb2Tc_intplte(nwork, Tb, Tc)

      implicit    none
      include     'parameters.inc'

      integer     nwork
      real        Tb(NPIX, MAX_SCANS, NCHAN)
      real        Tc(NPIX, MAX_SCANS, NCHAN)

      real        temp(NPIX, MAX_SCANS, NCHAN)

      integer     chan, lin, pix

      real        Tbmin, Tbmax
      data Tbmin, Tbmax / 50.0, 325.0 /
      logical     BADSCAN
c
c     Interpolate along the scan.  Extrapolate the last point.
c
      do chan = 1, 5
         do lin = 1, nwork, 2
c
c     First check all the A-scan pixels for unphysical values, until one
c     is found.  If so, the logical variable BADSCAN is set to .TRUE.
c

            BADSCAN = .FALSE.
            do pix = 1, NPIX, 2
               IF (Tb(pix,lin,chan) .LT. Tbmin .OR.
     &             Tb(pix,lin,chan) .GT. Tbmax) then
                  BADSCAN = .TRUE.
                  go to 10
               ENDIF
            end do
   10       continue
c
c     If an unphysical Tb was found, set all NPIX values along that
c     scan, at that channel, to a missing value of -9999.9.  The
c     missing value must be a large enough negative number to ensure
c     that the across-scan interpolation results in a negative number,
c     so that it will be caught in qualctrl-v1.f.  Hard-coding to
c     -9999.9 is done in case the MISS_FLT value in parameters.inc has
c     been set to too small a negative value to ensure an interpolated
c     value less than zero.
c
            IF (BADSCAN) THEN
               do pix = 1, NPIX, 2
                  temp(pix,lin,chan)   = -9999.9
                  temp(pix+1,lin,chan) = -9999.9
               end do
            ELSE
               do pix = 1, NPIX-2, 2
                  temp(pix,lin,chan)    = Tb(pix,lin,chan)
                  temp(pix+1,lin,chan)  = 0.5*(Tb(pix,lin,chan) + 
     $                                         Tb(pix+2,lin,chan))
               end do
               temp(NPIX-1,lin,chan)    = Tb(NPIX-1,lin,chan)
               temp(NPIX,lin,chan)      = Tb(NPIX-1,lin,chan)
            ENDIF
         end do
      end do
c
c     Now interpolate across the scan.
c
      do chan = 1, 5
         do pix = 1, NPIX
            do lin = 1, nwork-2, 2
               Tc(pix,lin,chan)    = temp(pix,lin,chan)
               Tc(pix,lin+1,chan)  = 0.5*(temp(pix,lin,chan) + 
     $                                    temp(pix,lin+2,chan))
            end do
            Tc(pix,nwork-1,chan) = temp(pix,nwork-1,chan)
            Tc(pix,nwork,chan)   = temp(pix,nwork-1,chan)
         end do
      end do
c
c     Now rename the high resolution channels
c
      do chan = 6, 7
         do lin = 1, nwork
            do pix = 1, NPIX
               Tc(pix,lin,chan)     = Tb(pix,lin,chan)
            end do
         end do
      end do

      return
      end


