C@***h* GPCP_Daily_Analysis/Main_GPCPcdr_d2_GPROFv2004_SSMIS.f
C       Global Precipitation Climatology Project (GPCP)
C
C NAME
C   Main_GPCPcdr_d2_GPROFv2004_SSMIS.f
C
C PURPOSE
C   This code takes the histogram file and produces a rainrate based on 
C   lookup tables derived from an RT model.
C
C DESCRIPTION
c   This program is the main entry way to run GPROF v2004. This code simply 
C   reads a list of files and calls the subroutine to do the retrieval
C
C   Original code ran with implicit declarations - need to figure out 
C   what each of these variables should be:
C   implicit none
C  
C AUTHOR
C   Robert Adler, Mathew Sapiano, Jian-Jian Wang
C   University of Maryland College Park
C
C COPYRIGHT
C   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE
C   PUBLIC
C   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
C   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
C   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
C   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
C   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
C   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE
C   TECHNICAL
C   SUPPORT TO USERS.
C
C HISTORY
C 1/2017 Initial Version prepared for NCEI
C
C@*****      

      program Main_GPCPcdr_d2_GPROFv2004_SSMIS
      implicit none
      INCLUDE 'netcdf.inc'


      integer*4      strlen, Ninputs, inunit, ios
      integer*4      iyear, imonth
      character*256  GPROFPATH, file_list, input_file
      character*2    cmonth
      character*3    sat
      character*4    cyear
      logical*1      opn


c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 4 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        ' Incorrect number of input arguments (should be 4)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, sat)
      call getarg(4, GPROFPATH)

c     Convert month and year to numbers
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth



c     ******************************************************
c     Open list of files

      file_list="file_listings/"//
     &     "/RSS_Tb_"//sat//"_"//cyear//cmonth//".list"


C     check for a free iounit
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

      open(inunit, file=file_list, IOSTAT = ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in: Main_GPCPcdr_d2_GPROFv2004_SSMIS'//
     &            'Failed to open input ASCII file'
         STOP
      endif



c     ******************************************************
c     Read each filename and process

      do while (ios .EQ. 0)
         read(inunit,'(A)', IOSTAT = ios) input_file
         if ( ios .NE. 0 .AND. ios .NE. -1 ) then
            WRITE(*,*) 'Error in Main_GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &             'Failed to read input ASCII file'
            STOP
         endif


         print *, "Processing: "//input_file

c        Call retrieval code
         call retrieval_SSMIS(input_file, GPROFPath)



c     Close loop over files
      end do

c     Close file
      close(inunit, IOSTAT = ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in: Main_GPCPcdr_d2_GPROFv2004_SSMIS'//
     &            'Failed to close input ASCII file'
         STOP
      endif





      end program Main_GPCPcdr_d2_GPROFv2004_SSMIS


