	SUBROUTINE HISTO_MATCH(first_line, nread, nfound,
     +    	TB, lat, lon,
     +	       	a19v, a19h, a22v, a37v, a37h, a91v, a91h,  
     +    	alat1, alon1, alat2, alon2, alat3, alon3, 
     +          lnd, lnd_hi, aline)
C
C  History:
C  Sapiano, Jan2009:  added "goto 101" line to correct a bug
C  Vila, Feb2010:     added eyear and ejday to take into account turn into new
C                     day and new year.
C  Nelkin, May 2011:  change "btime" to real*4 (was integer*4) for
C                     compatibility with select_unique
C  Nelkin, May 2011:  make irec a local variable
C  Nelkin, May 2011:  swap roles of irec, iline to properly handle
C                     first_line > 1 (as in cases of overlap with
C                     previous orbit file)
C  Nelkin, May 2011:  change (irec.ge.aline) to (iline.gt.aline) to
C                     allow processing of final scan line
C  Vila, Oct 2011:    new histogram-matching 91 GHz Ta adjustment to SSMI
C                     based on sfc type
C  Nelkin, Oct 2011:  modify Vila's arguments in call surface_db93_petty
C  Nelkin, Oct 2011:  set [1,161] bounds on kindex computations
C  Nelkin, Nov 2011:  pass in add_oce, add_lnd arrays, read in main
C                     program and passed here via subroutine retrieval
C  Nelkin, Nov 2011:  pass in cjobid, do_debug
C  Nelkin, Dec 2011:  modified copy of ingest-ssmis-tdrr; stdtime no
C                     longer needs to be computed, since it has been
C                     computed in retrieval; remove cjobid, do_debug
C  Nelkin, Dec 2011:  modify longitude averaging near dateline
C  Nelkin, Dec 2011:  set TB=0 if TA < 50K
C  Nelkin, Dec 2011:  don't do histogram matching if Tb value is < 50 K
C  Nelkin, Jan 2012:  eliminate surface-type-based look-up tables that
C                     Vila developed on CLASS data for SSMIS-to-SSMI
C                     calibration (since RSS already accounts for this)
C
      	implicit none
	include	'parameters.inc'


	!==== input parameters =================================================

        integer*2       LAT_SCENES, LON_SCENES      ! lat-lon matrixes
        parameter       (LAT_SCENES=3)
        parameter       (LON_SCENES=3)

	real*4 		a19v(NPIX/2,MAX_SCANS)      !array 19V data in kelvin
	real*4 		a19h(NPIX/2,MAX_SCANS)      !array 19H data in kelvin
	real*4 		a22v(NPIX/2,MAX_SCANS)      !array 22V data in kelvin
	real*4 		a37v(NPIX/2,MAX_SCANS)      !array 37V data in kelvin
	real*4 		a37h(NPIX/2,MAX_SCANS)      !array 37H data in kelvin
	
	real*4 		a91v(NPIX,MAX_SCANS)        !array 91V data in kelvin
	real*4 		a91h(NPIX,MAX_SCANS)        !array 91h data in kelvin

cc	integer*2	sfc_img(NPIX,MAX_SCANS)     !sfc tag for imaging
        integer*4	lnd_hi(NPIX,MAX_SCANS)	    !sfc tag for imaging
	real*4 		alat1(NPIX,MAX_SCANS)       !array of latitude in degree
	real*4 		alon1(NPIX,MAX_SCANS)       !array of longitude in degree

cc	integer*2	sfc_env(NPIX/2,MAX_SCANS)   !sfc tag for environmental
	integer*4	lnd(NPIX/2,MAX_SCANS)	    !sfc tag for environmental
	real*4 		alat2(NPIX/2,MAX_SCANS)     !array of latitude in degree
	real*4 		alon2(NPIX/2,MAX_SCANS)     !array of longitude in degree

	real*4 		alat3(NPIX/2,MAX_SCANS)     !array of latitude in degree
	real*4 		alon3(NPIX/2,MAX_SCANS)     !array of longitude in degree
	
	integer*4 	aline			!total # of scan lines in this file
	
	integer*4	first_line, nread	!start line and how many requested # of lines
	
	! ==== both input/output variable to keep track of lines process =====
	integer*4	irec
	
	! ==== helper variables ====
	integer*4 	iline, i, j, k, l, ichan, n
	real*4		TA(NPIX,MAX_SCANS,NCHAN)
	real*4		dlat, dlon
	real*4		xlon1, xlon2, xavg

        !=== Petty mask ===
        real*4          dummy_mask(1440,720)
        integer*1       sfctype(NPIX, MAX_SCANS)
        integer*1       sfckind(NPIX, MAX_SCANS)
cc      logical         do_debug
cc      character*256   cjobid

	!===== output parameters.=============================================
	integer*4	nfound			!actual found lines	
	real*4 		lat(NPIX,MAX_SCANS,LAT_SCENES)
	real*4		lon(NPIX,MAX_SCANS,LON_SCENES)
	real*4		TB(NPIX,MAX_SCANS,NCHAN)
	integer*2 	stdtime(MAX_SCANS, 6), time(6)
	
	!===== parameters used to convert TA to TB.=============================
      	real*4  AP19V,AP19H,AP22V,AP37V,AP37H,AP85V,AP85H,BP19V,BP19H,
     +  	BP22V,BP37V,BP37H,BP85V,BP85H
     
        !===== parameters based on SSMI channels====

	data    AP19V/.969/,AP19H/.969/,AP22V/.974/,AP37V/.986/,
     +          AP37H/.986/,AP85V/.988/,AP85H/.988/,BP19V/.00473/,
     +          BP19H/.00415/,BP22V/.0107/,
     +          BP37V/.0217/,BP37H/.02612/,BP85V/.01383/,BP85H/.01947/

C       !===== parameters based on Yan paper (provided by S. Swadley)
C      	
C	data    AP19V/0.972/,AP19H/.968/,AP22V/.982/,AP37V/.985/,
C     +		AP37H/.981/,AP85V/.982/,AP85H/.978/,BP19V/.00441/,
C     +		BP19H/.00503/,BP22V/.00292/,
C     + 		BP37V/.00415/,BP37H/.00343/,BP85V/.01319/,BP85H/.01876/
C	
	real*4  C19V,C19H,C22V,CC37V,C37H,CC85V,C85H 
	real*4  D19V,D19H,D22V,D37V,D37H,D85V,D85H 
	
       !===== Initialize variables ============================================

        do i=1,NPIX   
         do j=1,MAX_SCANS

          do k=1,NCHAN
           TA(i,j,k)=MISS_FLT
           TB(i,j,k)=MISS_FLT
          enddo

          do l=1,LON_SCENES
           lat(i,j,l)=MISS_FLT
           lon(i,j,l)=MISS_FLT
          enddo

         enddo
        enddo

C      !===== Open LUT files for TA matching ====

CC      COEFF_FILE='ssmis_lut_files/lut_add_ocean'
CC      OPEN(UNIT=20,FILE=COEFF_FILE, STATUS='OLD')
CC      DO i=1,161
CC      READ (20,FMT='(f6.2,7(f7.2))')SSMIS_OCE(I),(ADD_OCE(I,J),J=1,7)
CC      END DO
CC      CLOSE (20)

CC      COEFF_FILE='ssmis_lut_files/lut_add_land'
CC      OPEN(UNIT=20,FILE=COEFF_FILE, STATUS='OLD')
CC      DO i=1,161
CC      READ (20,FMT='(f6.2,7(f7.2))')SSMIS_LND(I),(ADD_LND(I,J),J=1,7)
CC      END DO
CC      CLOSE (20)

        !===== Open sea/land mask (db93-Petty)====

c       call surface_db93_petty(first_line, aline,
cEJN 12/2011 - The resulting sfctype, sfckind are not actually used,
cc             so comment out the subroutine call.
cc      call surface_db93_petty(cjobid, aline,
cc   +             dummy_mask, alat1, alon1, sfctype, sfckind, do_debug)

	!===== define variable used in conversion ====


	C19V = 1/(AP19V*(1-BP19V))
      	C19H = 1/(AP19H*(1-BP19H))
      	C22V = 1/(AP22V*(1-BP22V))
      	CC37V = 1/(AP37V*(1-BP37V))
      	C37H = 1/(AP37H*(1-BP37H))
      	CC85V = 1/(AP85V*(1-BP85V))
      	C85H = 1/(AP85H*(1-BP85H))
      	D19V = C19V*BP19V
      	D19H = C19H*BP19H
      	D22V = C22V*BP22V
      	D37V = CC37V*BP37V
      	D37H = C37H*BP37H
      	D85V = CC85V*BP85V
      	D85H = C85H*BP85H


cejn	iline=first_line
	irec = first_line - 1  ! increments upon entering do loop
        iline = 0

	do n=1, nread
    	  irec=irec+1
	  iline = iline + 1
	  
	  !end of file or not
	  !if(irec.ge.aline) then
cejn	  if (irec. gt. aline) then
	  if (iline .gt. aline) then
	    nfound = iline-first_line
	    goto 101

	  else  ! iline .le. (aline) 

	    do i=1,NPIX   

	      lat(i,iline,1)  = alat1(i,irec)
	      lon(i,iline,1)  = alon1(i,irec)

CEJN 12/22/2011:
              IF (a91h(i,irec) .LT. 50) THEN
                 TA(i,iline,6) = MISS_FLT
                 GO TO 21
              ENDIF
	      TA(i,iline,6) = a91h(i,irec)

   21         CONTINUE
              IF (a91v(i,irec) .LT. 50) THEN
                 TA(i,iline,7) = MISS_FLT
                 GO TO 22
              ENDIF
	      TA(i,iline,7) = a91v(i,irec)
   22         CONTINUE

              ! --- ta-tb conversion --------------!

C EJN, 12/2011: The RSS data came as Tb.  Vila's histogram-matching,
C developed on Ta's, was applied directly to the RSS Tb's here.
C At this step, simply re-assign the resulting values, which are
C now held in the "TA" array, to the "TB" array.
C EJN 12/2011: If the TA value is < 50 K, set TB to MISS_FLT.  The
C above calculations were performed even on missing values, resulting
C in strange results.
C EJN 12/2011: Added additional precaution of setting TA to MISS_FLT
C if value came in < 50 K, but retain this IF test as an extra check.
C
              do j = 6, 7
                 if (TA(i,iline,j) .GT. 50.) then
                    TB(i,iline,j) = TA(i,iline,j)
                 else
                    TB(i,iline,j) = MISS_FLT
                 endif
              end do
C
cc	      TB(i,iline,6) = TA(i,iline,6)
cc	      TB(i,iline,7) = TA(i,iline,7)
CC	      TB(i,iline,6)=C85H*TA(i,iline,6) - D85H*TA(i,iline,7) 
CC	      TB(i,iline,7)=CC85V*TA(i,iline,7) - D85V*TA(i,iline,6)	

	    enddo 
	    
	    do i=1,NPIX,2 

	      lat(i,iline,2)    = alat2((i/2)+1,irec)
	      lon(i,iline,2)    = alon2((i/2)+1,irec)
              lat(i+1,iline,2)  = MISS_FLT
  	      lon(i+1,iline,2)  = MISS_FLT

	      lat(i,iline,3)    = alat3((i/2)+1,irec)
	      lon(i,iline,3)    = alon3((i/2)+1,irec)
	      lat(i+1,iline,3)  = MISS_FLT
	      lon(i+1,iline,3)  = MISS_FLT

CEJN 12/22/2011:
              IF (a19h((i/2)+1,irec) .LT. 50) THEN
                 TA(i,iline,1) = MISS_FLT
                 GO TO 23
              ENDIF
	      TA(i,iline,1)=a19h((i/2)+1,irec)
   23         CONTINUE
              IF (a19v((i/2)+1,irec) .LT. 50) THEN
                 TA(i,iline,2) = MISS_FLT
                 GO TO 24
              ENDIF
	      TA(i,iline,2)=a19v((i/2)+1,irec)

   24         CONTINUE
              IF (a22v((i/2)+1,irec) .LT. 50) THEN
                 TA(i,iline,3) = MISS_FLT
                 GO TO 25
              ENDIF
	      TA(i,iline,3)=a22v((i/2)+1,irec)
   25         CONTINUE
              IF (a37h((i/2)+1,irec) .LT. 50) THEN
                 TA(i,iline,4) = MISS_FLT
                 GO TO 26
              ENDIF
	      TA(i,iline,4)=a37h((i/2)+1,irec)
   26         CONTINUE
              IF (a37v((i/2)+1,irec) .LT. 50) THEN
                 TA(i,iline,5) = MISS_FLT
                 GO TO 27
              ENDIF
	      TA(i,iline,5)=a37v((i/2)+1,irec)
   27         CONTINUE

              ! --- ta-tb conversion --------------!

C EJN 12/2011: The RSS data came as Tb.  They were scaled in
C retrieval before being passed here.  Vila's histogram-matching,
C developed on Ta's, was applied directly to the RSS Tb's here.
C At this step, simply re-assign the resulting values, which are
C now held in the "TA" array, to the "TB" array.
C EJN 12/2011: If the TA value is < 50 K, set TB to MISS_FLT.  The
C above calculations were performed even on missing values, resulting
C in strange results.
C EJN 12/2011: Added additional precaution of setting TA to MISS_FLT
C if value came in < 50 K, but retain this IF test as an extra check.
C
              do j = 1, 5
                 if (TA(i,iline,j) .GT. 50.) then
                    TB(i,iline,j) = TA(i,iline,j)
                 else
                    TB(i,iline,j) = MISS_FLT
                 endif
              end do
c             TB(i,iline,1) = TA(i,iline,1)
c             TB(i,iline,2) = TA(i,iline,2)
c             TB(i,iline,3) = TA(i,iline,3)
c             TB(i,iline,4) = TA(i,iline,4)
c             TB(i,iline,5) = TA(i,iline,5)
CC	      TB(i,iline,1) = C19H*TA(i,iline,1) - D19H*TA(i,iline,2)
CC	      TB(i,iline,2) = C19V*TA(i,iline,2) - D19V*TA(i,iline,1)
CC	      TB(i,iline,3) = C22V*TA(i,iline,3) -
CC   +			      D22V*(.653*TA(i,iline,1)+96.6)
CC	      TB(i,iline,4) = C37H*TA(i,iline,4) - D37H*TA(i,iline,5)
CC	      TB(i,iline,5) = CC37V*TA(i,iline,5) - D37V*TA(i,iline,4)

            enddo

	    do i=1,NPIX,2

 	      do ichan = 1, 5

	        if (i.eq.NPIX-1) then
	           TA(i+1,iline,ichan)=TA(i,iline,ichan)
	           TB(i+1,iline,ichan)=TB(i,iline,ichan)
		else
C
C EJN 12/2011 - Average only if both values are > 50K:
C
                   if (TA(i,iline,ichan) .GT. 50. .AND.
     +                 TA(i+2,iline,ichan) .GT. 50.) then
	               TA(i+1,iline,ichan)=(TA(i,iline,ichan)+
     +                                      TA(i+2,iline,ichan))/2.
                   else
                       TA(i+1,iline,ichan) = MISS_FLT
                   endif
C
                   if (TB(i,iline,ichan) .GT. 50. .AND.
     +                 TB(i+2,iline,ichan) .GT. 50.) then
                       TB(i+1,iline,ichan)=(TB(i,iline,ichan)+
     +                                      TB(i+2,iline,ichan))/2.
                   else
                       TB(i+1,iline,ichan) = MISS_FLT
                   endif
		endif

	      end do

	      do ichan = 2, 3

	        if (i.eq.NPIX-1) then
		   dlat=lat(i,iline,ichan)-lat(i-1,iline,ichan)
C
C EJN - Account for longitudes straddling the dateline.
C
                   xlon1 = lon(  i,iline,ichan)
                   xlon2 = lon(i-1,iline,ichan)
                   if (xlon1 .LT. -179. .AND. xlon2 .GT. 179.) then
                      xlon2 = xlon2 - 360.
                   else if ( xlon2 .LT.-179. .AND. xlon1 .GT. 179.) then
                      xlon1 = xlon1 - 360.
                   endif
                   dlon = xlon1 - xlon2
CEJN		   dlon=lon(i,iline,ichan)-lon(i-1,iline,ichan)
	           lat(i+1,iline,ichan)=lat(i,iline,ichan)+dlat
	           lon(i+1,iline,ichan)=lon(i,iline,ichan)+dlon
		else
	           lat(i+1,iline,ichan)=(lat(i,iline,ichan)+
     +                                  lat(i+2,iline,ichan))/2.
C
C EJN - Account for longitudes straddling the dateline.
C
                   xlon1 = lon(  i,iline,ichan)
                   xlon2 = lon(i+2,iline,ichan)
                   if (xlon1 .LT. -179. .AND. xlon2 .GT. 179.) then
                      xlon2 = xlon2 - 360. 
                      xavg = (xlon1 + xlon2) / 2.
                      if (xavg .LT. -180.) xavg = xavg + 360.
                      lon(i+1,iline,ichan) = xavg
                   else if ( xlon2 .LT.-179. .AND. xlon1 .GT. 179.) then
                      xlon1 = xlon1 - 360.
                      xavg = (xlon1 + xlon2) / 2.
                      if (xavg .LT. -180.) xavg = xavg + 360.
                      lon(i+1,iline,ichan) = xavg
                   else
	              lon(i+1,iline,ichan)=(lon(i,iline,ichan)+
     +                                     lon(i+2,iline,ichan))/2.
                   endif
		endif
C       
C	        TA(i+1,iline,ichan) = MISS_FLT
C	        TB(i+1,iline,ichan) = MISS_FLT
C		
 	      enddo

 	    enddo

cejn	    iline=iline+1
	  
	  endif 
	   	  
	enddo		! end of loop on n
	
	nfound = nread



 101	return

	end
