      subroutine retrieval_SSMIS(input_file, GPROFPath)


c     Modified version of GPROF V7
c     Obtained from E. Nelkin 

C This is SSMIS GPROF2004V.
C  SSMIS data is ingested and the 91GHz channel is matched to the 85 GHz
C  channel.  After this conversion, the input is treated as though it
C  came from an SSMI.

      implicit none          

      include 'parameters.inc'

      character*256 input_file, GPROFPath, oname
      LOGICAL         USESCN(MAX_SCANS)
      CHARACTER*24 BTIME(MAX_SCANS), CTIME
      real*8       frcrev(MAX_SCANS)
      REAL*4       XLAT_LO(NPIX/2,MAX_SCANS), XLON_LO(NPIX/2,MAX_SCANS),
     +     XLAT_HI(NPIX,MAX_SCANS), XLON_HI(NPIX, MAX_SCANS)
      REAL*4       TB_LO(5,NPIX/2,MAX_SCANS), TB_HI(2,NPIX,MAX_SCANS)
      INTEGER*4    iscn_flag(MAX_SCANS), ical_flag(MAX_SCANS),
     +     ical_flag_hi(MAX_SCANS)
      INTEGER*4    lnd(NPIX/2,MAX_SCANS), lnd_hi(NPIX,MAX_SCANS)
      INTEGER*4    alnd(NPIX/2,MAX_SCANS), alnd_hi(NPIX,MAX_SCANS)
      INTEGER*4 IYEAR, IMMDD, IYMD, IHMS
      INTEGER*4 ITIME87, ITIME87_ARR(MAX_SCANS)
      integer aline, iver


      ! Work arrays reserved for satellite input data
      real        Tb (NPIX, MAX_SCANS, NCHAN)  ! K
      real        lat(NPIX, MAX_SCANS)         ! [-90, +90]
      real        lon(NPIX, MAX_SCANS)         ! [-180, +180]
      integer     wentz_time(MAX_SCANS)        ! Wentz times (sec)
      integer*2   wentz_sfctype(NPIX, MAX_SCANS)

      ! For Petty background
      real        T_bkgrd_clear(NPIX, MAX_SCANS, NCHAN) ! K

      ! Arrays used for internal (std) surface types and times;
      ! Tc is used for modified or corrected brightness temperatures.
      real        Tc(NPIX, MAX_SCANS, NCHAN) ! K
      integer*1   sfctype(NPIX, MAX_SCANS)
      integer*1   sfckind(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)      ! YYYY MM DD HH MM SS
      integer*2   stdtime_all(MAX_SCANS, 6)  ! YYYY MM DD HH MM SS

      integer*4   istimeU(6), ietimeU(6)

      ! Internal variables
      real        T_bkgrd(NPIX, MAX_SCANS, NCHAN) ! K
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)
      character   sat_name*3   ! currently 'F16', 'F17', or 'F18'


      ! Retrieved variables
      real        frzheight(NPIX, MAX_SCANS)    ! km
      real        sfcrain(NPIX, MAX_SCANS)  ! mm/hr
      real        convect(NPIX, MAX_SCANS)  ! mm/hr
      real        quality(NPIX, MAX_SCANS)
      real        clw (NPIX, MAX_SCANS, NLYR)  ! kg/m^2
      real        rain(NPIX, MAX_SCANS, NLYR)  ! kg/m^2
      real        ciw (NPIX, MAX_SCANS, NLYR)
      real        ice (NPIX, MAX_SCANS, NLYR)
      real        Q1(NPIX, MAX_SCANS, 0:NLYR)

      ! Area averaged output variables
      integer     start_time(6)
      real        sumH_sfcrain(360,720)
      integer     sumH_ncount(360,720)
      integer     sumH_nambig(360,720)
      integer     sumH_nrpix(360,720) 
      integer*1   gridH_time(360,720)
      integer*2   gridH_sfcrain(360,720)
      integer*1   gridH_ncount(360,720)
      integer*1   gridH_nambig(360,720)
      integer*1   gridH_nrpix(360,720)

      ! Housekeeping variables
      integer      nwork, i, j, k, kk, ipos, jpos
      integer      ii, strlen, ios, inunit
      logical      more, opn
      integer      lin,pix,chan
      integer      irec
      integer      nwrite
      logical      pad
      integer      start_line, nread, nfound, nrec, npad
      integer      first_line, last_line, istrt, istop
      real         conv_obs(NPIX, MAX_SCANS)
      real         rain_obs(NPIX, MAX_SCANS)
      real         emisIndex(NPIX, MAX_SCANS, NEMIS)
      real         scatIndex(NPIX, MAX_SCANS, NSCAT)
      real         chanWgt(NPIX, MAX_SCANS, 7)
      character*8 cstdate, cendate
      character*6 csttime, centime
      logical      structure

      ! Database variables
      integer      iostat, ierror
      integer*1    sfmask1(1440, 720)
      integer*1    sfmask2(1440, 720)
      integer*2    sstdata(0:143,0:90,12)
      real         flttw_tab(131, 131)
      integer*1    elev(4320,2160)

      ! Rainfall profile database variables
      integer      n, ilat, ilon

      integer      dbw_nprofiles(DB_WATER)
      integer      dbw_sfc_code(DB_WATER)
      real         dbw_lyrhgt(DB_WATER,NLYR)
      real         dbw_occur(DB_WATER,DBW_MXPROFILE)
      real         dbw_frzlevel(DB_WATER, DBW_MXPROFILE)
      real         dbw_sfcrain(DB_WATER,DBW_MXPROFILE)
      real         dbw_convect(DB_WATER,DBW_MXPROFILE) 
      real         dbw_faconvmax(DB_WATER,DBW_MXPROFILE,NRING) 
      real         dbw_faconvmean(DB_WATER,DBW_MXPROFILE,NRING) 
      real         dbw_rain_cover(DB_WATER,DBW_MXPROFILE) 
      real         dbw_conv_cover(DB_WATER,DBW_MXPROFILE)
      real         dbw_clw(DB_WATER,DBW_MXPROFILE,NLYR)
      real         dbw_rain(DB_WATER,DBW_MXPROFILE,NLYR)
      real         dbw_ciw(DB_WATER,DBW_MXPROFILE,NLYR) 
      real         dbw_ice(DB_WATER,DBW_MXPROFILE,NLYR)
      real         dbw_Q1(DB_WATER,DBW_MXPROFILE,0:NLYR)
      real         dbw_Tb(DB_WATER,DBW_MXPROFILE,DB_NCHAN)
      real         dbw_Tbclr(DB_WATER,DBW_MXPROFILE,DB_NCHAN)
      real         dbw_Tb_cover(DB_WATER,DBW_MXPROFILE,DB_NCHAN)
      real         dbw_emisIndex(DB_WATER,DBW_MXPROFILE,NEMIS)
      real         dbw_scatIndex(DB_WATER,DBW_MXPROFILE,NSCAT)

      integer      dbl_nprofiles(DB_LAND)
      integer      dbl_sfc_code(DB_LAND)
      real         dbl_lyrhgt(DB_LAND,NLYR)
      integer      dbl_cstype(DB_LAND,DBL_MXPROFILE)
      real         dbl_occur(DB_LAND,DBL_MXPROFILE)
      real         dbl_sfcrain(DB_LAND,DBL_MXPROFILE)
      real         dbl_convect(DB_LAND,DBL_MXPROFILE)
      real         dbl_rain_cover(DB_LAND,DBL_MXPROFILE)
      real         dbl_conv_cover(DB_LAND,DBL_MXPROFILE)
      real         dbl_clw(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbl_rain(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbl_ciw(DB_LAND,DBL_MXPROFILE,NLYR) 
      real         dbl_ice(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbl_Q1(DB_LAND,DBL_MXPROFILE,0:NLYR)
      real         dbl_Tb(DB_LAND,DBL_MXPROFILE,DB_NCHAN)
      real         dbl_Tb_cover(DB_LAND,DBL_MXPROFILE,DB_NCHAN)

      integer      dbc_nprofiles(DB_LAND)
      integer      dbc_sfc_code(DB_LAND)
      real         dbc_lyrhgt(DB_LAND,NLYR)
      integer      dbc_cstype(DB_LAND,DBL_MXPROFILE)
      real         dbc_occur(DB_LAND,DBL_MXPROFILE)
      real         dbc_sfcrain(DB_LAND,DBL_MXPROFILE)
      real         dbc_convect(DB_LAND,DBL_MXPROFILE)
      real         dbc_rain_cover(DB_LAND,DBL_MXPROFILE)
      real         dbc_conv_cover(DB_LAND,DBL_MXPROFILE)
      real         dbc_clw(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbc_rain(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbc_ciw(DB_LAND,DBL_MXPROFILE,NLYR) 
      real         dbc_ice(DB_LAND,DBL_MXPROFILE,NLYR)
      real         dbc_Q1(DB_LAND,DBL_MXPROFILE,0:NLYR)
      real         dbc_Tb(DB_LAND,DBL_MXPROFILE,DB_NCHAN)
      real         dbc_Tb_cover(DB_LAND,DBL_MXPROFILE,DB_NCHAN)




      real*4          a19v(NPIX/2,MAX_SCANS)      !A-scan 19V data in Kelvin       
      real*4          a19h(NPIX/2,MAX_SCANS)      !A-scan 19H data in Kelvin       
      real*4          a22v(NPIX/2,MAX_SCANS)      !A-scan 22V data in Kelvin       
      real*4          a37v(NPIX/2,MAX_SCANS)      !A-scan 37V data in Kelvin       
      real*4          a37h(NPIX/2,MAX_SCANS)      !A-scan 37H data in Kelvin       
      real*4          a91v(NPIX,MAX_SCANS)      !A-scan and B-scan 91V data in Kelvin
      real*4          a91h(NPIX,MAX_SCANS)      !A-scan and B-scan 91h data in Kelvin
      real*4          alat1(NPIX,MAX_SCANS)     !latitude in degrees of 91         
      real*4          alon1(NPIX,MAX_SCANS)     !longitude in degrees of 91        
      real*4          alat2(NPIX/2,MAX_SCANS)     !latitude in degrees of 37       
      real*4          alon2(NPIX/2,MAX_SCANS)     !longitude in degrees of 37      
      real*4          alat3(NPIX/2,MAX_SCANS)     !latitude in degrees of 19, 22   
      real*4          alon3(NPIX/2,MAX_SCANS)     !longitude in degrees of 19, 22  
      real*4          orig_lat(NPIX,MAX_SCANS,3) !latitude in degrees of all channels
      real*4          orig_lon(NPIX,MAX_SCANS,3) !longitude in degrees of all channels




c       Initialize returned variable to be undefined
c      do lin=1,MAX_SCANS
c        hdf_time(1,lin)=0
c        hdf_time(2,lin)=0
c        hdf_time(3,lin)=0
c        hdf_time(4,lin)=0
c        hdf_time(5,lin)=0
c        hdf_time(6,lin)=0
c        do pix=1, NPIX
c          hdf_lon(pix,lin)=0.0
c          hdf_lat(pix,lin)=0.0
c          hdf_rain(pix,lin)=0.0      
c        enddo     
c      enddo      

C ---------- Initialization ----------


      ! Set variable "structure" to .TRUE. if structure is needed               
      !structure = .FALSE.                                                      
      structure = .TRUE.


      ! Initialize area averaged output variables
      do i = 1, 6
         start_time(i) = 0
      end do
      do ilat = 1, 360
         do ilon = 1, 720
            sumH_sfcrain(ilat,ilon) = 0.
            sumH_ncount(ilat,ilon)  = 0
            sumH_nambig(ilat,ilon)  = 0
            sumH_nrpix(ilat,ilon)   = 0
         end do
      end do


C  Initialize local lat/lon/Tb/surface type arrays
      do lin = 1, MAX_SCANS
            do pix = 1, NPIX
                  alat1(pix,lin) = -999.0
                  alon1(pix,lin) = -999.0
                  XLAT_HI(pix,lin) = -999.0
                  XLON_HI(pix,lin) = -999.0
                  a91v(pix,lin) = -999.0
                  a91h(pix,lin) = -999.0
                  do chan = 1, 3
                     orig_lat(pix,lin,chan) = -999.0
                     orig_lon(pix,lin,chan) = -999.0
                  end do
                  do chan = 1, 2
                     TB_HI(chan,pix,lin) = -999.0
                  end do
            end do
            do pix = 1, NPIX/2
               alat2(pix,lin) = -999.0
               alon2(pix,lin) = -999.0
               alat3(pix,lin) = -999.0
               alon3(pix,lin) = -999.0
               XLAT_LO(pix,lin) = -999.0
               XLON_LO(pix,lin) = -999.0
               a19v(pix,lin) = -999.0
               a19h(pix,lin) = -999.0
               a22v(pix,lin) = -999.0
               a37v(pix,lin) = -999.0
               a37h(pix,lin) = -999.0
               do chan = 1, 5
                  TB_LO(chan,pix,lin) = -999.0
               end do
            end do
      end do



C  Read databases into memory the first time this subroutine is called.
 
         call read_dbase(sfmask1, sfmask2, sstdata, 
     +     flttw_tab, dbw_nprofiles, dbw_sfc_code, dbw_lyrhgt,  
     +     dbw_occur, dbw_frzlevel, dbw_sfcrain, dbw_convect, 
     +     dbw_rain_cover, dbw_conv_cover, dbw_faconvmax,  
     +     dbw_faconvmean,  
     +     dbw_clw, dbw_rain, dbw_ciw, dbw_ice, dbw_Q1, dbw_Tb, 
     +     dbw_Tbclr, dbw_Tb_cover, elev, dbl_nprofiles, 
     +     dbl_sfc_code, dbl_lyrhgt, dbl_cstype, dbl_occur, 
     +     dbl_sfcrain, dbl_convect, dbl_rain_cover, dbl_conv_cover, 
     +     dbl_clw, dbl_rain, dbl_ciw, dbl_ice, dbl_Q1, 
     +     dbl_Tb, dbl_Tb_cover, dbc_nprofiles, 
     +     dbc_sfc_code, dbc_lyrhgt, dbc_cstype, dbc_occur, 
     +     dbc_sfcrain, dbc_convect, dbc_rain_cover, dbc_conv_cover, 
     +     dbc_clw, dbc_rain, dbc_ciw, dbc_ice, dbc_Q1, 
     +     dbc_Tb, dbc_Tb_cover)





c-------------------------------------------------------
c     Read number of scans from NetCDF4 file containing RSS Tb 
      call ReadNC_RSS_SSMIS_Tb_v6_nscans(input_file, aline)

c     Only proceed if there is some data
      if (aline .LT. 1) return

c     Read NetCDF4 file containing RSS Tb 
      iver=1 ! this option forces read code to correct V7 Tb to V6 Tb
      call ReadNC_RSS_SSMIS_Tb_v6(input_file, iver, aline,
     &     sat_name, stdtime_all,btime,frcrev, itime87_ARR, start_time, 
     &     TB_LO, TB_HI, XLAT_LO, XLON_LO, XLAT_HI,
     &     XLON_HI, iscn_flag, ical_flag, ical_flag_hi, lnd, lnd_hi)

 

c      print *, "fname = ", input_file
c      print *, "nscan = ", aline
c      k=100
c      j=1
c      print *, "nscan = ", k
c      print *, "iflags = ", iscn_flag(k), ical_flag(k), ical_flag_hi(k)
c      print *, "lnd mask = ", lnd(j,k), lnd_hi(j,k)
c      print *, "pos = ", j
c      print *, "xlat_lo, xlon_lo = ", xlat_lo(j,k), xlon_lo(j,k)
c      print *, "tb_lo = ",tb_lo(1,j,k),tb_lo(2,j,k),tb_lo(3,j,k),
c     &     tb_lo(4,j,k),tb_lo(5,j,k)
c      print *, "xlat_hi, xlon_hi = ", xlat_hi(j,k), xlon_hi(j,k)
c      print *, "tb_hi = ",tb_hi(1,j,k),tb_hi(2,j,k)
c      print *, "------------"






C EJN: First check the Wentz quality flags to determine which scans are
C      valid.

      DO K = 1, aline
         USESCN(K) = .TRUE.
         IF ( iscn_flag(K) .NE. 0 .OR. ical_flag(K) .NE. 0 .OR.
     +        ical_flag_hi(K) .NE. 0 ) USESCN(K) = .FALSE.
      END DO








C
C ---------- Main Retrieval Loop ----------
      if (do_debug) then
       write(*,*) 'Starting main retrieval loop...'
      endif
C
C  npad refers to the number of pad lines to expect at the beginning
C  and end of an orbit, whereas NBUFF refers to the number of buffer
C  lines for the interior-orbit segments of NCORE lines each.
C
c      if ( pad ) then
c         npad = 4
c      else
         npad = 0
c      endif

      !Fill all global variables with default values

      call init_arrays(pixel_status, rain_status, frzheight,
     +       sfcrain, convect, quality, structure, 
     +       clw, rain, ciw, ice, Q1)



c     MRPS:
c     The original Nelkin code used time to remove the padded scans
c     and overlaps. The RSS CDR does not have overlaps, although it
c     does have padded scans which are removed here
c     Note: Nelkin code does not seem to remove padded scans, so
c           comparisons of files obtained with his code should
c           have the extra padded scans... something to be mindful of

      do i=1,aline
         if (frcrev(i) .LT. 0 .OR. frcrev(i) .GT. 1) then
            usescn=.FALSE.
         end if
      end do
      istrt=0
      do i=1,aline
         if (frcrev(i) .GE. 0) then
            istrt=i
            exit
         end if
      end do
      istop=istrt
      do i=1,aline
         if (frcrev(i) .GT. 1) then
            exit
         end if
         istop=i
      end do


C EJN: If no unique data were found, return to the main program to
C      advance to the next input file in the list.

       IF ( istrt .EQ. 0 .OR. istop .EQ. 0 ) RETURN



C EJN, 12/7/2011: Instruct histo_match (patterned after Vila's
C      ingest-ssmis-tdrr) to process the scan range [istrt, istop].
C
      start_line = istrt
      nread = istop - istrt + 1
C
C EJN 9/22/2011:
C It is CRITICAL that I line up Tb, lat, lon and stdtime arrays so that
C their line 1 is actually "start_line" as defined by select_unique.
C 12/7/2011: Same applies to lnd, lnd_hi.
C
C *** Note that READ_L1B_FILE returns longitudes in range [0,360], so
C values above 180 are adjusted to achieve [-180,+180], as needed by
C this code.
C
C For the brightness temperatures, the RSS channel order 19V,19H,22V,
C 37V,37H,91V,91H must be re-arranged to 19H,19V,22V,37H,37V,91H,91V.
C
      ii = 0
      DO J = istrt, istop
         ii = ii + 1
C
         DO K = 1, 6
            stdtime(ii,K) = stdtime_all(J,K)
         END DO
C
         DO I = 1, NPIX
C lat, 91 GHz:
            alat1(I,ii) = XLAT_HI(I,J)
C lon, 91 GHz:
            alon1(I,ii) = XLON_HI(I,J)
            IF ( alon1(I,ii) .GE. 180.)
     +         alon1(I,ii) = alon1(I,ii) - 360.
C lnd_hi:
            alnd_hi(I,ii) = lnd_hi(I,J)
C 91H:
            a91h(I,ii) = TB_HI(2,I,J)
C 91V:
            a91v(I,ii) = TB_HI(1,I,J)
         END DO
cejn debug
c     write(1214,*) J, ii, alat1(90,ii)
C
         DO I = 1, NPIX/2
C lat, 37 GHz:
            alat2(I,ii) = XLAT_LO(I,J)
C
C lat, 19/22 GHz, same as 37 GHz in RSS data:
            alat3(I,ii) = alat2(I,ii)
C
C lon, 37 GHz:
            alon2(I,ii) = XLON_LO(I,J)
            IF ( alon2(I,ii) .GE. 180.)
     +         alon2(I,ii) = alon2(I,ii) - 360.
C
C lon, 19/22 GHz, same as 37 GHz in RSS data:
            alon3(I, ii) = alon2(I,ii)
C lnd:
            alnd(I,ii) = lnd(I,J)
C 19H:
            a19h(I,ii) = TB_LO(2,I,J)
C 19V:
            a19v(I,ii) = TB_LO(1,I,J)
C 22V:
            a22v(I,ii) = TB_LO(3,I,J)
C 37H:
            a37h(I,ii) = TB_LO(5,I,J)
C 37V:
            a37v(I,ii) = TB_LO(4,I,J)
         END DO
      END DO

C
C EJN, 12/14/2011 - Now that all the arrays being passed to HISTO_MATCH
C have been re-indexed to start at line 1, re-define start_line = 1,
C since start_line tells HISTO_MATCH the scan line to begin processing
C these arrays.
C

      start_line = 1
      CALL HISTO_MATCH(start_line, nread, nfound, Tb, orig_lat,
     +          orig_lon, a19v, a19h, a22v, a37v, a37h, a91v, a91h,
     +          alat1, alon1, alat2, alon2, alat3, alon3, alnd, alnd_hi,
     +          aline)




C EJN: For RSS files, force nfound = nread, to disturb the CLASS TDRR
C      code as little as possible.                            
C
      nfound = nread
C



      !==============================================================
      ! Call routine to interpolate SSMIS channels to common grid
      !==============================================================
      call geolocate_ssmis(Tb, orig_lat, orig_lon, aline, lat, lon)



      if (nfound .LE. nread) then
         nwork = min(nfound, MAX_SCANS)
         nwrite = nwork-npad

        ! Fill empty lines with missing values. 
         first_line = nwrite+1
         last_line = MAX_SCANS
               
C
C  The following two solutions give the same result.  As a group,
C  we should ultimately determine which of the two we prefer.
C
C  Solution 1: Do not call clear_scans if there is a pad.
C
C                 if ( .NOT. pad ) call clear_scans(nfound+1, nread, Tb,
C    +                   lat, lon, wentz_time, wentz_sfctype, stdtime,
C    +                   sfctype, sfckind)
C
C  Solution 2: Modify the argument for the first line to be cleared,
C              to account for a possible pad.
C
CEJN, 12/2011: Clear out many more arrays than before.
C
C        call clear_scans(nfound+npad+1, nread, Tb, lat, lon,
C    +                    wentz_time, wentz_sfctype, stdtime,
C    +                    sfctype, sfckind, alat1, alon1, alat2,
C    +                    alon2, alat3, alon3, orig_lat, orig_lon,
C    +                    XLAT_LO, XLAT_HI, XLON_LO, XLON_HI,
C    +                    a19v, a19h, a22v, a37v, a37h, a91v, a91h,
C    +                    TB_LO, TB_HI, lnd, lnd_hi)
         call clear_scans(nfound+npad+1, nread, Tb, lat, lon,
     +                    wentz_time, wentz_sfctype, stdtime,
     +                    sfctype, sfckind)
      else
         nwork  = MAX_SCANS
         nwrite = MAX_SCANS
      end if
        


      ! Deconvolve temperatures to fill high resolution
      ! array.  In this code, low-res temperatures are merely
      ! copied.
      call Tb2Tc_intplte(nwork, Tb, Tc)

      ! Calculate emission and scattering indices for the ocean
      ! database profiles.
      call dbw_emis_scat_Index(dbw_nprofiles, dbw_Tb, dbw_Tbclr, 
     +  dbw_emisIndex, dbw_scatIndex)  

      call surface_db93_petty(nfound,
     +             sfmask2, lat, lon, sfctype, sfckind)

      ! Perform generic quality control
      call qualctrl(nwork, Tb, Tc, lat, lon, stdtime,
     +        sat_name, sfctype, pixel_status)

      ! Compute auxiliary parameters.
      call frzhgt_ttw(nwork, Tc, flttw_tab, sstdata,
     +       pixel_status, lat, lon, stdtime, sfctype,
     +       frzheight)

      ! Rainfall screening routines.  Each surface type is
      ! handled independently.  Other surface types are
      ! not considered.
      call screen_land(nwork, Tc, sfctype, sfckind, 
     +      stdtime, pixel_status, rain_status, lat, lon, elev)
      call screen_ocean(nwork, Tc, sfctype, sfckind, frzheight,
     +      pixel_status, rain_status)
      call screen_coast(nwork, Tc, sfctype, 
     +      pixel_status, rain_status)
      call screen_ice(nwork, Tc, sfctype, sfckind, 
     +      pixel_status, rain_status)
        
      ! Background Tb - rainfall routines sometimes need
      ! to know the background Tb (where it is not raining).
      call bkgrd_fill(nwork, Tc, sfctype, pixel_status,
     +      rain_status, T_bkgrd)

      ! Compute Petty Clear-air radiances
      call bkgrd_clear_ocean (nwork, lon, lat, stdtime,
     +    pixel_status, sfctype, 1, sstdata,
     +    Tc, T_bkgrd_clear)
      call T_bkgrd_ocean_ctrl(nwork, pixel_status, 
     +            sfctype, T_bkgrd_clear, lat, lon)



      ! Compute emission and scattering indeces over ocean
      call emis_scat_index(nwork, Tc, T_bkgrd, 
     +  1, T_bkgrd_clear, pixel_status, sfctype,
     +  emisIndex, scatIndex)

      ! Compute convective fraction over ocean
      call comp_frac(nwork, Tc, T_bkgrd, pixel_status,  
     +  sfctype, conv_obs, rain_obs, emisIndex, scatIndex)

      ! Rainfall retrieval.
      call rain_ocean(nwork, sfctype, sfckind, frzheight, 
     +      pixel_status, rain_status, dbw_nprofiles, 
     +      dbw_sfc_code, dbw_occur, dbw_frzlevel, dbw_rain_cover,
     +      dbw_sfcrain, dbw_convect, dbw_conv_cover, dbw_faconvmax,
     +      dbw_faconvmean, dbw_clw, dbw_rain, dbw_ciw,
     +      dbw_ice, dbw_Q1, dbw_Tb_cover, dbw_emisIndex,  
     +      dbw_scatIndex, conv_obs, rain_obs, emisIndex, scatIndex,  
     +      sfcrain, convect, quality, structure, clw, rain, ciw, ice, 
     +      Q1) 

      call rain_land(nwork, Tc, sfctype, pixel_status,
     +      rain_status, dbl_nprofiles, dbl_sfc_code,
     +      dbl_occur, dbl_sfcrain, dbl_convect,
     +      dbl_clw, dbl_rain, dbl_ciw, dbl_ice, dbl_Q1,
     +      dbl_Tb, sfcrain, convect, quality, structure,
     +      clw, rain, ciw, ice, Q1, dbl_cstype)

      call rain_coast(nwork, Tc, sfctype, sfckind, frzheight, 
     +      pixel_status, rain_status, dbc_nprofiles, 
     +      dbc_sfc_code, dbc_cstype, dbc_occur, 
     +      dbc_sfcrain, dbc_convect, dbc_clw, dbc_rain, dbc_ciw,  
     +      dbc_ice, dbc_Q1, dbc_Tb, dbc_Tb_cover, sfcrain, 
     +      convect, quality, structure, clw, rain, ciw, ice, Q1)
           
c      call rain_ctrl(nwork, pixel_status, rain_status,
c     +            sfctype, sfcrain, convect)
c      call rain_ctrl(nwork, pixel_status, rain_status,
c+          sfctype, sfcrain, convect, quality, structure,
c+          clw, rain, ciw, ice, Q1)







      !==============================================================
      ! assign to output varaibles  
      !==============================================================
c       do lin=1, nwork
c          do pix=1,6
c             hdf_time(pix,lin)=stdtime(lin, pix)
c          enddo
c          do pix=1, NPIX
            !=========================================================
            ! assign to output varaibles 
            !=========================================================
c            hdf_lon(pix, lin ) = lon(pix, lin) 
c            hdf_lat(pix, lin ) = lat(pix, lin) 
c            hdf_rain(pix,lin ) = sfcrain(pix, lin) 
c         enddo
c      enddo







      !==============================================================
      ! Write retrieved parameters to output file.
      ! Only write lines 1 through nwork.  Ordinarily this will be
      ! the contents of the entire core area (i.e. nwork = NCORE),
      ! but on the last iteration of the main retrieval loop (i.e.
      ! nwork < NCORE) this will only be the portion of the core area
      ! that contains data.




      ipos = 1
      more = .FALSE.
      call output_halfdeg(nwrite, ipos, more, lat, lon,
     &        stdtime, pixel_status, rain_status, sfcrain,
     &        start_time, sumH_sfcrain, sumH_ncount, sumH_nambig,
     &        sumH_nrpix, gridH_time)




c      get averages
         do ilat = 1, 360
           do ilon = 1, 720
               if ( sumH_ncount(ilat,ilon) .gt. 0 ) then
                 gridH_sfcrain(ilat,ilon) = nint(100.*
     &                sumH_sfcrain(ilat,ilon)/ sumH_ncount(ilat,ilon))
                 gridH_ncount(ilat,ilon) = sumH_ncount(ilat,ilon)
                 gridH_nambig(ilat,ilon) = sumH_nambig(ilat,ilon)
                 gridH_nrpix(ilat,ilon) = sumH_nrpix(ilat,ilon)
               else
                 gridH_sfcrain(ilat,ilon) = -9999
                 gridH_ncount(ilat,ilon) = 0
                 gridH_nambig(ilat,ilon) = 0
                 gridH_nrpix(ilat,ilon) = 0
                 gridH_time(ilat,ilon) = 0
               endif
             end do
           end do


      
c     construct output filename
      do strlen = 1,len_trim(GPROFPath)
         if (GPROFPath(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      
      IF ( more ) THEN
         jpos = ipos - 1
      ELSE
         jpos = ipos
      ENDIF

      write(cstdate, '(I4.4I2.2I2.2)' ) start_time(1:3)
      write(csttime, '(I2.2I2.2I2.2)' ) start_time(4:6)
      write(cendate, '(I4.4I2.2I2.2)' ) stdtime(jpos,1:3)
      write(centime, '(I2.2I2.2I2.2)' ) stdtime(jpos,4:6)
      
      oname=GPROFPath(1:strlen)//
     &     "/GPROF2004V."//sat_name//"."//cstdate//"_"//
     &     csttime//"_"//cendate//"_"//centime

      do strlen = 1,len_trim(oname)
         if (oname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

C     check for a free iounit
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      
c     open output file
      open(inunit, FILE = oname(1:strlen), STATUS = 'replace',
     &     FORM='UNFORMATTED', iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to open output file'
         STOP
      end if
      
c     write data
      write(inunit, iostat=ios) start_time
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_time
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_sfcrain
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_ncount
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_nambig
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to write output file'
         STOP
      end if
      write(inunit, iostat=ios) gridH_nrpix
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to write output file'
         STOP
      end if
      
      
c     close output file
      close(inunit,iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_d2_GPROFv2004_SSMIS:'//
     &        'Failed to close output file'
         STOP
      end if
      
      












C
C EJN 5/20/2011
C If there are more scan lines still to process, re-initialize the
C sum arrays and call output_quarterdeg again, this time starting
C on scan line "ipos".  Then loop back up to line 50 to set the
C start and end date/times and write the next gridded file.
C Continue iterating in this fashion until all scan lines have been
C output.  (Most likely, there will never be more than 2 output
C files of .LE. 108-minute duration per input file.)
C
c         IF ( more ) THEN
c            do i = 1, 6
c               start_time(i) = 0
c            end do
c
c               do ilat = 1, 360
c                  do ilon = 1, 720
c                     sumH_sfcrain(ilat,ilon) = 0.
c                     sumH_ncount(ilat,ilon)  = 0
c                     sumH_nambig(ilat,ilon)  = 0
c                     sumH_nrpix(ilat,ilon)   = 0
c                  end do
c               end do
c               call output_halfdeg(nwrite, ipos, more, lat, lon,
c     &              stdtime, pixel_status, rain_status, sfcrain,
c     &              start_time, sumH_sfcrain, sumH_ncount, sumH_nambig,
c     &              sumH_nrpix, gridH_time)
c
c
c            GO TO 50
c         ENDIF
c      endif





      return

      end







C------------------------------------------------------------------
C                     Housekeeping subroutines
C------------------------------------------------------------------


C ---------- Subroutine clear_scans ----------------

      ! Fill lines first_line to last_line (inclusive) of arrays with
      ! missing values.
      subroutine clear_scans(first_line, last_line, Tb, lat, lon,
     +      wentz_time, wentz_sfctype, stdtime, sfctype, sfckind)
      implicit none
      include 'parameters.inc'

      integer     first_line, last_line
      real        Tb(NPIX, MAX_SCANS, NCHAN)
      real        lat(NPIX, MAX_SCANS)
      real        lon(NPIX, MAX_SCANS)
      integer     wentz_time(MAX_SCANS)
      integer*2   wentz_sfctype(NPIX, MAX_SCANS)
      integer*2   stdtime(MAX_SCANS, 6)   ! YYYY MM DD HH MM SS
      integer*1   sfctype(NPIX, MAX_SCANS)
      integer*1   sfckind(NPIX, MAX_SCANS)


      integer     chan, lin, pix
      integer     i

      do lin = first_line, last_line
            do pix = 1, NPIX
                  do chan = 1, NCHAN
                        Tb(pix, lin, chan) = -999.0
                  end do

                  lat(pix, lin) = -999.0
                  lon(pix, lin) = -999.0

                  wentz_sfctype(pix, lin) = -1

                  sfctype(pix, lin) = -1
                  sfckind(pix, lin) = -1

            end do

            do i = 1, 6
               stdtime(lin,i) = -999
            end do

            wentz_time(lin) = -999

      end do

      return
      end

