      ! Screening subroutine for coast surface (sfctype = 2)

      subroutine screen_coast(nwork, Tc, sfctype, 
     &	 	  pixel_status, rain_status)

      implicit none

      include 'parameters.inc'

      integer     nwork
      real        Tc(NPIX, MAX_SCANS, NCHAN)
      integer*1   sfctype(NPIX, MAX_SCANS)
c      integer*1   sfckind(NPIX, MAX_SCANS)
c      real        frzheight(NPIX, MAX_SCANS)
      integer*2   pixel_status(NPIX, MAX_SCANS)
      integer*2   rain_status(NPIX, MAX_SCANS)

      integer     lin, pix
      integer     i, j, n_ok
      integer     n_dim, n_half, N2, N2N21
      real        sumx, sumxx, sig85h
      real        sumy, sumyy, sig37h
      real        sumxy, cr_cor, slope 

      logical     app_sfc
      real        T19V, DT19, T22V, T37H, T85H, d85_37

      real        WARM85_CS, T37_COLD, T22_COLD
      real        AMB_HI, AMB_HI2, AMB_LO, AMB_INT
      real        POL, DT_OC, PCP_INT 
      real        SIG_CUT, RHO_CUT, SLP_CUT, CDIFF

      integer     MASKS, MASKP, MASKI, MASKO, MASKC

c     Set values for screens that will be used for rain_status.
c     Masks imply that no rain is possible so mask values < 0.

      MASKS = -21
      MASKP = -41
      MASKI = -31
      MASKO = -51
      MASKC = -61

c     Set threshold values needed by the screening algorithm
c     The added values are for Tbs.

      WARM85_CS = 257.
      T22_COLD  = 192. + 0.      ! This may be a bug is GSCAT
      T37_COLD  = 185. + 1.7
      AMB_HI    = 262. + 7.1
      AMB_HI2   = 262. - 5.1
      AMB_LO    = 255. + 6.9
      AMB_INT   = 158. + 5.3
      POL       =  20. + 0.25
      DT_OC     =   0. + 0.5
      PCP_INT   =  38. - 0.1
      SIG_CUT   = 6.0
      RHO_CUT   = 0.5
      SLP_CUT   = 1.2
      CDIFF     = -10. - 0.5

      do lin = 1, nwork
         do pix = 1, NPIX
            if ( pixel_status(pix,lin) .lt. 0 ) goto 10

            ! Identify applicable surfaces.  This screen is
            ! for coastal surfaces only.

            app_sfc = .FALSE.
            if (sfctype(pix,lin) .EQ. 2) app_sfc = .TRUE.

            if ( app_sfc ) then

               ! The following screens come from GSCAT

               T19V   = Tc(pix,lin,2)
               DT19   = Tc(pix,lin,2) - Tc(pix,lin,1) 
               T22V   = Tc(pix,lin,3)
               T37H   = Tc(pix,lin,4)
               T85H   = Tc(pix,lin,6)
               d85_37 = T85H-T37H 

               !  Warm Tb at 85 but unusually low 22 GHz is defined
               !  as not raining
              
               if ((T85H .gt. WARM85_CS).and.(T22V .lt. AMB_HI)) then
c                 rain_status(pix,lin) = MASKO
                  rain_status(pix,lin) = 0

               !  If the 22 GHz is too cold, then it is likely ice

               elseif (T22V .le. T22_COLD) then
                  rain_status(pix,lin) = MASKI

               !  If no scattering is present (T85 > T37) and the 
               !  37 GHz is very cold, then clear ocean likely

               elseif ((D85_37 .gt. DT_OC) .and. 
     &                                (T37H .le. T37_COLD)) then
                  rain_status(pix,lin) = MASKO

               !  Adler/Negri tests:  a large channel difference 
               !  implies that it's likely not coast; do tests for
               !  light precip with ambig., then ambig/cold sfc/
               !  regular.

               elseif (D85_37 .le. CDIFF ) then

                  if (T22V .gt. (PCP_INT + 0.88*T19V)) then

                     if (T22V .lt. AMB_HI2 ) then
                        rain_status(pix,lin) = 63
                     else
                        rain_status(pix,lin) = 0
                     endif

                  elseif (T22V .lt. AMB_HI) then

                     if ((T22V .lt. AMB_LO) .and.
     &                     (T22V .lt. (AMB_INT + 0.49*T85H))) then
                        rain_status(pix,lin) = MASKI

                     else
                        rain_status(pix,lin) = 64
                     endif
                  else

                     ! Rain is possible
                     rain_status(pix,lin) = 0
                  endif

               !  Get the t85h/t37h correlations on an NxN for the
               !  Adler/Negri coast test.  First, check for bad data

               else

                  n_dim  = 5
                  n_half = 2
                  N2     = n_dim*n_dim
                  N2N21  = n_dim*n_dim*(n_dim*n_dim-1)

                  n_ok = 0
                  do j = lin-n_half, lin+n_half
                     do i  = pix-n_half, pix+n_half
                        if (i .lt. 1 ) goto 30
                        if (i .gt. NPIX) goto 30
                        if (j .lt. 1) goto 30
                        if (j .gt. nwork) goto 30
                        if (pixel_status(i,j) .lt. 0 ) goto 30
                        n_ok = n_ok + 1
                     end do
                  end do

   30             if (n_ok .ne. N2) then
                  ! temorarily set pixel status to +99 
                  pixel_status(pix,lin) = 99

                  !  The 5x5 matrix contains good values

                  else

                  !  Accumulate sums and sums of squares

                     sumx  = 0.
                     sumy  = 0.
                     sumxx = 0.
                     sumyy = 0.
                     sumxy = 0.
                     do j  = lin-n_half, lin+n_half
                        do i = pix-n_half, pix+n_half
                           sumx = sumx + Tc(i,j,6)
                           sumy = sumy + Tc(i,j,4)
                           sumxx = sumxx + Tc(i,j,6)*Tc(i,j,6)
                           sumyy = sumyy + Tc(i,j,4)*Tc(i,j,4)
                           sumxy = sumxy + Tc(i,j,6)*Tc(i,j,4)
                        end do
                     end do
                     
                     sig85h = sqrt((N2*sumxx - sumx*sumx)/N2N21)
                     sig37h = sqrt((N2*sumyy - sumy*sumy)/N2N21)
                     if (sig85h*sig37h .ne. 0.) then
                        cr_cor = (N2*sumxy - sumx*sumy)/
     &                           (N2N21*sig85h*sig37h)
                        slope  = cr_cor*sig85h/sig37h
                     else
                        cr_cor = 0.
                        slope  = 0.
                     endif

                     !  A/N estimate for probable coastline

                     if ((sig85h .gt. SIG_CUT) .and. (cr_cor .gt.
     &                     RHO_CUT) .and. (slope .lt. SLP_CUT)) then
                        rain_status(pix,lin) = MASKC

                     !  Gordy light precip. test with ambiguous

                     elseif (T22V .gt. (PCP_INT + 0.88*T19V)) then

                        if (T22V .lt. AMB_HI2) then
                           rain_status(pix,lin) = 65
                        else
                           rain_status(pix,lin) = 0
                        endif

                     !  Huffman ambiguous and Grody cold sfc test.
                     !  No std. dev. test is done because the
                     !  fluctuations in the background may be large

                     elseif (T22V .lt. AMB_HI) then

                        if ((T22V .lt. AMB_LO) .and. 
     &                       (T22V .lt. (AMB_INT + 0.49*T85H))) then
                           rain_status(pix,lin) = MASKI
                        else
                           rain_status(pix,lin) = 66
                        endif

                     !  No more tests

                     else

                        rain_status(pix,lin) = 0
                     endif
                  endif
               endif
            end if
  10     end do
      end do
      ! Finally, set pixels w. status = +99 to "bad"
      do lin = 1, nwork
         do pix = 1, NPIX
           if (pixel_status(pix,lin) .eq. 99) 
     &                       pixel_status(pix,lin) = -15
         end do
      end do

      return
      end
