      SUBROUTINE TIMS87(YMD,HMS,SEC87)
c******************************************************************************
c*
c*      TIMS87(YMD,HMS,SEC87)
c*
c*      PURPOSE:  Convert time (YYYYMMDD,HHMMSS) to seconds starting 1987
c*
c*      I/O PARAMETERS:
c*         YMD     INPUT  INTEGER  year, month and day: YYYYMMDD
c*         HMS     INPUT  INTEGER  hour, minute, second: HHMMSS
c*         SEC87   OUTPUT INTEGER  seconds starting 1987
c*
c*      SUBPROGRAMS REFERENCED
c*         TAE:
c*         IIS:
c*         M75SUB:
c*         TPIO:
c*         LOCAL:   HMSSEC - get seconds in a day
c*                  IDAYS - get days between 2 dates
c*                  JULDAY - find julian day
c*
c*      I/O ():  none
c*
c*      COMMON AREAS: none
c**
c*      BY: W. Robinson, GSC, 20 Apr 88
c*
c*      MODIFICATION HISTORY:
c*
c******************************************************************************
c
      IMPLICIT NONE
      INTEGER YMD, HMS, SEC87, SEC, YEAR, XYMD, DAY, IDAYS,
     1     DAY87, YJD
      REAL XSEC
C
C
C       START, CONVERT HOURS, MIN, SEC TO SECONDS
      CALL HMSSEC(HMS,XSEC)
      SEC=XSEC
C
C       CONVERT YMD TO YYDDD FORM
      YEAR=YMD/10000
      XYMD=MOD(YMD,1000000)
      CALL JULDAY(XYMD,DAY)
      YJD=(YEAR*1000+DAY) - 1970000
C
C       NOTE THAT IDAYS WORKS WITH YEAR/JULIAN DAY WHICH IS
C       OFFSET BY 1970, HENCE THE 1970000 BIAS ABOVE AND USE OF 17001
C       IN IDAYS BELOW. NOTE ALSO THAT IDAYS PROBABLY DOSEN'T
C       WORK CORRECTLY PAST THE YEAR 2000
C
C       GET # DAYS SINCE JAN 1 1987
      DAY87=IDAYS(17001,YJD)
C
C       COMBINE DAYS AND SECONDS TO SECONDS
      SEC87=(DAY87*86400)+SEC
C
C       AND RETURN
      RETURN
      END





      SUBROUTINE HMSSEC(HHMMSS,SECDAY)
c******************************************************************************
c*
c*      hmssec(hhmmss,secday)
c*
c*      PURPOSE: Convert time in HHMMSS format to seconds of day.
c*
c*      I/O PARAMETERS:
c*         hhmmss    input   Integer*4  time in HHMMSS
c*         secday    output  Real       time in seconds of day.
c*
c*      SUBPROGRAMS REFERENCED
c*         TAE:
c*         IIS:
c*         M75SUB:
c*         TPIO:
c*         LOCAL:
c*
c*      I/O ():
c*
c*      COMMON AREAS: none
c**
c*      BY: James K. Firestone (GSC)  August 1, 1985
c*
c******************************************************************************
      IMPLICIT NONE
C
      REAL      SECDAY
      INTEGER*4 HHMMSS
      INTEGER   HOUR,MINUTE,SECOND
C
      HOUR   =  HHMMSS / 10000
      MINUTE = (HHMMSS - HOUR * 10000) / 100
      SECOND =  HHMMSS - (HOUR*10000 + MINUTE*100)
      SECDAY = FLOAT(HOUR*3600 + MINUTE*60 + SECOND)
C
      RETURN
      END





      SUBROUTINE JULDAY (YYMMDD,JDAY)
c******************************************************************************
c*
c*      julday
c*
c*      PURPOSE:  Converts month and date to julian day
c*
c*      INPUT PARAMETER:
c*         yymmdd     integer     yy*10000 + mm*100 + dd
c*      OUTPUT PARAMETER:
c*         jday       integer     converted julian date
c*
c*      SUBPROGRAMS REFERENCED
c*         TAE:
c*         IIS:
c*         M75SUB:
c*         TPIO:
c*         LOCAL:
c*
c*      I/O ():
c*
c*      COMMON AREAS: none
c**
c*      BY:   Jean Liu                  GSC             3/23/1984
c*
c******************************************************************************
c
      IMPLICIT NONE
      INTEGER*4 YYMMDD, JDAY, MONTH(12), YY, MM, DD, IT
c
      DATA MONTH /0,31,59,90,120,151,181,212,243,273,304,334/
c
c********************************************************************
C
C
      DD = MOD (YYMMDD, 100)
      IT = YYMMDD / 100
      MM = MOD (IT, 100)
      YY = IT / 100
C
      JDAY = MONTH(MM) + DD
      IF (MOD (YY, 4) .EQ. 0) THEN
          IF (MM .GT. 2) JDAY = JDAY + 1
      ENDIF
C
C
C     EXIT
C
C
      RETURN
      END




      INTEGER FUNCTION IDAYS(YRDA1,YRDA2)
c******************************************************************************
c*
c*      idays(yrda1,yrda2)
c*
c*      PURPOSE:
c*           This function calculates the number of days difference
c*           between two integer packed times in the form YYDDD, where
c*           YY is the years since 1970.
c*
c*      I/O PARAMETERS:
c*         yrda1   input   integer    first date in YYDDD form (since 1970)
c*         yrda2   input   integer    seconde date in YYDDD form (since 1970)
c*
c*      RETURNED VALUE:
c*         idays   output  integer    number of days difference between dates
c*
c*      SUBPROGRAMS REFERENCED
c*         TAE:
c*         IIS:
c*         M75SUB:
c*         LOCAL:
c*
c*      I/O ():
c*
c*      COMMON AREAS: none
c**
c*      BY: C.T. Mottershead      CSC   MAY 18, 1976
c*      G.J.Huffman/USRA        11/91   Documentation
c*
c******************************************************************************
C
      IMPLICIT NONE
      INTEGER YRDA1,YRDA2,Y1,D1,FAC1,Y2,D2,FAC2
C
      Y1 = MOD (YRDA1/1000,100) + 70
      D1 = MOD (YRDA1,1000)
      FAC1 = (Y1-1)/4 + 1
      Y2 = MOD (YRDA2/1000,100) + 70
      D2 = MOD (YRDA2,1000)
      FAC2 = (Y2-1)/4 + 1
      IDAYS = 365*(Y2-Y1) + FAC2 - FAC1 + D2 - D1
C
      RETURN
      END



