C@***h* GPCP_Daithly_Analysis/Main_GPCPcdr_d4_adjir.f
C       Global Precipitation Climatology Project (GPCP)
C
C NAME
C   Main_GPCPcdr_d4_adjir.f
C
C PURPOSE
C   To match IR to GPROF PMW data and scale to GPCP monthly
C
C DESCRIPTION
C   Input command-line arguments:
C   help      CHAR      Request help
C   <null>            No entry flags interactive input
C   X            CHAR      Any other entry flags non-interactive input
C                                              
C   Input data (interactive or a script):
C   THR_NAME      CHAR      File name of thresholds from threshold matching
C                 of geo-IR Tb to SSM/I, or 'none'
C   RTE_NAME      CHAR      File name of rainrates from threshold matching
C                 of geo-IR Tb to SSM/I, or 'none'
C   LZPT_NAME     CHAR      File name of leo-IR revised zero points, or 'none'
C   LRAT_NAME     CHAR      File name of leo-IR correction ratios, or 'none'
C   IR_NAMES      CHAR    File names of 1x1-deg 3-hrly merged IR data;
C                  - there can be up to 31 entries,
C                  - with the last entry is '999';
C                  or the name of a file containing such a list
C                  (without the terminating '999') as NAME,file
C   HQ_TPLATE     CHAR      High-quality file name template, with YYYY (or
C                  YY), MM, DD, HH to be filled in with IR data's
C                  year, month, day, hour, or 'none'
C   OUTBASE       CHAR      Base name of output files
C  
C AUTHOR
C   Robert Adler, Mathew Sapiano, Jian-Jian Wang
C   University of Maryland College Park
C
C COPYRIGHT
C   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE
C   PUBLIC
C   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
C   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
C   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
C   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
C   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
C   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE
C   TECHNICAL
C   SUPPORT TO USERS.
C
C HISTORY
C 1/2017 Initial Version prepared for NCEI
C
C@*****      
      program Main_GPCPcdr_d4_adjir

      IMPLICIT      NONE
      
      character*256, InterimPATH, irPATH, IRlist, fname
      character*6 yyyymm
      integer Ninputs, inunit, ios, strlen, l_fbas, l_ir
      logical opn
      integer*4     iyear, imonth, ndays
      
      INTEGER     LN, LT, LBIN, NHR, MAXNAM
      REAL        RMISS
      PARAMETER      ( LN     = 360,
     +            LT     = 80,
     +     LBIN   = 24,
     +     NHR    = 8,
     +     MAXNAM = 31,
     +     RMISS  = -99999. )
      


      CHARACTER*256  fbas, tbas
      CHARACTER*256  iname (MAXNAM)

        CHARACTER*2 c_hh

        INTEGER nir, iret, iir, ihr, jyrir, jdayir, jminir
        INTEGER*4  i, j, m, n, it

        REAL  rate(LN,LT)
        REAL  zptl(LN,LT)
        REAL  ratl(LN,LT)
        REAL  pcum(LN,LT)
        REAL  pcumg(LN,LT)
        REAL  pcumo(LN,LT)
        REAL  pcuml(LN,LT)
        REAL  pcumh(LN,LT)
        REAL  p_g(LN,LT)
        REAL  gpi(LN,LT)
        REAL  p_l(LN,LT)
        REAL  po(LN,LT)
        REAL  pg(LN,LT)
        REAL  p(LN,LT)

        INTEGER*4  ithr(LN,LT)
        INTEGER*4  ncum(LN,LT)
        INTEGER*4  ncumg(LN,LT)
        INTEGER*4  ncumo(LN,LT)
        INTEGER*4  ncuml(LN,LT)
        INTEGER*4  ncumh(LN,LT)

        INTEGER*2  ltb(LN,LT,LBIN)
        INTEGER*2  lza(LN,LT)
        INTEGER*2  igeop(LN,LT)
        INTEGER*2  leop(LN,LT)
        INTEGER*2  ntbh(189:270, LN, LT)




c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 4 ) then
         WRITE(*,*) 'Error in GPCPcdr_d4_adjir:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif
      
C     Get arguments and set the length of the month in hours.
      call getarg(1, yyyymm)
      call getarg(2, InterimPATH)
      call getarg(3, irPATH)
      call getarg(4, IRlist)
      
      
c     Convert month to a number
      read(yyyymm(1:4), '(i10)' ) iyear
      read(yyyymm(5:6), '(i10)' ) imonth
      
      
      
c     get basename for output files
      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
 
      fbas = InterimPATH(1:strlen)//'/ADJIR/'//yyyymm//'/1dd'
      do l_fbas = 1,len_trim(fbas)
         if (fbas(l_fbas:l_fbas) .eq. ' ') then
            continue
         end if
      end do
      l_fbas=l_fbas-1
       

      
c     Read files produced by CORR_G that are required as inputs
      fname = InterimPATH(1:strlen)//'/CORR_G/'//
     +     yyyymm//'/1dd.'//yyyymm//'.thr'
      CALL Read_Binary_File_int4 (fname, LN, LT, 1, ithr)
      fname = InterimPATH(1:strlen)//'/CORR_G/'//
     +     yyyymm//'/1dd.'//yyyymm//'.rte'
      CALL Read_Binary_File (fname, LN, LT, 1, rate)
      fname = InterimPATH(1:strlen)//'/CORR_G/'//
     +     yyyymm//'/1dd.'//yyyymm//'.zptl'
      CALL Read_Binary_File (fname, LN, LT, 1, zptl)
      fname = InterimPATH(1:strlen)//'/CORR_G/'//
     +     yyyymm//'/1dd.'//yyyymm//'.ratl'
      CALL Read_Binary_File (fname, LN, LT, 1, ratl)

c     Read contents of IRlist
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      open(inunit, file=IRlist, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_adjir:'
     &        //'Failed to open IR file listing file'
         STOP
      end if
      nir=0
      do i=1,MAXNAM
         read(inunit, '(A256)', iostat=ios ) iname(i)
         if ( ios .EQ. 0 ) then
            nir=nir+1
         else
            iname(i) = '999'
         endif
      end do
      if ( nir .EQ. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_adjir:'
     &        //'Failed to read IR file listing file'
         STOP
      end if
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_adjir:'
     &        //'Failed to close IR file listing file'
         STOP
      end if
      
      


c************************************************************
C     Initialize fields.  The IR reader handles file number 
C     internally, because there are 8 fields in each IR file.  [So 
C     initialize iir,ihr to not valid.]
C
      iir = 0
      ihr = 0
      

C     General concept:  Load the coefficients, then open each IR file
C     and step through each hour.  The indices of the DO loops are
C     checked against the internal looping in GET_IRTB.  Compute 
C     averages at the end of each day and open the next day.


c     Loop over days in teh month
      DO  n = 1, nir

c        Initialize variables
         DO  j = 1, LT
            DO  i = 1, LN
               pcum  (i, j) = 0.
               pcumg (i, j) = 0.
               pcumo (i, j) = 0.
               pcuml (i, j) = 0.
               pcumh (i, j) = 0.
               ncum  (i, j) = 0
               ncumg (i, j) = 0
               ncumo (i, j) = 0
               ncuml (i, j) = 0
               ncumh (i, j) = 0
            END DO
         END DO
         
c        Loop over 3 hour periods in current day         
         DO  m = 1, NHR

c           Read IR data for current 3-hour period
            CALL GET_IRTB ( iname,  nir, iir, ihr,   jyrir, jdayir, 
     +           jminir, ltb, lza, igeop, leop,  iret )

            IF  ( iret .NE. 0 ) THEN
               WRITE (*, *) 'Error in GPCPcdr_d4_ADJIR: could not '//
     +              'read IR data: '//iname(iir)
               STOP
            END iF
            IF  ( n .NE. iir ) THEN
               WRITE (*, *) 'Error in GPCPcdr_d4_ADJIR: DO loop and'//
     +              'IR data reader out of synch; n,iir =', n, ',', iir
               STOP
            END IF
            IF  ( m .NE. ihr )  THEN
               WRITE (*, *) 'Error in GPCPcdr_d4_ADJIR: DO loop and '//
     +              'IR data reader out of synch; m,ihr =', m, ',', ihr
               STOP
            END IF


c           Apply ZA correction
            CALL ZA_TB  ( LN, LT, LBIN, ltb,    lza,
     +           -1, -1, -1,   'none', ntbh, iret)
            DO  j = 1, LT
               DO  i = 1, LN
                  IF  ( ntbh (189, i, j) .GE. 0 )  THEN
                     DO  it = 190, 270
                        ntbh (it, i, j) = ntbh (it-1, i, j) +
     +                       ntbh (it,   i, j)
                     END DO
                  END IF
               END DO
            END DO

            DO  j = 1, LT
               DO  i = 1, LN
                  IF  ( ntbh (270, i, j) .GT. 0 )  THEN
                     IF  ( rate (i, j) .GE. 0. )  THEN
                        p_g (i, j) = rate (i, j) *
     +                       ntbh (ithr (i, j), i, j) /
     +                       ntbh (270,         i, j)
                     ELSE
                        p_g (i, j) = RMISS
                     END IF
                     gpi (i, j) = 72. *
     +                    ntbh (234, i, j) /
     +                    ntbh (270, i, j)
                  ELSE
                     p_g (i, j) = RMISS
                     gpi (i, j) = RMISS
                  END IF
               END DO
            END DO
            

C           Compute the adjusted IR field where 
            DO  j = 1, LT
               DO  i = 1, LN
                  IF  ( leop (i, j) .GE. 0  .AND.
     +                 zptl (i, j) .GE. 0. .AND.
     +                 ratl (i, j) .GE. 0. )  THEN
                     p_l (i, j) = AMAX1 ( 0., ratl (i, j) * 
     +                    ( leop (i, j) * .01 - zptl (i, j) ) )
                     pcuml (i, j) = pcuml (i, j) + p_l (i, j)
                     ncuml (i, j) = ncuml (i, j) + 1

                  ELSE
                     p_l (i, j) = RMISS
                  END IF
               END DO
            END DO
         


C           Accumulate original GPI, internally computed GPI
C           (except uses original leo), and TMPI as geo, else leo,
C           else missing.

            DO  j = 1, LT
               DO  i = 1, LN
                  IF  ( igeop (i, j) .GE. 0 )  THEN
                     po    (i, j) =                igeop (i, j) * .01
                     pcumo (i, j) = pcumo (i, j) + igeop (i, j) * .01
                     ncumo (i, j) = ncumo (i, j) + 1
                  ELSE IF  ( leop (i, j) .GE. 0 )  THEN
                     po    (i, j) =                leop (i, j)  * .01
                     pcumo (i, j) = pcumo (i, j) + leop (i, j)  * .01
                     ncumo (i, j) = ncumo (i, j) + 1
                  ELSE
                     po    (i, j) = RMISS
                  END IF
                  
                  IF  ( gpi (i, j) .NE. RMISS )  THEN
                     pg    (i, j) =                gpi (i, j)
                     pcumg (i, j) = pcumg (i, j) + gpi (i, j)
                     ncumg (i, j) = ncumg (i, j) + 1
                  ELSE IF  ( p_l (i, j) .NE. RMISS )  THEN
                     pg    (i, j) =                leop (i, j) * .01
                     pcumg (i, j) = pcumg (i, j) + leop (i, j) * .01
                     ncumg (i, j) = ncumg (i, j) + 1
                  ELSE
                     pg    (i, j) = RMISS
                  END IF
                  
                  IF  ( p_g (i, j) .NE. RMISS )  THEN
                     p     (i, j) =                p_g (i, j)
                     pcum  (i, j) = pcum  (i, j) + p_g (i, j)
                     ncum  (i, j) = ncum  (i, j) + 1
                  ELSE IF  ( p_l (i, j) .NE. RMISS )  THEN
                     p     (i, j) =                p_l (i, j)
                     pcum  (i, j) = pcum  (i, j) + p_l (i, j)
                     ncum  (i, j) = ncum  (i, j) + 1
                  ELSE
                     p     (i, j) = RMISS
                  END IF
               END DO
            END DO



C           Done with IR for this hour; build a temporary basename 
C           by appending the (presumed) date in the IR filename and 
C           the hour number to the basename, and dump out fields.

            do l_ir = 1,len_trim(iname(iir))
               if (iname(iir)(l_ir:l_ir) .eq. ' ') then
                  continue
               end if
            end do
            l_ir=l_ir-1
            WRITE ( c_hh, 10 )  ( ihr - 1 ) * 3
 10         FORMAT ( I2.2 )

            tbas = fbas (1:l_fbas) // '.' // 
     +           iname (iir) (l_ir-7:l_ir) // c_hh

C     

            do strlen = 1,len_trim(tbas)
               if (tbas(strlen:strlen) .eq. ' ') then
                  continue
               end if
            end do
            strlen=strlen-1

            fname=tbas(1:strlen)//'.plo'
            CALL Write_Binary_File(fname, po, LN, LT)
            fname=tbas(1:strlen)//'.pl'
            CALL Write_Binary_File(fname, p_l, LN, LT)
            fname=tbas(1:strlen)//'.pg'
            CALL Write_Binary_File(fname, pg, LN, LT)
            fname=tbas(1:strlen)//'.pt'
            CALL Write_Binary_File(fname, p, LN, LT)

         END DO



     
C        Done with hours in the day; compute average, build a 
C        temporary basename by appending the (presumed) date in the
C        IR filename to the basename, and dump out fields.

         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( ncuml (i, j) .GT. 0 )  THEN
                  pcuml (i, j) = pcuml (i, j) / ncuml (i, j)
               ELSE
                  pcuml (i, j) = RMISS
               END IF
               IF  ( ncumo (i, j) .GT. 0 )  THEN
                  pcumo (i, j) = pcumo (i, j) / ncumo (i, j)
               ELSE
                  pcumo (i, j) = RMISS
               END IF
               IF  ( ncumg (i, j) .GT. 0 )  THEN
                  pcumg (i, j) = pcumg (i, j) / ncumg (i, j)
               ELSE
                  pcumg (i, j) = RMISS
               END IF
               IF  ( ncum (i, j) .GT. 0 )  THEN
                  pcum  (i, j) = pcum  (i, j) / ncum (i, j)
               ELSE
                  pcum  (i, j) = RMISS
               END IF
            END DO
         END DO


         do l_ir = 1,len_trim(iname(iir))
            if (iname(iir)(l_ir:l_ir) .eq. ' ') then
               continue
            end if
         end do
         l_ir=l_ir-1
         tbas = fbas (1:l_fbas) // '.' // iname (iir) (l_ir-7:l_ir)

         do strlen = 1,len_trim(tbas)
            if (tbas(strlen:strlen) .eq. ' ') then
               continue
            end if
         end do
         strlen=strlen-1
         
         fname=tbas(1:strlen)//'.p'
         CALL Write_Binary_File(fname, pcum, LN, LT)
         fname=tbas(1:strlen)//'.gpi'
         CALL Write_Binary_File(fname, pcumg, LN, LT)
         fname=tbas(1:strlen)//'.ogpi'
         CALL Write_Binary_File(fname, pcumo, LN, LT)
         fname=tbas(1:strlen)//'.pl'
         CALL Write_Binary_File(fname, pcuml, LN, LT)
         fname=tbas(1:strlen)//'.np'
         CALL Write_Binary_File_int4(fname, ncum, LN, LT)
         fname=tbas(1:strlen)//'.ngpi'
         CALL Write_Binary_File_int4(fname, ncumg, LN, LT)
         fname=tbas(1:strlen)//'.nogpi'
         CALL Write_Binary_File_int4(fname, ncumo, LN, LT)
         fname=tbas(1:strlen)//'.npl'
         CALL Write_Binary_File_int4(fname, ncuml, LN, LT)

      END DO

     
      STOP
      
      end program Main_GPCPcdr_d4_adjir
      















