C@***h* Main_GPCPcdr_d4_corr_g.f
C       Global Precipitation Climatology Project (GPCP)
C
C NAME
C   Main_GPCPcdr_d4_corr_g.f
C
C PURPOSE
C   To match IR to GPROF PMW data and scale to GPCP monthly
C
C DESCRIPTION
C This program computes the correction coefficients for threshold-
C matched geo-IR and total leo-IR from instantaneous high-quality
C estimates (HQ) - up to 3 SSM/I and 1 TMI -  and scales to the HQ or
C     the monthly GPCP Monthly fields.
c
C Input data (interactive or a script):
C   IR_NAMES      CHAR    File names of 1x1-deg 3-hrly merged IR data;
C                  - files are sorted in ascending date/time,
C                  - there can be up to 910 entries,
C                  - with the last entry is '999';
C                  or the name of a file containing such a list
C                  (without the terminating '999') as NAME,file
C   S1_NAMES      CHAR    As above for first SSM/I
C   S2_NAMES      CHAR    As above for second SSM/I, or 'none'
C   S3_NAMES      CHAR    As above for third SSM/I, or 'none'
C   TM_NAMES      CHAR    As in IR_NAMES for TMI, or 'none'
C   OCN_CUMS      CHAR      File name of SSM/I ocean cumulative histogram,
C                  or 'none' (land will be used, if available)
C   OCN_CUMT      CHAR      File name of TMI ocean cumulative histogram,
C                  or 'none' (land will be used, if available)
C   LND_CUMS      CHAR      File name of SSM/I land cumulative histogram,
C                  or 'none' (ocean will be used, if available)
C   LND_CUMT      CHAR      File name of TMI land cumulative histogram,
C                  or 'none' (ocean will be used, if available)
C   MON_NAME      CHAR      File name of 2.5x2.5-deg monthly precip field
C                  (in mm/mo) for getting rain rate, or 'none'
C   DATE          CHAR      Date as YYMM
C   OUTBASE       CHAR      Base name of output files
C   SMOOTH        CHAR      Size of spatial smoother for IR histograms and
C                 HQ fractional coverage; 0 = none, 3 = only
C                  valid non-zero value
C   ADFRAC        CHAR      Linear fit of current microwave fractional
C                  coverage to F13 fractional coverage as slope,
C                  intercept of fit line (units are non-dim.,
C                  fraction [not %]), or '-' for no adjustment (1.,0.)
c     Note that SSMI fields used to have capacity to take 3
c     ssmi inputs. In practice, only 1 SSMI/SSMIS is used at a time and 
c     this was eliminated for the CDR.  Variables with HXMX=3 dimension removed:
c        sname, nss, jyrhq, jdayhq, jhq_n, jhq_a, hq_p, hq_f
C      subscript on nss is HXMX to accomodate IF test below; really 
C      only carries first 3 values.
c      Sapiano: note that HXMX is not really needed for CDR, since HXMX=1
c           is the only subscript used.  Dimension of ssmi
C      hq_f  - SSMI pixel fractional coverage by overpass
C      fcum  - SSMI pixel fractional coverage by month (in fcumu), 
C              then smoothed version (in fcum1) for use with geo
C            fraction of pixels
C      fcums - SSMI 1x1-deg fractional coverage by month (in fcumsu),
C              then smoothed version (in fcums) for use with leo
C            fraction of grid boxes
C
      program Main_GPCPcdr_d4_corr_g


      IMPLICIT      NONE

      character*256, InterimPATH, irPATH, IRlist, gprofPATH, GPROFlist
      character*6 yyyymm
      integer Ninputs, inunit, ios, strlen, l_fbas, l_ir
      logical opn, xst
      integer*4     iyear, imonth, ndays
      integer*4     NdaysInMonth(12)
     &     / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

      INTEGER*4      LN,    LT, LBIN, LHR, MAXNAM, MAXPDF, MXBIN
      INTEGER*2      I2MISS
      REAL*4            RMISS, T_TO_S
      PARAMETER      ( LN     = 360,
     +                    LT     = 80,
     +                    LBIN   = 24,
     +                    LHR    = 8,
     +                    MAXNAM = 910,
     +                    MAXPDF = 124,
     +                    MXBIN  = 10000,
     +                    RMISS  = -99999.,
     +                    I2MISS = -31999,
     +                    T_TO_S = .8 )
C
      INTEGER*4      iunit,  iostat, i,   j, k,   m,      ihq, 
     +                  it,     jdiff,  npass,  l_m, iret1,  iret2,
     +                  iret3,  iret4,  iret5,  nin, test
      LOGICAL*1     active,  more
C
      CHARACTER*256      iname (MAXNAM), sname (MAXNAM), 
     +                  mname, fbas, tbas, tempfile, fname
      CHARACTER*2      c_hr

      INTEGER*4      nir, nss, iret, iir, ihr, jyrir, 
     +                  jdayir, jminir, jyrhq, jdayhq
      INTEGER*2      jhq_n  (         LN, LT),
     +                  jhq_a  (         LN, LT)
      REAL*4            pcum   (         LN, LT),    fcum   (LN, LT),
     +                  hq_p   (         LN, LT), 
     +                  hq_f   (         LN, LT),
     +                  rate   (         LN, LT),    thrsh  (LN, LT),
     +                  ftot   (         LN, LT),    p_mon  (144,72),    
     +                  fcums  (         LN, LT),    ratl   (LN, LT),    
     +                  pdf    (MAXPDF,  LN, LT),    z_pt   (LN, LT),
     +                  pcum3  (         LN, LT),    conv,
     +                  histo  (         MXBIN, 2),  histl  (MXBIN, 2),
     +                  dum, adfrat, adfcon
      INTEGER*4      ntbcum (189:270, LN, LT),    ncum   (LN, LT),
     +                  ncuma3 (         LN, LT),
     +                  ncuma  (         LN, LT),    ithrsh (LN, LT),
     +                  ntbtot (189:270, LN, LT),    ncum3  (LN, LT),
     +                  npdf   (         LN, LT),    ncums  (LN, LT),
     +                  ncumas (         LN, LT),    nsm
      INTEGER*2      ltb    (         LN, LT, LBIN),
     +                  lza    (         LN, LT), leop   (LN, LT),
     +                  ntbh   (189:270, LN, LT), jminhq (LN, LT),
     +                  ngstrt (189:270, LN, LT), igeop  (LN, LT)

      INTEGER*4      INIT_DISK, INITW_DISK, READ_DISK, WRITE_DISK,
     +                  EXIT_DISK

      REAL*4            smth (15, 15)
      DATA            smth / 225*1. /






c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 6 ) then
         WRITE(*,*) 'Error in GPCPcdr_d4_CORR_G:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, yyyymm)
      call getarg(2, InterimPATH)
      call getarg(3, irPATH)
      call getarg(4, IRlist)
      call getarg(5, gprofPATH)
      call getarg(6, GPROFlist)


c     Convert month to a number
      read(yyyymm(1:4), '(i10)' ) iyear
      read(yyyymm(5:6), '(i10)' ) imonth

c     Determine number of days in month
      ndays=NdaysInMonth(imonth)
      if ( imonth.EQ.2 ) then 
         if ( mod(iyear,400).EQ.0 ) then
            ndays=29
         elseif ( mod(iyear,100).EQ.0 ) then
            ndays=28
         elseif ( mod(iyear,4).EQ.0 ) then
            ndays=29
         endif
      endif
      conv=1./ndays

c     get basename for output files
      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      fbas =InterimPATH(1:strlen)//'/CORR_G/'//yyyymm//'/1dd.'//yyyymm

c     set size of spatial smoother for IR, SSMI
      nsm=3

c     set constant and slope of linear fit of current 
c     microwave fractional coverage to F13 fractional 
c     coverage (units of fraction, not %)
c      adfrat=1.0
c      adfcon=0.0
      adfrat=0.927
      adfcon=-0.00267


c     Read contents of IRlist
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      open(inunit, file=IRlist, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Failed to open IR file listing file'
         STOP
      end if
      nir=0
      do i=1,MAXNAM
         read(inunit, '(A256)', iostat=ios ) iname(i)
         if ( ios .EQ. 0 ) then
            nir=nir+1
         else
            iname(i) = '999'
         endif
      end do
      if ( nir .EQ. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Failed to read IR file listing file'
         STOP
      end if
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Failed to close IR file listing file'
         STOP
      end if

c     Read contents of GPROFlist and set mss
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      open(inunit, file=GPROFlist, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Failed to open GPROF file listing file'
         STOP
      end if
      nss=0
      do i=1,MAXNAM
         read(inunit, '(A256)', iostat=ios ) sname(i)
         if ( ios .EQ. 0 ) then
            nss=nss+1
         else
            sname(i) = '999'
         endif
      end do
      if ( nss .EQ. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Failed to read GPROF file listing file'
         STOP
      end if
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Failed to close GPROF file listing file'
         STOP
      end if

c     get name of monthly SG field from monthly CDR code
c     Run check that monthly code exists before proceeding
      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      mname=InterimPATH(1:strlen)//'FUSE2/'//'/oc.'//
     +     yyyymm//'.sg'

      do strlen = 1,len_trim(mname)
         if (mname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      INQUIRE ( FILE = mname, EXIST = xst )
      if ( .NOT. xst ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_d4_corr_g:'
     &        //'Could not access monthly precip SG file'
         STOP
      end if



c************************************************************
C      Initialize fields.  "active" starts .TRUE. for all available
C      HQ, then goes .FALSE. as each runs out of data.  The PMW
C      reader (GET_GORB) depends on the user to set the file number, 
C      but the IR reader handles it internally, because there are 8 
C      fields in each IR file.  [So initialize iir,ihr to not valid.]
C      Also, zero accumulation arrays.


      IF  ( nss .GT. 0 )  THEN
         active  = .TRUE.
      ELSE
         active = .FALSE.
      END IF
      ihq = 1
      iir = 0
      ihr = 0
C
      DO  j  = 1,   LT
      DO  i  = 1,   LN
      DO  it = 189, 270
          ntbtot (it, i, j) = 0
          ntbcum (it, i, j) = 0
          ngstrt (it, i, j) = 0
      END DO
      END DO
      END DO
      DO  j = 1, LT
      DO  i = 1, LN
          pcum   (i, j) = 0.
          fcum   (i, j) = 0.
          fcums  (i, j) = 0.
          ncum   (i, j) = 0
          ncums  (i, j) = 0
          ncuma  (i, j) = 0
          ncumas (i, j) = 0
          npdf   (i, j) = 0
          ratl   (i, j) = 0.
      END DO
      END DO




C     General concept: Open an IR time, then step through SSMI
C     in turn until all times prior to the end of the IR time span 
C     have been processed.  Then (with the SSMI still open)
C     open the next IR and continue; this proceeds until the IR data
C     or all the SSMI are exhausted.  To get started, open the 
C     first in each list; check that the SSMI is within a year of
C     the IR and read the next if it's not. 

      CALL GET_IRTB ( iname, nir, iir,   ihr,  jyrir, jdayir, jminir, 
     +     ltb,   lza, igeop, leop, iret )

      IF  ( iret .NE. 0 )  STOP
      DO  j = 1, LT
         DO  i = 1, LN
            pcum3  (i, j) = 0.
            ncum3  (i, j) = 0
            ncuma3 (i, j) = 0
         END DO
      END DO
      
      IF  ( active )  THEN
         CALL GET_GORB ( sname, jyrhq, jdayhq,
     +        jminhq(1,1), hq_p(1,1), hq_f(1,1), 
     +        jhq_n(1,1), jhq_a(1,1), iret )

         IF  ( iret .LT. 0 )  THEN
            active = .FALSE.
         ELSE
            IF  ( jyrhq - jyrir .GT. 1 )  THEN
               active = .FALSE.
               WRITE (*, *) 'Warning: first of HQ field', m,
     +              ' >1 yr ahead of IR start time'
            ELSE IF  ( jyrhq - jyrir .LT. -1 )  THEN
               ihq = ihq + 1
               IF  ( ihq .GT. nss )  THEN
                  active = .FALSE.
                  WRITE (*, *) 'Warning: ran through all of',
     +                 ' HQ field', m, 
     +                 ' >1 yr behind IR start time'
               END IF
            END IF
         END IF
      END IF
      WRITE (*, *) 'Looping over IR data...'
      




C     Top of main loop.  Start by accumulating total histogram and 
C     coldest Tb of IR with the implicit assumption that the IR handed 
C     in are approximately the month of interest.  Do the ZA 
C     correction (and shift to 1K bins) on the IR Tb (stored for use 
C     in the matching below), then accumulate it.  Unlike the code 
C     for ntbcum (below) there is no weighting by HQ population.  
C     Also, accumulate the leo-IR precipitation PDF.
C     
      DO  WHILE ( active .AND. iir .GT. 0 )
         CALL ZA_TB  ( LN, LT, LBIN, ltb, lza, -1, -1, -1, 'none',
     +        ntbh,   iret )

         IF  ( iret .NE. 0 )  THEN
            WRITE(*,*) 'Error in GPCPcdr_d4_CORR_G:'//
     &           'Failed apply ZA correction'
            STOP
         END IF



         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( lza (i, j) .GE. 0 .AND. lza (i, j) .LE. 900 ) THEN
                  DO  it = 189, 270
                     ntbtot (it, i, j) = ntbtot (it, i, j) +
     +                    ntbh   (it, i, j)
                  END DO
                  DO  it = 189, 270
                     IF  ( ntbh (it, i, j) .GT. 0 )  THEN
                        ngstrt (it, i, j) = ngstrt (it, i, j) + 1
                        GO TO 15
                     END IF
                  END DO
 15               CONTINUE
               END IF
            END DO
         END DO
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( leop (i, j) .GE. 0 )  THEN
                  npdf  (             i, j) = npdf  (i, j) + 1
                  pdf   (npdf (i, j), i, j) = leop  (i, j) * .01
               END IF
            END DO
         END DO
         
C          Top of loop for each HQ.  If the HQ is active, start by
C          setting HQ time field relative to IR time field.  It was
C          checked above that mismatches between IR and HQ times 
C          are less than a year.

         IF  ( active )  THEN
 20         IF  ( jyrhq .NE. jyrir )  THEN
               IF  ( MOD ( jyrhq, 4 ) .EQ. 0 )  THEN
                  jdiff = 366 * ( jyrhq - jyrir )
               ELSE
                  jdiff = 365 * ( jyrhq - jyrir )
               END IF
               jyrhq = jyrir
               jdayhq = jdayhq + jdiff
            END IF
            IF  ( jdayhq .NE. jdayir )  THEN
               jdiff      = 1440 * ( jdayhq - jdayir )
               IF  ( jdiff .LT. I2MISS )  THEN
                  WRITE (*, *) 'Error: start time for ', sname (ihq), 
     +                 ' must be closer to IR start time than', I2MISS,
     +                 ' minutes, but appears to be', jdiff
                  STOP
               END IF
               jdayhq = jdayir
               DO  j = 1, LT
                  DO  i = 1, LN
                     IF  ( jminhq (i, j) .NE. I2MISS ) THEN
                        jminhq(i,j) = jminhq(i,j) + jdiff
                     end if
                  END DO
               END DO
            END IF
C     
C     Compute coincidence as 90 min either side of the
C     nominal IR time.  The order of IFs is chosen such 
C     that the most common missings come first.  The 
C     first jminhq test also catches non-existent GPROF 
C     (most of the field).  "More" flags the existance of 
C     HQ data that should match up with the next IR field.
C     A valid ZA is a convenient flag of the  geo-IR
C     existing.  All of the HQ is accumulated for use
C     with low-orbit.
            more = .FALSE.
            DO  j = 1, LT
               DO  i = 1, LN
                  
                  IF  ( jminhq (i, j) .LE. 
     +                 ( jminir - 90 ) )  THEN
c                     print *, 'This section of code does nothing?'
                  ELSE IF  ( jminhq (i, j) .GT. 
     +                    ( jminir + 90 ) )  THEN
                     more = .TRUE.
                  ELSE IF  ( lza (i, j) .LT. 0 .OR.
     +                    lza (i, j) .GT. 900 )  THEN
                     IF ( hq_f (i, j) .GT. 0. ) THEN
                        fcums(i,j) = fcums(i,j) + jhq_n(i,j)
                     END IF
                     
                     ncums (i, j) = ncums (i, j) + jhq_n (i, j)
                     ncumas(i, j) = ncumas(i, j) + jhq_a (i, j)
                     pcum3 (i, j) = pcum3 (i, j) + 
     +                    hq_p (i,j) * jhq_n (i,j) * 24.
                     ncuma3(i, j) = ncuma3(i, j) + jhq_a (i, j)
                     ncum3 (i, j) = ncum3 (i, j) + jhq_n (i, j)
C     
C     Do the ZA correction (and shift to 1K bins) 
C     on the IR Tb, then accumulate it, the HQ 
C     precip and frequency of precip, and increment 
C     the number-of-cases counter.  At this point
C     the HQ is converted from mm/hr to mm/d.
C     
                  ELSE
                     DO  it = 189, 270
                        ntbcum (it, i, j) = ntbcum (it, i, j) +
     +                       (INT (ntbh(it,i,j)) * INT(jhq_n(i,j)))
                     END DO
                     pcum  (i, j) = pcum  (i, j) + 
     +                    hq_p (i, j) * jhq_n (i, j) * 24.
                     fcum  (i, j) = fcum  (i, j) +
     +                    hq_f (i, j) * jhq_n (i, j)
                     ncum  (i, j) = ncum  (i, j) + jhq_n (i, j)
                     
                     IF ( hq_f (i, j) .GT. 0. ) THEN
                        fcums(i,j) = fcums(i,j) + jhq_n(i,j)
                     END IF
                     ncums (i, j) = ncums (i, j) + jhq_n (i, j)
                     pcum3 (i, j) = pcum3 (i, j) + 
     +                    hq_p (i, j) * jhq_n (i, j) * 24.
                     ncuma3(i, j) = ncuma3(i, j) + jhq_a (i, j)
                     ncum3 (i, j) = ncum3 (i, j) + jhq_n (i, j)
                     ncumas(i, j) = ncumas(i, j) + jhq_a (i, j)
                     ncuma (i, j) = ncuma (i, j) + jhq_a (i, j)
                  END IF
               END DO
            END DO

C     
C     Done with this HQ for this IR.  If there aren't HQ
C     times past the end of the IR, get the next file
C     for this satellite (if one exists) and jump back
C     to process it.
C     
            IF  ( .NOT. more )  THEN
               ihq = ihq + 1
               IF  ( ihq .GT. nss) THEN
                  active = .FALSE.
               ELSE
                  CALL GET_GORB ( 
     +                 sname  (ihq), jyrhq, jdayhq, jminhq (1,1), 
     +                 hq_p(1,1), hq_f(1,1), 
     +                 jhq_n(1,1), jhq_a(1,1), iret )
                  IF  ( iret .LT. 0 )  THEN
                     active = .FALSE.
                  ELSE
                     GO TO 20
                  END IF
               END IF
            END IF
            
         END IF
         


C     Done looping through each HQ satellite.  If at least one
C     is still active and the next IR exists, get the next IR
C     field.  This is the end of the main accumulation loop.
C     First, audit the 3-hly HQ for ambiguous, dump them out,
C     and reset the accumulation registers.
C     
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( ncum3 (i, j) .GT. 0 )  THEN
                  IF  ( FLOAT ( ncuma3 (i, j) ) / 
     +                 FLOAT ( ncum3  (i, j) ) .LT. .4 )  THEN
                     pcum3 (i, j) = pcum3 (i, j) / FLOAT (ncum3 (i, j))
                  ELSE
                     pcum3 (i, j) = RMISS
                  END IF
               ELSE
                  pcum3 (i, j) = RMISS
               END IF
            END DO
         END DO
         
C     Build a temporary basename by appending the (presumed) date 
C     in the IR filename and the hour number to the basename, and 
C     dump out 3-hrly HQ field.
C     
         tempfile=iname(iir)
         do l_ir = 1,len_trim(tempfile)
            if (tempfile(l_ir:l_ir) .eq. ' ') then
               continue
            end if
         end do
         l_ir=l_ir-1
         
         do l_fbas = 1,len_trim(fbas)
            if (fbas(l_fbas:l_fbas) .eq. ' ') then
               continue
            end if
         end do
         l_fbas=l_fbas-1
         
         do strlen = 1,len_trim(InterimPATH)
            if (InterimPATH(strlen:strlen) .eq. ' ') then
               continue
            end if
         end do
         strlen=strlen-1

         WRITE ( c_hr, 910 )  ( ihr - 1 ) * 3
 910     FORMAT ( I2.2 )
         tbas = fbas(1:l_fbas)
     +        //'.'//tempfile (l_ir-7:l_ir) // c_hr
         
         do strlen = 1,len_trim(tbas)
            if (tbas(strlen:strlen) .eq. ' ') then
               continue
            end if
         end do
         strlen=strlen-1

         fname=tbas(1:strlen)//'.pcum3'
         CALL Write_Binary_File(fname, pcum3, LN, LT)
         
         
         DO  j = 1, LT
            DO  i = 1, LN
               pcum3  (i, j) = 0.
               ncum3  (i, j) = 0
               ncuma3 (i, j) = 0
            END DO
         END DO
         
         
         IF  ( active )  THEN
            CALL GET_IRTB ( iname,  nir,    iir, ihr, jyrir, 
     +           jdayir, jminir, ltb, lza, igeop, leop,   iret )
            IF  ( iret .NE. 0 )  STOP
         END IF
         
c     End major do loop
      END DO
      
      




C     Convert the total and matched IR Tb histograms to cumulative, 
C     then compute averages in all fields.  ntbtot and ntbcum are left 
C     as scaled integer so more huge arrays aren't needed.  Also, 
C     compute averages from the GPROF summations, using the 20% 
C     threshold on ambiguous pixels to screen out possible artifacts.

      DO  j = 1, LT
         DO  i = 1, LN
            DO  it = 190, 270
               ntbtot (it, i, j) = ntbtot (it,   i, j) + 
     +              ntbtot (it-1, i, j)
            END DO
            IF  ( ntbtot (270, i, j) .GT. 0 )  THEN
               DO  it = 189, 270
                  ntbtot (it, i, j) = 10000. * 
     +                 FLOAT ( ntbtot (it,  i, j) ) / 
     +                 FLOAT ( ntbtot (270, i, j) )
               END DO
            ELSE
               DO  it = 189, 270
                  ntbtot (it, i, j) = -1
               END DO
            END IF
         END DO
      END DO
      DO  j = 1, LT
         DO  i = 1, LN
            DO  it = 190, 270
               ntbcum (it, i, j) = ntbcum (it,   i, j) + 
     +              ntbcum (it-1, i, j)
            END DO
            IF  ( ntbcum (270, i, j) .GT. 0 )  THEN
               DO  it = 189, 270
                  ntbcum (it, i, j) = 10000. * 
     +                 FLOAT ( ntbcum (it,  i, j) ) / 
     +                 FLOAT ( ntbcum (270, i, j) )
               END DO
            ELSE
               DO  it = 189, 270
                  ntbcum (it, i, j) = -1
               END DO
            END IF
         END DO
      END DO
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ncum (i, j) .GT. 0 )  THEN
               IF  ( FLOAT ( ncuma (i, j) ) / 
     +              FLOAT ( ncum  (i, j) ) .LT. .2 )  THEN
                  pcum (i, j) = pcum (i, j) / ncum (i, j)
                  fcum (i, j) = fcum (i, j) / ncum (i, j)
               ELSE
                  pcum (i, j) = RMISS
                  fcum (i, j) = RMISS
               END IF
            ELSE
               pcum (i, j) = RMISS
               fcum (i, j) = RMISS
            END IF
         END DO
      END DO
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ncums (i, j) .GT. 0 )  THEN
               IF  ( FLOAT ( ncumas (i, j) ) / 
     +              FLOAT ( ncums  (i, j) ) .LT. .2 )  THEN
                  fcums (i, j) = fcums (i, j) / ncums (i, j)
               ELSE
                  fcums (i, j) = RMISS
               END IF
            ELSE
               fcums (i, j) = RMISS
            END IF
         END DO
      END DO
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( npdf (i, j) .GT. 0 )
     +           CALL SORT ( pdf (1, i, j), npdf (i, j), pdf (1, i, j),
     +           iret )
         END DO
      END DO
      

C     Smooth if requested.
      IF  ( nsm .NE. 0 )  THEN
C     Dump out fields before smoothing.  The sample IR histograms
C     are the northern-most 5 boxes along the date line.
         
c         DO  j = 1, 5
C            WRITE (*, 996) j, (ntbtot (it, 180, j), it = 189, 270) 
C 996        FORMAT ( 'orig. ntbtot j =', I3, /, 6(15I5, /) )
c         END DO
c         DO  j = 1, 5
C            WRITE (*, 997) j, (ntbcum (it, 180, j), it = 189, 270) 
C 997        FORMAT ( ' orig ntbcum j =', I3, /, 6(15I5, /) )
c         END DO
         
         fname=fbas(1:l_fbas)//'.pcumu'
         CALL Write_Binary_File(fname, pcum, LN, LT)
         fname=fbas(1:l_fbas)//'.fcumu'
         CALL Write_Binary_File(fname, fcum, LN, LT)
         fname=fbas(1:l_fbas)//'.fcumsu'
         CALL Write_Binary_File(fname, fcums, LN, LT)
         

C         Compute the adjusted microwave fractional coverage (even if
C         it's just a pass-through of 1.,0.).  adfrat,adfcon units are
C         ratio,fraction (not %).  The first IF avoids processing the
C         missing special value and the nested if prevents adjusting
C         the fraction to a negative.
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( fcum  (i, j) .GE. 0. )  THEN
                  fcum  (i, j) = fcum  (i, j) * adfrat + adfcon
                  IF  ( fcum  (i, j) .LT. 0. )  fcum  (i, j) = 0.
               END IF
               IF  ( fcums (i, j) .GE. 0. )  THEN
                  fcums (i, j) = fcums (i, j) * adfrat + adfcon
                  IF  ( fcums (i, j) .LT. 0. )  fcums (i, j) = 0.
               END IF
            END DO
         END DO
         fname=fbas(1:l_fbas)//'.fcumad'
         CALL Write_Binary_File(fname, fcum, LN, LT)
         fname=fbas(1:l_fbas)//'.fcumsad'
         CALL Write_Binary_File(fname, fcums, LN, LT)

         CALL SMHIST4 ( ntbtot, 270-188, LN, LT, 4, ntbtot, iret1 )
         CALL SMHIST4 ( ntbcum, 270-188, LN, LT, 4, ntbcum, iret2 )
         CALL SMOOTH  ( nsm, smth, 4.0, pcum,  LN, LT, pcum,  iret3 )
         CALL SMOOTH  ( nsm, smth, 4.0, fcum,  LN, LT, fcum,  iret4 )
         CALL SMOOTH  ( nsm, smth, 4.0, fcums, LN, LT, fcums, iret5 )
         IF  ( iret1 .NE. 0 .OR. iret2 .NE. 0 .OR. iret3 .NE. 0 .OR.
     +        iret4 .NE. 0 .OR. iret5 .NE. 0 )  STOP
          
C        Dump out sample IR histograms after smoothing, northern-
C        most 5 boxes along the Date Line.
c         DO  j = 1, 5
C            WRITE (*, 998) j, (ntbtot (it, 180, j), it = 189, 270) 
C 998        FORMAT ( 'smoothed ntbtot j =', I3, /, 6(15I5, /) )
c         END DO
c         DO  j = 1, 5
C            WRITE (*, 999) j, (ntbcum (it, 180, j), it = 189, 270) 
C 999        FORMAT ( 'smoothed ntbcum j =', I3, /, 6(15I5, /) )
c         END DO

      END IF


C     If mname is not 'none', dump the current precip accumulation,
C     read in the 2.5x2.5-deg monthly global precip (in mm/mo), and
C     convert to 1x1-deg monthly on the tropical strip in mm/hr in 
C     the right array.
      
      do strlen = 1,len_trim(mname)
         if (mname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      
      IF  ( mname(1:strlen) .NE. 'none' )  THEN
                  
c        Write out current precip accumulation
         fname=fbas(1:l_fbas)//'.gprofcum'
         CALL Write_Binary_File(fname, pcum, LN, LT)
                  
c        Read monthly CDR data from mname
         CALL Read_Binary_File (mname, 144, 72, 1, p_mon)

c        Use bilinear interpolation to interpolate 2.5deg grid to 1deg
         CALL BILIN ( p_mon, 144,    72, 0., 20.,  360, 80, 2.5, 
     +        2.5,   .TRUE., .TRUE., pcum, iret )

C         Also, when a monthly accumulation is brought in, smooth-fill
C         HQ fractional coverage and then mask it by the new pcum.
C         This permits more realistic results in blanks in the HQ 
C         compared to smooth-filling rate.
C
         npass = -1
         CALL FILLSM  ( fcum,  LN, LT, LN, 0, npass, fcum,  iret )
         npass = -1
         CALL FILLSM  ( fcums, LN, LT, LN, 0, npass, fcums, iret )
      END IF
      
C     Find the IR Tb that gives the fractional coverage that's the
C     closest to that of the HQ.  Matching the coldest bin means 
C     you get that bin.  The warmest bin allowed is 269 to prevent 
C     100% coverage in the IR.  Then compute the volume rain rate 
C     using pcum and coverage in the total IR histograms.  This should
C     be comparable to using the coverage of the matched IR threshold 
C     bin at intermediate bin values, ensures that the monthly sum of
C     daily values is close to pcum, and prevents weirdness at very 
C     high or low values of bin or fractional coverage.  Set a 
C     fractional coverage floor of ftot=.0002, based on inspection of 
C     values for very small rain accumulations (approx. 10 min/month) 
C     to allow correct adjustments to rate and ithrsh further down.  
C     Likewise, rate is given a small, non-zero floor.
C
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( fcum (i, j) .NE. RMISS )  THEN
               
C              The usual case, have both fcum and ntbcum.
               IF  ( ntbcum (270, i, j) .GT. 0 )  THEN
 
c     Sapiano: added this code to remove loops, but it is buggy
c              decided to go back to loops.
c                  test=0
c                  DO  it = 189, 269
c                     IF  ( ntbcum (it, i, j) * .0001 .GT.                      
c     +                    fcum   (    i, j) )  THEN
c                        test=1
c                        continue
c                     end if 
c                  END DO
c                  if (test .EQ. 0) then
c                     it = 269
c                     write(*,*) ' Warning: In i,j=', i, ',', j, 
c     +                    ' couldn''t match fcum=', fcum (i,j),
c     +                    '; setting threshold to 269K' 
c                  end if

                  DO  it = 189, 269
                     IF  ( ntbcum (it, i, j) * .0001 .GT.
     +                    fcum   (    i, j) )  GO TO 30
                  END DO
c                  WRITE (*, 28) i, j, fcum (i,j)
 28               FORMAT ( ' Warning: In i,j=', I3, ',', I3,
     +                 ' couldn''t match fcum=', F5.3,
     +                 '; setting threshold to 269K' )
                  it = 269
 30               CONTINUE
                  
                  IF  ( it .EQ. 189 )  THEN
                     ithrsh (i, j) = it
                  ELSE IF  ( ( ntbcum (it,   i, j) * .0001 - 
     +                    fcum   (      i, j) ) .GT.
     +                    ( fcum   (      i, j) - 
     +                    ntbcum (it-1, i, j) * .0001 ) )  THEN
                     ithrsh (i, j) = it - 1
                  ELSE
                     ithrsh (i, j) = it
                  END IF
                  ftot (i, j) = AMAX1 ( .0002, .0001 *
     +                 ntbtot (ithrsh (i, j), i, j) )
                  rate (i, j) = AMAX1 ( .1, pcum (i, j) / ftot (i, j))
                  

C                Cold land drop-outs in HQ.  Have pcum (from SG or HQ) 
C                and fcum (from smooth-filling fcum; when not doing 
C                smooth-fill the IF just inside the i,j DOs has already
C                skipped this whole treatment), but not ntbcum 
C                (from coincidence with HQ), so use ntbtot.                  
                  
               ELSE IF  ( ntbtot (270, i, j) .GT. 0 )  THEN
                  DO  it = 189, 269
                     IF  ( ntbtot (it, i, j) * .0001 .GT.
     +                    fcum   (    i, j) )  GO TO 40
                  END DO
c                  WRITE (*, 38) i, j, fcum (i,j)
 38               FORMAT ( ' Warning: In i,j=', I3, ',', I3,
     +                 ' couldn''t match fcum=', F5.3,
     +                 '; setting threshold to 269K' )
                  it = 269
 40               CONTINUE
                  IF  ( it .EQ. 189 )  THEN
                     ithrsh (i, j) = it
                  ELSE IF  ( ( ntbtot (it,   i, j) * .0001 -
     +                    fcum   (      i, j) ) .GT.
     +                    ( fcum   (      i, j) -
     +                    ntbtot (it-1, i, j) * .0001 ) )  THEN
                     ithrsh (i, j) = it - 1
                  ELSE
                     ithrsh (i, j) = it
                  END IF
                  ftot (i, j) = AMAX1 ( .0002, .0001 *
     +                 ntbtot (ithrsh (i, j), i, j) )
                  rate (i, j) = AMAX1 ( .1, pcum (i, j) / ftot (i, j) )
                  
C              fcum present, but both ntbcum and ntbtot are missing,
C              i.e., no geo-IR for the month.
               ELSE
                  ithrsh (i, j) = -999
                  ftot   (i, j) = RMISS
                  rate   (i, j) = RMISS
               END IF
               
C              fcum is missing.
            ELSE
               ithrsh (i, j) = -999
               ftot   (i, j) = RMISS
               rate   (i, j) = RMISS
            END IF
         END DO
      END DO
      

C     Dump out fields before auditing and filling rate.
      fname=fbas(1:l_fbas)//'.thr1'
      CALL Write_Binary_File_int4(fname, ithrsh, LN, LT)
      fname=fbas(1:l_fbas)//'.rte1'
      CALL Write_Binary_File(fname, rate, LN, LT)
      fname=fbas(1:l_fbas)//'.ftot1'
      CALL Write_Binary_File(fname, ftot, LN, LT)
      fname=fbas(1:l_fbas)//'.fcum1'
      CALL Write_Binary_File(fname, fcum, LN, LT)

C     Audit the rate field, throwing out the ones that are "too big,"
C     where size seems best defined by reference to IR threshold.  
C     Colder thresholds are associated with higher rates.  Subjective
C     inspection of the 9801 threshold-rate diagram based on 3x3-
C     smoothed data suggests a cut-off running from 10 mm/hr at 210K 
C     to 2.5 mm/hr at 270K (but work in mm/d).  Rate is re-set in 
C     preparation for smooth-filling; ithrsh is given the special 
C     value 188 in preparation for re-estimation.
C
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ithrsh (i, j) .GE. 189 )  THEN
               IF  ( rate (i, j) .GT. 
     +              240. - 3. * ( ithrsh (i, j) - 210. ) )  THEN
                  rate   (i, j) = RMISS
                  ithrsh (i, j) = 188
               END IF
            END IF
         END DO
      END DO
      
C     Smooth-fill rate to get values in the audited regions (also
C     fills ambiguous regions when matched HQ is used for pcum).  
C     This also puts values in the no-data areas, but they won't ever 
C     be used.  Now work backwards at the audited points (flagged by 
C     ithrsh=188) by re-estimating fractional coverage as pcum/rate 
C     and pick a new threshold.

      npass = -1
      CALL FILLSM  ( rate, LN, LT, LN, 0, npass, rate, iret )
      IF  ( iret .EQ. -1 )  THEN
         WRITE (*, *) 'Error: check smoother size in FILLSM'
         STOP
      END IF
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ithrsh (i, j) .EQ. 188 )  THEN
               ftot (i, j) = pcum (i, j) / rate (i, j)
               DO  it = 189, 269
                  IF  ( ntbtot (it, i, j) * .0001 .GT. ftot (i, j) )
     +                 GO TO 50
               END DO
c               WRITE (*, 48) i, j, ftot (i,j)
 48            FORMAT ( ' Warning: In i,j=', I3, ',', I3,
     +              ' couldn''t match ftot=', F5.3,
     +              '; setting threshold to 269K' )
               it = 269
 50            CONTINUE
               IF  ( it .EQ. 189 )  THEN
                  ithrsh (i, j) = it
               ELSE IF  ( ( ntbtot (it, i, j) * .0001 -
     +                 ftot (i, j) ) .GT.
     +                 ( ftot (i, j) -
     +                 ntbtot (it-1, i, j) * .0001 ) )  THEN
                  ithrsh (i, j) = it - 1
               ELSE
                  ithrsh (i, j) = it
               END IF
               ftot (i, j) = ntbtot (ithrsh (i, j), i, j) * .0001
            END IF
         END DO
      END DO

      
C     Smooth-fill ithrsh to ensure values everywhere.  When using 
C     HQ pcum this fills in the ambiguous regions.  This also puts
C     values in the no-data areas, but they won't ever be used.  
C     ithrsh is integer, so it has to be converted to real to do the 
C     interpolation, then converted back afterwards.

      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ithrsh (i, j) .GE. 0 )  THEN
               thrsh (i, j) = ithrsh (i, j)
            ELSE
               thrsh (i, j) = RMISS
            END IF
         END DO
      END DO
      npass = -1
      CALL FILLSM  ( thrsh, LN, LT, LN, 0, npass, thrsh, iret )
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ithrsh (i, j) .LT. 189 )  
     +           ithrsh (i, j) = NINT ( thrsh (i, j) )
         END DO
      END DO
      
C     Now compute leo-IR revised zero point and new scaling.  [We 
C     could do PDF matching on freq. of raining pixels, since GPI/3.
C     is fractional coverage, and fcums could be like fcum.]  Here,
C     ngstrt is the cum. dist. of coldest geo-IR Tb (at the upper
C     bin boundary), fcums is the (pixel-weighted) fraction of HQ 
C     gridboxes with non-zero rain, while the PDF has gridbox average 
C     leo-GPI.  Convert the histogram of coldest geo-IR Tb to 
C     cumulative, scaled to 10000 so more huge arrays aren't needed.
C     All images with some geo-IR pixels no warmer than ithrsh will
C     be "rainy," so the cum. coldest-Tb at ithrsh gives the fraction
C     of rainy images.  z_pt and ratl temporarily hold the fraction of 
C     rainy geo-IR images and the ratio of geo-IR to HQ images,
C     respectively.

      DO  j = 1, LT
         DO  i = 1, LN
            DO  it = 190, 270
               ngstrt (it, i, j) = ngstrt (it,   i, j) + 
     +              ngstrt (it-1, i, j)
            END DO
            IF  ( ngstrt (270, i, j) .GT. 0 )  THEN
               DO  it = 189, 270
                  ngstrt (it, i, j) = 10000. * 
     +                 FLOAT ( ngstrt (it,  i, j) ) / 
     +                 FLOAT ( ngstrt (270, i, j) )
               END DO
            END IF
         END DO
      END DO
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ngstrt (270, i, j) .LE. 0   .OR. 
     +           ithrsh (i, j)      .LT. 189 )  THEN
               z_pt (i, j) = RMISS
            ELSE
               z_pt (i, j) = .0001 * ngstrt (ithrsh (i, j), i, j)
            END IF
         END DO
      END DO
      
C     Smooth z_pt if requested and then compute the ratio of geo-IR
C     to HQ rainy images.
      IF  ( nsm .NE. 0 )  THEN
         CALL SMOOTH  ( nsm, smth, 4.0, z_pt,  LN, LT, z_pt, iret )
         IF  ( iret .NE. 0 )  STOP
      END IF
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( z_pt  (i, j) .EQ. RMISS .OR. 
     +           fcums (i, j) .EQ. 0. )  THEN
               ratl (i, j) = RMISS
            ELSE
               ratl (i, j) = z_pt (i, j) / fcums (i, j)
               IF  ( ratl (i, j) .LT. 0.4 )  ratl (i, j) = 0.4
               IF  ( ratl (i, j) .GT. 2.5 )  ratl (i, j) = 2.5
            END IF
         END DO
      END DO
      
C     Smooth-fill the rainy image ratio and use with HQ in no-geo-IR 
C     areas (mostly the INSAT sector before July 1998) to try to 
C     capture variation across large regions, then dump the fields
      npass = -1
      CALL FILLSM  ( ratl, LN, LT, LN, 0, npass, ratl, iret )
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( z_pt  (i, j) .EQ. RMISS .AND. 
     +           ratl  (i, j) .NE. RMISS .AND.
     +           fcums (i, j) .NE. RMISS )  
     +           z_pt (i, j) = ratl (i, j) * fcums (i, j)
         END DO
      END DO
      
      fname=fbas(1:l_fbas)//'.fgim'
      CALL Write_Binary_File(fname, z_pt, LN, LT)
      fname=fbas(1:l_fbas)//'.ratgs'
      CALL Write_Binary_File(fname, ratl, LN, LT)
      
C     Points with "no HQ, or no pcum, or no leo" data are set to 
C     missing and smooth-filled below.  The (rounded) index of the 
C     lowest non-zero leo-IR rate that's allowed is m.  If it's npdf, 
C     there's only 0 or 1 value, implying that the calculation is 
C     unstable, so set to RMISS and smooth-fill below.  If it's 0 or 
C     1, all rates are allowed and z_pt is automatically zero.  For 
C     all other points, z_pt is the value of the next smaller (sorted)
C     pdf value.  When z_pt is non-missing (last 2 cases), accumulate 
C     the leo excesses over the zero point value for allowed non-zero 
C     rates and then scale as the ratio of pcum to average leo excess.
C     Note that it is possible for some of the allowed non-zero values
C     to have the z_pt value and therefore be rescaled to zero.
C     
      WRITE (*, *) 'Ready for leo-IR zpt and scaling.'
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( fcums (i, j) .LT. 0. .OR. pcum (i, j) .LT. 0. .OR.
     +           npdf  (i, j) .LE. 0  .OR. z_pt (i, j) .LT. 0.)  THEN
               z_pt (i, j) = RMISS
               ratl (i, j) = RMISS
            ELSE
               m = NINT ( npdf (i, j) * ( 1. - z_pt (i, j) ) )
               IF  ( m .EQ. npdf (i, j) )  THEN
                  z_pt (i, j) = RMISS
                  ratl (i, j) = RMISS
               ELSE
                  IF  ( m .LE. 1 )  THEN
                     z_pt (i, j) = 0.
                  ELSE
                     z_pt (i, j) = pdf (m-1, i, j)
                  END IF
                  ratl (i, j) = 0.
                  DO  m = 1, npdf (i, j)
                     IF  ( pdf (m, i, j) .GT. z_pt (i, j) )
     +                    ratl (i, j) = ratl (i, j) + pdf (m, i, j) - 
     +                    z_pt (i, j)
                  END DO
                  IF  ( ratl (i, j) .GT. 0. )  THEN
                     ratl (i, j) = pcum (i, j) *  npdf (i, j) /
     +                    ratl (i, j)
                  ELSE
                     ratl (i, j) = RMISS
                  END IF
               END IF
            END IF
         END DO
      END DO
      
C     Wrap up leo-IR by clipping the ratios to avoid maximum 
C     rainrates>150 mm/d.  Tests show that these all come in regions 
C     where geo-IR max is 50-100 mm/d.  Then smooth-fill to ensure 
C     values everywhere.
C
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( npdf  (i, j) .GT. 0 )  THEN
               IF  ( pdf (npdf (i, j), i, j) * ratl (i, j) .GT. 
     +              150. )  THEN
                  ratl (i, j) = 150. / pdf (npdf (i, j), i, j)
               END IF
            END IF
         END DO
      END DO
      npass = -1
      CALL FILLSM  ( z_pt, LN, LT, LN, 0, npass, z_pt, iret )
      npass = -1
      CALL FILLSM  ( ratl, LN, LT, LN, 0, npass, ratl, iret )



C     Dump out fields.
      fname=fbas(1:l_fbas)//'.thr'
      CALL Write_Binary_File_int4(fname, ithrsh, LN, LT)
      fname=fbas(1:l_fbas)//'.pcum'
      CALL Write_Binary_File(fname, pcum, LN, LT)
      fname=fbas(1:l_fbas)//'.ftot'
      CALL Write_Binary_File(fname, ftot, LN, LT)
      fname=fbas(1:l_fbas)//'.ncum'
      CALL Write_Binary_File_int4(fname, ncum, LN, LT)
      fname=fbas(1:l_fbas)//'.acum'
      CALL Write_Binary_File_int4(fname, ncuma, LN, LT)
      fname=fbas(1:l_fbas)//'.rte'
      CALL Write_Binary_File(fname, rate, LN, LT)
      fname=fbas(1:l_fbas)//'.fcums'
      CALL Write_Binary_File(fname, fcums, LN, LT)
      fname=fbas(1:l_fbas)//'.zptl'
      CALL Write_Binary_File(fname, z_pt, LN, LT)
      fname=fbas(1:l_fbas)//'.ratl'
      CALL Write_Binary_File(fname, ratl, LN, LT)
      

      STOP
      end program Main_GPCPcdr_d4_corr_g
