      SUBROUTINE GET_GORB ( sname, jyrss, jdayss, 
     +     jminss, ss_p,  ss_f,  jss_n, jss_a, iret )
C***********************************************************************
C* GET_GORB  Get the next GPROF orbit file in the list
C*
C*                                                       *
C* This subroutine gets the next GPROF 0.5x0.5-deg orbit file, and     *
C* averages the 40N-S band to 1x1-deg for output.                   *
C*                                                       *
C* GET_GORB ( sname, jyrss, jdayss, jminss, ss_p, ss_f,  *
C*            jss_n, jss_a, iret )                               *
C*                                                       *
C* Input parameters:                                           *
C*   sname      CHAR*      File name of GPROF orbit file
C*
C*                                                       *
C* Output parameters:                                           *
C*   jyrss      INT      4-digit year                               *
C*   jdayss      INT      Julian day                               *
C*   jminss      INT*2      Minutes into day (size LN,LT)
C*
C*   ss_p      REAL      Precip (size LN,LT)                         *
C*   ss_f      REAL      Fractional coverage by precip (size LN,LT)
C*
C*   jss_n      INT*2      Number of high-res pixels (size LN, LT)
C*
C*   jss_a      INT*2      Number of ambiguous pixels (size LN, LT)
C*
C*   iret      INT      Return code: 0 = normal
C*
C**                                                       *
C* Log:                                                       *
C* G.Huffman/SSAI       6/98                                     *
C* G.Huffman/SSAI       6/98      New GPROF file format; re-order loops
C*
C* G.Huffman/SSAI       5/00      Add land/oc. hist. corr.; fix msg
C*
C*                        time const; jss_n,jss_a to I*2             *
C* D.Bolvin/SSAI       3/08      Compatibility with latest compopts.f
C*
C* D.Bolvin/SSAI       8/12      Remove never-used fields from the
C*
C*                        file read                         *
C* D.Bolvin/SSAI       1/13      Port to LINUX; swap bytes on MW data
C*
c  Sapiano/CICS          4/14   Substantial edits for CDR project
C***********************************************************************

      IMPLICIT      NONE
      
      INTEGER*4      LN, LT, MXBIN
      INTEGER*2      I2MISS
      REAL*4            RMISS
      PARAMETER      ( LN     = 360,
     +     LT     = 80,
     +     MXBIN  = 10000,
     +     RMISS  = -99999.,
     +     I2MISS = -31999 )
      
      CHARACTER*256 sname
      logical opn
      integer strlen, inunit, ios
      integer i, j, ii, jj
      
c     SSMI data upon input
      integer*4 istart (6)
      INTEGER*2      in_p(360, 720)
      BYTE            in_n(360, 720),  in_t(360, 720), 
     +     in_a(360, 720),  in_r(360, 720)
      
c     Interim data
      REAL*4        p(360, 720) 
      INTEGER*2     jminss(LN, LT)
      INTEGER*4     iyy, idate, imin

c     Outputs
      INTEGER*4     jyrss, jdayss, iret
      REAL*4        ss_p(LN, LT), ss_f(LN, LT)
      INTEGER*2     jss_n(LN, LT), jss_a(LN, LT)




c     Check that 'no more files' (999) isn't in the file name
      do strlen = 1,len_trim(sname)
         if (sname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      
      if ( sname(1:strlen) .EQ. '999' ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'File name is 999'
         iret=-1
         return
      end if
      
C     check for a free iounit
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

      
c     Open SSMI file
      open(inunit, FILE=sname(1:strlen), FORM='UNFORMATTED', 
     +     STATUS='OLD', IOSTAT=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to open input SSMI file: '//sname(1:strlen)
         RETURN
      end if
      
      
C     The actual read of the data, which are unformatted sequential,
C     as 0.5x0.5-deg grids transposed to 360x720 for better 
C     compression.  There are 9 fields total:
C     
C     Param.      |   Var.  |     Units      | Type  |  Size   
C     ------------------+---------+----------------+-------+--------
C     Orb. start time   | istart  | YMDhms (UTC)   | INT*4 | 6       
C     Time after start  | jminss  | min            | INT*1 | 360x720 
C     Sfc rain rate     | sfcrain | mm/hr * 100    | INT*2 | 360x720 
C     # pixels          | ncount  | 12.5-km pixels | INT*1 | 360x720 
C     # ambig. pixels   | nambig  | 12.5-km pixels | INT*1 | 360x720 
C     # raining pixels  | nrain   | 12.5-km pixels | INT*1 | 360x720 
C     
      
      read(inunit,IOSTAT=ios) istart
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to read from SSMI file: '//sname(1:strlen)
         RETURN
      end if
      
      read(inunit,IOSTAT=ios) in_t
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to read from SSMI file: '//sname(1:strlen)
         RETURN
      end if
      
      read(inunit,IOSTAT=ios) in_p
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to read from SSMI file: '//sname(1:strlen)
         RETURN
      end if
      
      read(inunit,IOSTAT=ios) in_n
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to read from SSMI file: '//sname(1:strlen)
         RETURN
      end if
      
      read(inunit,IOSTAT=ios) in_a
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to read from SSMI file: '//sname(1:strlen)
         RETURN
      end if
      
      read(inunit,IOSTAT=ios) in_r
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to read from SSMI file: '//sname(1:strlen)
         RETURN
      end if

C     Done - don't need last 3 fields, so they don't have to be read.
      close(inunit, IOSTAT=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in get_gorb:'//
     &        'Failed to close input SSMI file: '//sname(1:strlen)
         RETURN
      end if
            
      
      
C     Convert to the real precip in the region of interest
      DO  i = 1,   720
         DO  j = 101, 260
            IF  ( in_t (j, i) .GE. 0 )  THEN
               p (j, i) = .01 * in_p (j, i)
            ELSE
               p (j, i) = RMISS
            END IF
         END DO
      END DO


c     Adjust rain using land/ocean cum. histos
c      CALL PBCORR ( p, histo, histl, 100, 160, p, iret )
c      IF  ( iret .LT. 0 )  STOP
      
c     load the output variables.
      DO  j = 1, LT
         DO  i = 1, LN
            jminss (i, j) = 0
            ss_p   (i, j) = 0.
            jss_n  (i, j) = 0
            jss_a  (i, j) = 0
            ss_f   (i, j) = 0.

            DO  jj = j*2 + 99, j*2 + 100
               DO  ii = i*2 - 1,  i*2
                  IF  ( in_t (jj, ii) .GE. 0 )  THEN
                     jminss (i, j) = jminss (i,  j)  + 
     +                    (INT(in_t  (jj, ii)) * INT(in_n (jj, ii)))
                     ss_p   (i, j) = ss_p   (i,  j)  + 
     +                    p      (jj, ii) * in_n (jj, ii)
                     jss_n  (i, j) = jss_n  (i,  j)  + in_n (jj, ii)
                     jss_a  (i, j) = jss_a  (i,  j)  + in_a (jj, ii)
                     ss_f   (i, j) = ss_f   (i,  j)  + in_r (jj, ii)
                  END IF
               END DO
            END DO

            IF  ( jss_n  (i, j) .EQ. 0 )  THEN
               jminss (i, j) = -999
               ss_p   (i, j) = RMISS
               ss_f   (i, j) = RMISS
            ELSE
               jminss (i, j) = NINT ( FLOAT ( jminss (i, j) ) / 
     +              jss_n  (i, j) )
               ss_p   (i, j) = ss_p (i, j) / jss_n  (i, j)
               ss_f   (i, j) = ss_f (i, j) / jss_n (i, j)
            END IF
          END DO
      END DO
      




C     Set start date/time information, including getting Julian day.
C     The start time is assessed to the nearest minute, then jminss
C     is converted to minutes since the start of the day.  Adjust if
C     the rounding throws you into the next day.
      
      jyrss = istart (1)
      iyy   = MOD ( istart (1), 100 )
      idate = iyy * 10000 + istart (2) * 100 + istart (3)
      CALL JULDAY ( idate, jdayss )
      IF  ( istart (6) .LT. 30 )  THEN
         imin = istart (4) * 60 + istart (5)
      ELSE
         imin = istart (4) * 60 + istart (5) + 1
      END IF
      
      IF  ( imin .GT. 1439 )  THEN
         imin = imin - 1440
         IF  ( ( jdayss .EQ. 365 .AND. MOD ( istart (1), 4 ) .NE. 0 ) 
     +        .OR.
     +        ( jdayss .EQ. 366 .AND. MOD ( istart (1), 4 ) .EQ. 0 )
     +        )  THEN
            jyrss  = jyrss + 1
            jdayss = 1
         ELSE
            jdayss = jdayss + 1
         END IF
      END IF
      
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( jminss (i, j) .GE. 0 )  THEN
               jminss (i, j) = jminss (i,  j) + imin
            ELSE
               jminss (i, j) = I2MISS
            END IF
         END DO
      END DO
      
      
      RETURN
      END SUBROUTINE GET_GORB 
      
