      SUBROUTINE SORT ( arrin, nin, arrout, iret )
C************************************************************************
C* SORT Sort a 1-D array of REALs in ascending order
C*
C*
C*
C* This subroutine uses the Shell algorithm to sort a one-dimensional
C*
C* array of real numbers in ascending order.
C*
C*
C*
C* NOTE: This version seems to require input/output arrays of size
C*
C*       IGRDSQ*NMAX; use SORTO.F for a more general sizing.
C*
C*
C*
C* SORT ( arrin, nin, arrout, iret )
C*
C*
C*
C* Input Parameters:
C*
C*   arrin      REAL*   Input array of unsorted numbers
C*
C*   nin        INT     Number of input values
C*
C*
C*
C* Output Parameters:
C*
C*   arrout     REAL*   Output array containing sorted numbers
C*
C*   iret       INT     Return code: 0 = normal
C*
C*                                  -1 = nin outside [1,NMAX*IGRDSQ]
C*
C**
C*
C* Log:
C*
C* D.Bolvin/SSAI         8/96
C*
C* G.Huffman/SSAI        3/99   Generalize
C*
C* G.Huffman/SSAI        7/06   Header note about array sizes; correct
C*
C*                              loading of input into output array (was
C*
C*                              backwards).
C*
C************************************************************************
      IMPLICIT        NONE
      INTEGER*4       n, ndelta, nswitch, max, i
      REAL*4          temp
C     
      INTEGER*4       IGRDSIZE, IGRDSQ, NMAX
      PARAMETER       ( IGRDSIZE = 3,
     +     IGRDSQ   = IGRDSIZE**2,
     +     NMAX     = 31 )
C     
      INTEGER*4       nin, iret
      REAL*4          arrin (IGRDSQ * NMAX), arrout (IGRDSQ * NMAX)
C     
C     Check dimension, initialize the output array, and set constants.
C     
      IF  ( nin .LT. 1 )  THEN
         WRITE (*, *) 'Error: nin =', nin, ' .LT. 1 in SORT'
         iret = -1
         RETURN
      ELSE IF  ( nin .GT. IGRDSQ * NMAX )  THEN
         WRITE (*, *) 'Error: nin =', nin, ' .GT.', IGRDSQ*NMAX,
     +        ' in SORT'
         iret = -1
      ELSE
         iret = 0
      END IF
      DO  i = 1, IGRDSQ * NMAX
         arrout (i) = arrin (i)
      END DO
      n      = nin
      ndelta = nin
C     
C     Do the sort by comparisons over distances that are progressively
C     halved.
C     
      DO  WHILE ( ndelta .GT. 1 )
         ndelta  = ndelta / 2
 100     nswitch = 0
         max     = n - ndelta
         DO  i = 1, max
            IF  ( arrout (i) .GT. arrout (i + ndelta) )  THEN
               temp               = arrout (i)
               arrout (i)          = arrout (i + ndelta)
               arrout (i + ndelta) = temp
               nswitch            = 1
            END IF
         END DO
         IF  (nswitch .NE. 0)  GO TO 100
      END DO
C     
      RETURN
      END
      
      
