      SUBROUTINE ZA_TB  ( LN,  LT,  LBIN, ltb, lza, 
     +     ipt, jpt, mpt,  out, ntbh, iret )
C************************************************************************
C* ZA_TB  Do the zenith-angle correction to IR Tb			*
C*                                                                      *
C* This subroutine computes the zenith-angle-corrected merged IR Tb's	*
C* held in the CPC 1-deg 3-hrly 24-bin format.  The zenith-angle-	*
C* correction coefficients are computed from data provided by Bob Joyce	*
C* (CPC).  Throughout the documentation ZA denotes "zenith angle."	*
C*									*
C* >> NOTE: The order of temperatures in the output is reversed from <<	*
C* >>       that in the input - it's low-to-high.                    <<	*
C*									*
C* ZA_TB  ( LN, LT, LBIN, ltb, lza, ipt, jpt, mpt, ntbh, iret )		*
C*									*
C* Input parameters:							*
C*   LN		INT	X dimension					*
C*   LT		INT	Y dimension					*
C*   LBIN	INT	Number of original limb bins (must be 24)	*
C*   ltb	INT*2	Original limb bins as fractional population*	*
C*			10000 (size LN,LT,LBIN,NHR)			*
C*   lza	INT*2	Zenith angle as deg.*10 (size LN,LT)		*
C*   ipt	INT	X for single-pt. plot (-1=n/a)			*
C*   jpt	INT	Y for single-pt. plot (-1=n/a)			*
C*   mpt	INT	HR index for single-pt. plot (-1=n/a)		*
C*   out	CHAR*	Output name for single-pt. plot ('none'=n/a)	*
C*									*
C* Output parameters:							*
C*   ntbh	INT*2	Nadir 1K bins as fractional population (size	*
C*			189:270,LN,LT)					*
C*   iret	INT	Return code:  1 = bad single-pt. input		*
C*				      0 = normal			*
C*				     -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 5/98						*
C* D.Bolvin/SSAI	 3/08	CHARACTER*80 -> CHARACTER*256    	*
C************************************************************************
	IMPLICIT	NONE
	CHARACTER*256	outlc
	REAL*4		t1, t2, avg1, avg2, avg3, del
	INTEGER*4	i, j, k, m, n, kh, lk, nk, it1, it2, n1, n2, 
     +			iunit, ios, i1, i2, j1, j2, lzadex,   l_out
	LOGICAL*1	opn
C
C	A leading "L" generally refers to limb, "N" to nadir.  A 
C	trailing "H" refers to 1K bins, which run 184-270 K for limb 
C	and 189-270 K for nadir.  The extra 5K are needed at limb to 
C	allow ZA-correction to around 190K at high ZA.  Bins are 
C	referenced by their lower bound temperature.
C
	INTEGER*4	LN, LT, LBIN, ipt, jpt, mpt, iret
	INTEGER*2	ltb  (         LN, LT, LBIN),
     +			lza  (         LN, LT      ),
     +			ntbh (189:270, LN, LT      )
	CHARACTER*(*)	out
C
	REAL*4		corr  (   184:270),
     +			cumf  (2, 184:270, 26:70)
	INTEGER*4	ltbh  (   184:270), 
     +			ltbhp (   184:270),
     +			idex1 (   184:270, 26:70)
c	STATIC		cumf, idex1
c	SAVE		cumf, idex1
C
	LOGICAL*1	init
	REAL*4		coef (6)
	DATA		init, coef 
     +			/ .FALSE.,        -37.3694765,  1.03633573,
     +			  -1.11555577E-2, 5.8242541E-5, -1.47441788E-7,
     +			  1.4507664E-10 /
C
C	Immediate sanity check that ltb has the expected number of bins.
C
	IF  ( LBIN .EQ. 24 )  THEN
	    iret = 0
	  ELSE
	    WRITE (*, *) 'Error: expected 24 coarse limb bins but got',
     +			 LBIN
	    iret = -1
	    RETURN
	END IF
C
C			>> CODE FOR THE FIRST TIME <<
C	Initialize the correction coefficient array if not previously
C	done.  Start by setting the correction factor (how much to warm
C	the limb Tb for each deg. of ZA past 25 deg) for each 1K 
C	temperature class using a 5th-order fit to Bob Joyce's 
C	intersatellite comparison.
C

c       MRPSapiano: Original code used STATIC (aka SAVE in gfortran) command
c       so that idex1 (and cumf) did not have to be calculated every time 
c       za_tb.f is called.  I am assuming that NCDC will not like SAVE command, 
c       so it is commented out and init is set to F, so that idex1 is recalculated 
c       at every call (only about 30 times, so not a huge deal).  The code is left 
c       unchanged otherwise so that this can be undone easily later.
        init = .FALSE.

 	IF  ( .NOT. init )  THEN
	    init = .TRUE.
	    DO  i = 184, 270
		corr (i) = coef (1) * 1.D0    + coef (2) * i**1.D0 + 
     +			   coef (3) * i**2.D0 + coef (4) * i**3.D0 +
     +			   coef (5) * i**4.D0 + coef (6) * i**5.D0
	    END DO
C
C	    Loop over all reasonable ZAs and 1K limb temperature
C	    classes.  Compute the adjusted temperatures and the start 
C	    of the corresponding bins.
C
	    DO  n = 26,  70
	    DO  m = 184, 270
		t1 = m + corr (m) * ( n - 25. )
		IF  ( m .NE. 270 )  THEN
		    t2 = m + 1 + corr (m+1) * ( n - 25. )
		  ELSE
		    t2 = t1 + .01
		END IF
		IF  ( t1 .GT. 270. )  t1 = 270.
		IF  ( t2 .GT. 270. )  t2 = 270.
		it1 = INT ( t1 )
		it2 = INT ( t2 )
C
C		CASE 1:  Both ends of 1K limb bin map to the same 1K 
C		nadir bin.
C
		IF  ( it1 .EQ. it2 )  THEN
		    idex1 (m, n)    = it1
		    cumf  (1, m, n) = 1.
		    cumf  (2, m, n) = 1.
		    IF  ( it1 .LT. 190 )  THEN
			idex1 (m, n) = 189
		      ELSE IF  ( it1 .GE. 270 )  THEN
			idex1 (m, n) = 270
		    END IF
C
C		  CASE 2:  Ends of 1K limb bin map to consecutive 1K 
C		  nadir bins.
C 
		  ELSE IF  ( it1 .EQ. it2 - 1 )  THEN
		    idex1 (m, n)    = it1
		    cumf  (1, m, n) = ( it1 + 1 - t1 ) / ( t2 - t1 )
		    cumf  (2, m, n) = 1.
		    IF  ( it1 .LT. 190 )  THEN
			idex1 (m, n) = 189
			IF  ( it2 .LT. 190 )  cumf (1, m, n) = 1.
		      ELSE IF  ( it1 .GE. 270 )  THEN
			idex1 (m, n)    = 270
			cumf  (1, m, n) = 1.
		    END IF
C
C		  CASE 3:  Ends of 1K limb bin map across 3 1K nadir 
C		  bins.  There shouldn't be any wider spread.
C 
		  ELSE
		    idex1 (m, n)    = it1
		    cumf  (1, m, n) = ( it1 + 1 - t1 ) / ( t2 - t1 )
		    cumf  (2, m, n) = 1. - 
     +				      ( t2 - it2 ) / ( t2 - t1 )
		    IF  ( it1 .LT. 190 )  THEN
			idex1 (m, n)    = 189
			cumf  (2, m, n) = 1.
			IF  ( it1 + 1 .LT. 190 )  THEN
			    IF  ( it2 .LT. 190 )  THEN
				cumf  (1, m, n) = 1.
			      ELSE
				cumf  (1, m, n) = cumf (2, m, n)
			    END IF
			    cumf  (2, m, n) = 1.
		        END IF
		      ELSE IF  ( it1 + 1 .GE. 270 )  THEN
			cumf (2, m, n) = 1.
			IF  ( it1 .GE. 270 )  THEN
			    idex1 (m, n)    = 270
			    cumf  (1, m, n) = 1.
			END IF
		    END IF
		END IF
	    END DO
	    END DO
	END IF
C
C				>> ROUTINE CODE <<
C	Set DO limits; key on ipt .LT. 0 to flag "not single-pt. plot."
C	In the case of a single-pt. plot check that the stated 
C	coordinates are in bounds.
C
	IF  ( ipt .LT. 0 )  THEN
	    j1 = 1
	    j2 = LT
	    i1 = 1
	    i2 = LN
	  ELSE
	    IF  ( ipt .LT. 1 .OR. ipt .GT. LN .OR. 
     +		  jpt .LT. 1 .OR. jpt .GT. LT  )  THEN
		WRITE (*, *) 'Error: single-pt. location out of bounds'
		WRITE (*, *) 'ipt =', ipt, '; range = 1,', LN
		WRITE (*, *) 'jpt =', jpt, '; range = 1,', LT
		iret = 1
		RETURN
	      ELSE
		j1 = jpt
		j2 = jpt
		i1 = ipt
		i2 = ipt
	    END IF
	END IF
C
C	Interpolate the original "limb" Tb bins to 1K bins.  Based on
C	code by R. Joyce (CPC/NWS).  Notes:
C	1. The temperature order is reversed to cold->warm.
C	2. The array indices are reordered to put bins first.
C	3. The interpolation is done down to 184K, but not above 270K, 
C	   since temperatures will shift *up*.
C
	DO  j = j1, j2
	DO  i = i1, i2
C
C	    First, check for no data.  Go ahead and load "no data" into 
C	    the output array while we're here.  In the current concept
C	    it's not necessary to fill ltbh, except for diagnostics.
C

           IF  ( ltb (i,j, 24) .LT. 0 )  THEN
              DO  kh = 184, 270
                 ltbh (kh) = -9999
              END DO
              DO  kh = 189, 270
                 ntbh (kh, i, j) = -9999
              END DO
C     
C	      Warmest bin transfers "as is."
C
           ELSE
              ltbh (270) = ltb (i,j, 1)
C
C		Then the next 6 bins are 5K wide and need to be 
C		interpolated.  Get the avg 1K bin value of the current 
C		and adjacent bins.  [The avg of the next 5K bin for bin 
C		7 is the avg of the next 5 1K bins.]
C
              DO  k = 2, 7
                 avg1 = ltb (i, j, k-1) * .2
                 avg2 = ltb (i, j,   k) * .2
                 IF  ( k .NE. 7 )  THEN
                    avg3 = ltb (i, j, k+1) * .2
                 ELSE
                    avg3 = ( ltb (i, j,  8) + ltb (i, j,  9) +
     +                   ltb (i, j, 10) + ltb (i, j, 11) +
     +                   ltb (i, j, 12) ) * .2
                 END IF
C     
C		    Compute a slope with avg1, avg3 centered on the 
C		    adjacent 5K bins (10K separation), but control the
C		    slope such that the end 1K bins get at least zero.
C		    The 1K populations are initially computed in 
C		    cumulative form to avoid rounding errors changing 
C		    the total (a=avg2, d=del).
C		      BIN          1      2       3       4     5
C		      BIN VALUE  a-2*d   a-d      a      a+d   a+2d
C		      CUM. VALUE a-2*d 2*a-3*d 3*a-3*d 4*a-2*d ltb
C		    We don't need to compute bin 5 in the cumulative,
C		    since it's just ltb, and we don't need to compute
C		    bin value 1 because it equals the cumulative value.
C
                 del = ( avg3 - avg1 ) * .1
                 IF  ( 2. * del .GT. avg2 )  THEN
                    del = avg2 * .5
                 ELSE IF  ( -2. * del .GT. avg2 )  THEN
                    del = avg2 * (-.5)
                 END IF
                 ltbh (279-5*k) = NINT (      avg2 - 2. * del )
                 ltbh (278-5*k) = NINT ( 2. * avg2 - 3. * del )
                 ltbh (277-5*k) = NINT ( 3. * avg2 - 3. * del )
                 ltbh (276-5*k) = NINT ( 4. * avg2 - 2. * del )
C     
                 ltbh (275-5*k) = ltb (i,j, k) - ltbh (276-5*k)
                 ltbh (276-5*k) = ltbh (276-5*k)  - ltbh (277-5*k)
                 ltbh (277-5*k) = ltbh (277-5*k)  - ltbh (278-5*k)
                 ltbh (278-5*k) = ltbh (278-5*k)  - ltbh (279-5*k)
              END DO
C     
C		The next 10 bins are 1K wide and transfer "as is."
C
              DO  k = 8, 17
                 ltbh (247-k) = ltb (i, j, k)
              END DO
C     
C		The next 4 bins are 5K wide and need to be interpolated 
C		(see above for detailed comments).  The preceeding 5 
C		bins are 1K, need to be summed; the next bin is 10K -
C		use 1/10 of the value and ascribe it to 2.5K from edge.
C
              DO  k = 18, 21
                 IF  ( k .NE. 18 )  THEN
                    avg1 = ltb (i, j, k-1) * .2
                 ELSE
                    avg1 = ( ltb (i, j, 13) + ltb (i, j, 14) +
     +                   ltb (i, j, 15) + ltb (i, j, 16) +
     +                   ltb (i, j, 17) ) * .2
                 END IF
                 avg2 = ltb (i, j, k) * .2
                 IF  ( k .NE. 21 )  THEN
                    avg3 = ltb (i, j, k+1) * .2
                 ELSE
                    avg3 = ltb (i, j, k+1) * .1
                 END IF
C     
C		    Compute a slope, cumulative, then bin values.
C
                 del = ( avg3 - avg1 ) * .1
                 IF  ( 2. * del .GT. avg2 )  THEN
                    del = avg2 * .5
                 ELSE IF  ( -2. * del .GT. avg2 )  THEN
                    del = avg2 * (-.5)
                 END IF
                 ltbh (319-5*k) = NINT (      avg2 - 2. * del )
                 ltbh (318-5*k) = NINT ( 2. * avg2 - 3. * del )
                 ltbh (317-5*k) = NINT ( 3. * avg2 - 3. * del )
                 ltbh (316-5*k) = NINT ( 4. * avg2 - 2. * del )
C     
                 ltbh (315-5*k) = ltb (i, j, k)  - ltbh (316-5*k)
                 ltbh (316-5*k) = ltbh (316-5*k) - ltbh (317-5*k)
                 ltbh (317-5*k) = ltbh (317-5*k) - ltbh (318-5*k)
                 ltbh (318-5*k) = ltbh (318-5*k) - ltbh (319-5*k)
              END DO
C     
C		The next 2 bins are 10K wide and need to be interpolated 
C		(see above for detailed comments).  The preceeding 2 
C		bins are 5K, need to be summed to provide the adjacent
C		bin; the final bin is semi-infinite, but is treated as 
C		size 10K.
C
              DO  k = 22, 24
                 IF  ( k .NE. 22 )  THEN
                    avg1 = ltb (i, j, k-1) * .1
                 ELSE
                    avg1 = ( ltb (i, j, 20) +
     +                   ltb (i, j, 21) ) * .1
                 END IF
                 avg2 = ltb (i, j, k) * .1
                 IF  ( k .NE. 24 )  THEN
                    avg3 = ltb (i, j, k+1) * .1
                 ELSE
                    avg3 = 0.
                 END IF
C     
C		    Compute a slope, cumulative, then bin values.
C		      B#    1      2        3          4         5
C		      B. a-4.5*d a-3.5*d   a-2.5*d   a-1.5*d   a-.5*d
C		      C. a-4.5*d 2*a-8*d 3*a-10.5*d 4*a-12*d 5*a-12.5*d
C		      B#    6         7         8        9       10
C		      B. a+.5*d    a+1.5*d   a+2.5*d  a+3.5*d  a+4.5*d
C		      C. 6*a-12*d 7*a-10.5*d 8*a-8*d 9*a-4.5*d   ltb
C		    Only 5 1K bins are split out of bin 24; the IF puts
C		    the rest in the 184 1K bin and skips down to 
C		    computing 1K bin values from 185 up.
C
                 del = ( avg3 - avg1 ) * .05
                 IF  ( 4.5 * del .GT. avg2 )  THEN
                    del = avg2 / 4.5
                 ELSE IF  ( -4.5 * del .GT. avg2 )  THEN
                    del = avg2 / (-4.5)
                 END IF
                 ltbh (429-10*k) = NINT (      avg2 -  4.5 * del )
                 ltbh (428-10*k) = NINT ( 2. * avg2 -  8.0 * del )
                 ltbh (427-10*k) = NINT ( 3. * avg2 - 10.5 * del )
                 ltbh (426-10*k) = NINT ( 4. * avg2 - 12.0 * del )
                 ltbh (425-10*k) = NINT ( 5. * avg2 - 12.5 * del )
                 IF  ( k .EQ. 24 )  THEN
                    ltbh (424-10*k) = ltb (i, j, k)
                 ELSE
                    ltbh (424-10*k) = NINT (  6.  * avg2 -
     +                   12.0 * del )
                    ltbh (423-10*k) = NINT (  7.  * avg2 -
     +                   10.5 * del )
                    ltbh (422-10*k) = NINT ( 8.  * avg2 -
     +                   8.0 * del )
                    ltbh (421-10*k) = NINT ( 9.  * avg2 -
     +                   4.5 * del )
C     
                    ltbh (420-10*k) = ltb  (i, j, k)  -
     +                   ltbh (421-10*k)
                    ltbh (421-10*k) = ltbh (421-10*k) - 
     +                   ltbh (422-10*k)
                    ltbh (422-10*k) = ltbh (422-10*k) -
     +                   ltbh (423-10*k)
                    ltbh (423-10*k) = ltbh (423-10*k) -
     +                   ltbh (424-10*k)
                 END IF
                 ltbh (424-10*k) = ltbh (424-10*k) - ltbh (425-10*k)
                 ltbh (425-10*k) = ltbh (425-10*k) - ltbh (426-10*k)
                 ltbh (426-10*k) = ltbh (426-10*k) - ltbh (427-10*k)
                 ltbh (427-10*k) = ltbh (427-10*k) - ltbh (428-10*k)
                 ltbh (428-10*k) = ltbh (428-10*k) - ltbh (429-10*k)
              END DO
C     
C		Zero the 1K nadir bins, then shift the 1K limb bins to 
C		1K nadir bins.  When ZA < 26 the bins transfer directly
C		(and note that lk=184-189 map to nk=189).  ZA > 90 is
C		an error, so we ignore the bin.  At higher ZA 
C		(bounded at the array limit of 70) there is a shift in 
C		temperature.  In that case the fraction array is 
C		cumulative so that rounding will not change the total 
C		population:  Compute the integer cumulative 
C		distribution, convert to bin values, and add.
C
              DO  nk = 189, 270
                 ntbh (nk, i, j) = 0
              END DO
              DO  lk = 184, 270
                 lzadex = NINT ( lza (i, j) * .1 )
                 IF  ( lzadex .LT. 26 )  THEN
                    nk = MAX ( 189, lk )
                    ntbh (nk, i, j) = ntbh (nk, i, j) + ltbh (lk)
                 ELSE IF  ( lzadex .GT. 90 )  THEN
                 ELSE
                    IF  ( lzadex .GT. 70 )  THEN
                       WRITE (*, *) 'Warning: at i,j =', i, j,
     +                      ' reset ZA =', lzadex, ' to 70'
                       lzadex = 70
                    END IF
                    nk = idex1 (lk, lzadex)
                    n1 = NINT ( ltbh (lk) * cumf  (1, lk, lzadex) )
                    n2 = NINT ( ltbh (lk) * cumf  (2, lk, lzadex) )
                    IF  ( nk + 2 .LE. 270 )  GO TO 121
                    IF  ( nk + 1 .LE. 270 )  GO TO 122
                    GO TO 123
 121                ntbh (nk + 2, i, j) = ntbh (nk + 2, i, j) + 
     +                   ltbh (lk) - n2
 122                ntbh (nk + 1, i, j) = ntbh (nk + 1, i, j) + 
     +                   n2 - n1
 123                ntbh (nk,     i, j) = ntbh (nk,     i, j) + n1
                 END IF

              END DO
C     
           END IF
c     c
c	    IF  ( i .EQ. 50 )  THEN
c		WRITE (*, *) 'i,j', i, j
c		WRITE (*, *) 'ltb',  ( ltb  (  50,1,k), k = 1,  24)
c		WRITE (*, *) 'ltbh', ( ltbh (       k), k = 184,270)
c		WRITE (*, *) 'ntbh', ( ntbh (k,50,1),   k = 189,270)
c	    END IF
cc
	END DO
      END DO
C
C		     >> CONTINUATION FOR SINGLE-POINT <<
C	Return if output isn't requested.  Otherwise, interpolate
C	the original histogram to 1K bins by pixel replication (again,
C	flipping temperature range end-for-end).  Do it on the range
C	[184,270] for compatibility with the real 1K limb bins.
C


c	CALL ST_UCLC ( out, outlc, iret )
c	CALL ST_LSTR ( out, l_out, iret )
c	IF  ( outlc (1:l_out) .EQ. 'none' )  RETURN
c	IF  ( ipt .LE. 0 )  THEN
c	    WRITE (*, *) 'Error: OUT isn''t ''none'', but ipt > 0'
c	    iret = -1
c	    RETURN
c	END IF
C
C	First, re-set indices (incremented leaving the loops above, 
C	even though they only executed once).  Then print a message 
C	and return if there's no data.
C
	i = ipt
	j = jpt
	IF  ( ltb (i, j, 1) .LT. 0 )  THEN
	    WRITE (*, *) 'The box ipt,jpt,mpt = ', ipt, ',', jpt, ',',
     +			 mpt, ' has a blank histogram'
	    RETURN
C
C	  Warmest bin transfers "as is."
C
	  ELSE
	    ltbhp (270) = ltb (i, j, 1)
C
C	    Then the next 6 bins are 5K wide and need to be pixel-
C	    replicated using the avg 1K bin value of the current bin.
C	    Do it in cumulative form to ensure that all samples are
C	    kept.  We don't need to compute bin 5 in the cumulative,
C	    since it's just ltb, and we don't need to compute bin value
C	    1 because it equals the cumulative value.

C
	    DO  k = 2, 7
		avg2            = ltb (i, j, k) * .2
		ltbhp (279-5*k) = NINT (      avg2 )
		ltbhp (278-5*k) = NINT ( 2. * avg2 )
		ltbhp (277-5*k) = NINT ( 3. * avg2 )
		ltbhp (276-5*k) = NINT ( 4. * avg2 )
C
		ltbhp (275-5*k) = ltb (i, j, k)   - ltbhp (276-5*k)
		ltbhp (276-5*k) = ltbhp (276-5*k) - ltbhp (277-5*k)
		ltbhp (277-5*k) = ltbhp (277-5*k) - ltbhp (278-5*k)
		ltbhp (278-5*k) = ltbhp (278-5*k) - ltbhp (279-5*k)
	    END DO
C
C	    The next 10 bins are 1K wide and transfer "as is."
C
	    DO  k = 8, 17
		ltbhp (247-k) = ltb (i, j, k)
	    END DO
C
C	    The next 4 bins are 5K wide and need to be  pixel-
C	    replicated (see above for detailed comments).
C
	    DO  k = 18, 21
		avg2            = ltb (i, j, k) * .2
		ltbhp (319-5*k) = NINT (      avg2 )
		ltbhp (318-5*k) = NINT ( 2. * avg2 )
		ltbhp (317-5*k) = NINT ( 3. * avg2 )
		ltbhp (316-5*k) = NINT ( 4. * avg2 )
C
		ltbhp (315-5*k) = ltb (i, j, k)   - ltbhp (316-5*k)
		ltbhp (316-5*k) = ltbhp (316-5*k) - ltbhp (317-5*k)
		ltbhp (317-5*k) = ltbhp (317-5*k) - ltbhp (318-5*k)
		ltbhp (318-5*k) = ltbhp (318-5*k) - ltbhp (319-5*k)
	    END DO
C
C	    The next 2 bins are 10K wide and need to be pixel-
C	    replicated (see above for detailed comments).  The final 
C	    bin is semi-infinite, but is treated as size 10K.  Only
C	    5 1K bins are split out of bin 24; the IF puts the rest
C	    in the 184 1K bin and skips down to computing 1K bin 
C	    values from 185 up.
C
	    DO  k = 22, 24
		avg2             = ltb (i, j, k) * .1
		ltbhp (429-10*k) = NINT (      avg2 )
		ltbhp (428-10*k) = NINT ( 2. * avg2 )
		ltbhp (427-10*k) = NINT ( 3. * avg2 )
		ltbhp (426-10*k) = NINT ( 4. * avg2 )
		ltbhp (425-10*k) = NINT ( 5. * avg2 )
		IF  ( k .EQ. 24 )  THEN
		    ltbhp (424-10*k) = ltb (i, j, k)
		  ELSE
		    ltbhp (424-10*k) = NINT ( 6. * avg2 )
		    ltbhp (423-10*k) = NINT ( 7. * avg2 )
		    ltbhp (422-10*k) = NINT ( 8. * avg2 )
		    ltbhp (421-10*k) = NINT ( 9. * avg2 )
C
		    ltbhp (420-10*k) = ltb   (i, j, k)  -
     +				       ltbhp (421-10*k)
		    ltbhp (421-10*k) = ltbhp (421-10*k) - 
     +				       ltbhp (422-10*k)
		    ltbhp (422-10*k) = ltbhp (422-10*k) -
     +				       ltbhp (423-10*k)
		    ltbhp (423-10*k) = ltbhp (423-10*k) -
     +				       ltbhp (424-10*k)
		END IF
		ltbhp (424-10*k) = ltbhp (424-10*k) - ltbhp (425-10*k)
		ltbhp (425-10*k) = ltbhp (425-10*k) - ltbhp (426-10*k)
		ltbhp (426-10*k) = ltbhp (426-10*k) - ltbhp (427-10*k)
		ltbhp (427-10*k) = ltbhp (427-10*k) - ltbhp (428-10*k)
		ltbhp (428-10*k) = ltbhp (428-10*k) - ltbhp (429-10*k)
	    END DO
C
	END IF





C	Open 'out' and write the data.  Start by finding a free FORTRAN 
C	unit number.
C
        IF (out .NE. 'none') THEN

c          get unused unit number
           iunit = 0
           opn = .TRUE.
           DO WHILE (opn)
              iunit = iunit + 1
              INQUIRE(iunit, OPENED = opn)
           END DO
           
c          open file
           open(iunit,file=out, IOSTAT=ios, STATUS="replace",
     &          form='formatted',access='sequential')
           if ( ios .NE. 0 ) then
              WRITE(*,*) 'Error in '//out//':'//'Failed to open file'
              STOP
           end if
           
c          write to file
           WRITE (iunit, 310, IOSTAT=ios)  i, j, mpt, lza (i, j) * .1
 310       FORMAT ( '# i,j,m,zenith angle =', I4, I3, I2, F5.1, /,
     &          '# Tb  ltbhp  ltbh  ntbh' )
           if ( ios .NE. 0 ) then
              WRITE(*,*) 'Error in '//out//':'//'Failed to write file'
              STOP
           end if
           
           DO  i = 184, 188
              WRITE (iunit, 315, IOSTAT=ios) i+.5, ltbhp (i), ltbh (i)
 315          FORMAT ( F5.1, 2I6, '     0' )
              if ( ios .NE. 0 ) then
                 WRITE(*,*) 'Error in '//out//':'//
     &                'Failed to write file'
                 STOP
              end if
           END DO
           DO  i = 189, 270
              WRITE (iunit, 320, IOSTAT=ios) i+.5, ltbhp (i), ltbh (i),
     +             ntbh (i, ipt, jpt)
 320          FORMAT ( F5.1, 3I6 )
              if ( ios .NE. 0 ) then
                 WRITE(*,*) 'Error in '//out//':'//
     &                'Failed to write file'
                 STOP
              end if
           END DO
           
c          close file
           close(iunit, IOSTAT=ios)
           if ( ios .NE. 0 ) then
              WRITE(*,*) 'Error in '//out//':'//'Failed to close file'
              STOP
           end if
           WRITE (*, *) 'Write completed to ', out
           
        end if
        
        RETURN
	END
