C@***h* Main_GPCPcdr_d4_FUSE2.f
C       Global Precipitation Climatology Project (GPCP)
C
C NAME
C   Main_GPCPcdr_d4_FUSE2.f
C
C PURPOSE
C   To combine SSMI, IR and gauge data for GPCP Daily
C
C DESCRIPTION
C This program "fuses" the precip estimates from SSMI, IR, gauges, and
C model.  The user can specify smoothing on the bias-adjustment and
C how the SSMI/adjusted GEO offset is to be extended outside 40 deg N
C and S.  Depending on the compilation, work files are on a 2.5x2.5 or
C 1x1-deg lat/lon grid.  All internal computations and output are done
C in mm/d.
C  
C AUTHOR
C   Robert Adler, Mathew Sapiano, Jian-Jian Wang
C   University of Maryland College Park
C
C COPYRIGHT
C   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE
C   PUBLIC
C   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
C   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
C   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
C   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
C   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
C   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE
C   TECHNICAL SUPPORT TO USERS.
C
C HISTORY
C 1/2017 Initial Version prepared for NCEI
C
C@*****      
      program Main_GPCPcdr_d4_fuse2

      IMPLICIT      NONE
      
      INTEGER*4 MXSM,  LN,    LT
      REAL*4    SSLIM, GENLIM, RMISS
      PARAMETER( MXSM   = 15,
     +     LN     = 360,
     +     LT     = 180,
     +     SSLIM  = 0.,
     +     GENLIM = 4.,
     +     RMISS  = -99999. )

      INTEGER*4 i, j, next,  ltskp, npass, iret
      DATA ltskp / 50 /

      REAL*4 sfbrk, wtlim
      INTEGER*4nadjsm, ngeosm, nratsm, nsfsm
      LOGICAL*1mrg40ns
      DATA nadjsm, ngeosm, nratsm, nsfsm, sfbrk, wtlim, 
     +     mrg40ns / 7,5,5,3,65.,14., .FALSE. /

      REAL*4 aconst(7)
     &     / 30, 30, 40, 15, 15, 8, 100 /
      REAL*4 econst(7)
     &        / 3, 3.2, 4, .0025, .45, .0075, 10000 /


      character*256, InterimPATH, OutputPATH, TOVSlist
      character*8 yyyymmdd
      integer Ninputs, inunit, ios, strlen, l_fbas
      logical opn
      CHARACTER*256 fname, fbas, snam, snnam, ssnam, genam, gennam
      
      REAL*4 smth (MXSM, MXSM), difext, ratext,  tapext, taper
      INTEGER*4  ismth

      REAL*4 ext,  dum
  
      REAL*4  geosm (MXSM, MXSM)    
      REAL*4 geo    (LN, LT), geon   (LN, LT), 
     +     ssmi   (LN, LT), snadj  (LN, LT),
     +     add    (LN, LT), ratio  (LN, LT),
     +     adgeo  (LN, LT), smss   (LN, LT),
     +     smadge (LN, LT), diff   (LN, LT),
     +     adssmi (LN, LT), egeo   (LN, LT),
     +     avg    (LN, LT), ess    (LN, LT),
     +     sat    (LN, LT), esat   (LN, LT),
     +     wtge   (LN, LT), wtss   (LN, LT),
     +     qgeo   (LN, LT), qss    (LN, LT), 
     +     qsat   (LN, LT), 
     +     geo_tmp(LN, 80), 
     +     ss    (LN, LT), sn    (LN, LT)
      integer*4 geon_tmp(LN, 80)


      


c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 3 ) then
         WRITE(*,*) 'Error in GPCPcdr_d4_FUSE2:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif
      
C     Get arguments and set the length of the month in hours.
      call getarg(1, yyyymmdd)
      call getarg(2, InterimPATH)
      call getarg(3, OutputPATH)
      
c     smoothing settings
      ismth=5
      DO  j = 1, ismth
         DO  i = 1, ismth
            smth (i, j) = 1.
         END DO
      END DO
      difext=0.
      ratext=0.
      tapext=5.
      taper=10.
      



c     ******************************************************
c     read data from various input files

c     get basename for output files
      call String_Length(OutputPATH,strlen)
      fbas=OutputPATH(1:strlen)//'bin/'//yyyymmdd(1:6)//'/'
      call String_Length(fbas,l_fbas)

c     Read input data
      call String_Length(InterimPATH,strlen)
      
c     SSMI/SSMIS precip filename from CORR_T
      snam=InterimPATH(1:strlen)//'/CORR_T/'//yyyymmdd(1:6)//
     +     '/1dd.'//yyyymmdd//'.t_fsg'
      CALL Read_Binary_File (snam, 360, 180, 1, ssmi)
      
c     SSMI number-of-samples - this is a standard dummy file
      snnam='GPCPcdr_d4/GPCPcdr_d4_FUSE2/Ancillary/'//
     &     'dummy.1_deg.tovs.Ifix.le'
      CALL Read_Binary_File (snnam, 360, 180, 1, sn)
      
c     SSMI source - this is a standard dummy file
c     Legal src values:
c       0-1 SSM/I fields 1 to 2 (water to land)
c       2-3 SSM/I fields 1 to 2 and TOVS
c       4   TOVS
c       5   MSU
c       6   MSU and TOVS
      ssnam ='GPCPcdr_d4/GPCPcdr_d4_FUSE2/Ancillary/'//
     &     'dummy.1_deg.tovs.src.le'
      CALL Read_Binary_File (ssnam, 360, 180, 1, ss)
      
c     GEO filename and format (2.5 deg,50N/S)
      genam=InterimPATH(1:strlen)//'/ADJIR/'//yyyymmdd(1:6)//
     +     '/1dd.'//yyyymmdd//'.p'
      CALL Read_Binary_File (genam, 360, 80, 1, geo_tmp)

c     GEO image number filename and format (2.5 deg,50N/S)
      gennam=InterimPATH(1:strlen)//'/ADJIR/'//yyyymmdd(1:6)//
     +     '/1dd.'//yyyymmdd//'.np'
      CALL Read_Binary_File_int4 (gennam, 360, 80, 1, geon_tmp)
      
      DO  j = 51, 130
         DO  i = 1, LN
            geo(i,j) = geo_tmp(i,j-50)
            geon(i,j) = geon_tmp(i,j-50)
         END DO
      END DO
      




c     ******************************************************
c     Main part of code


C     Find the j index of the last non-zero-adjustment lat., using
C     N-S grid size = 180 deg. / number of N-S grid boxes.  Also,
C     convert taper to change per grid box.
C     
      IF  ( difext .GT. 0. )  THEN
         ext = difext
      ELSE IF  ( ratext .GT. 0. )  THEN
         ext = ratext
      ELSE IF  ( tapext .GT. 0. )  THEN
         ext = tapext
      ELSE
         ext = 0.
      END IF
C     
      IF  ( AMOD ( ext * LT / 180., 1. ) .EQ. 0. )  THEN
         next = INT ( ext * LT / 180. )
      ELSE
         next = INT ( ext * LT / 180. ) + 1
      END IF
      taper = taper / next
      
C     Compute number of SSM/I samples adjusted by snow/ice coverage,
C     and handle cases of marginal or missing SSMI and IR.  In the
C     current scheme snow/ice are handled in building the Chang/Grody
C     SSM/I combination, so they are passed in here as 'none.'
C     Set the SSM/I and adjustment coefficients to missing when SSM/I 
C     is marginally sampled or any negative value; note the simple 
C     number-of-samples-based flagging.  Set geo and coefficients to
C     missing when there is no geo data.  [Resetting ratio and add is 
C     immaterial when not recomputing AGPI.]
C     
      DO  j = 1, LT
         DO  i = 1, LN
            snadj (i, j) = sn (i, j)
            IF  ( snadj (i, j) .LE. 0. .OR. ssmi (i, j) .LT. 0. .OR.
     +           ( snadj (i, j) .LE. SSLIM .AND. ss (i, j) .LE. 2. ) )  
     +           THEN
               ssmi  (i, j) = RMISS
               snadj (i, j) = RMISS
               ratio (i, j) = RMISS
               add   (i, j) = RMISS
            END IF
            IF  ( geon (i, j) .LE. 0. )  THEN
               geo   (i, j) = RMISS
               ratio (i, j) = RMISS
               add   (i, j) = RMISS
            END IF
         END DO
      END DO
      
      
C     When geo-IR exists, smooth-fill SSM/I (in parallel with 1-deg.) 
C     into array smss and smooth it, isolate and smooth the geo-only-
C     IR, and compute the adjustment coefficients.  Done here to avoid 
C     changing CORR.F.
C     [Philosophy: Smooth-filled SSM/I and adjustment ratios should 
C     be flat across the Himalayas; re-computing ratios with the
C     smooth-filled SSM/I will squash the cold sfc artifact in the 
C     IR, while using the smooth-filled ratios will continue to show 
C     it.]

      CALL FILLSM  ( ssmi (1, 1+ltskp), LN, LT-2*ltskp, LN, 0, 
     +     npass, smss (1, 1+ltskp), iret )
      
      IF  ( nadjsm .NE. 0 )  THEN
         DO  j = 1, nadjsm
            DO  i = 1, nadjsm
               geosm (i, j) = 1.
            END DO
         END DO
         CALL SMOOTH  ( nadjsm, geosm, wtlim, smss (1, 1+ltskp), 
     +        LN,     LT-2*ltskp,   smss (1, 1+ltskp), iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *) 'Error in ''SSM/I'' data'
            STOP
         END IF
         
      END IF
      
C     Set the adjusted geo to missing outside the data area.
      DO  j = 1, ltskp
         DO  i = 1, LN
            adgeo (i, j     ) = RMISS
            adgeo (i, LT+1-j) = RMISS
         END DO
      END DO
      
C     GPI is intended to serve as the AGPI, so transfer it.
      DO  j = 1+ltskp, LT-ltskp
         DO  i = 1,  LN
            adgeo (i, j) = geo (i, j)
         END DO
      END DO
      
      
      
C     Smooth SSM/I and IR fields in the region of geo data preparatory 
C     to computing adjustments.

      DO  j = 1, ngeosm
         DO  i = 1, ngeosm
            geosm (i, j) = 1.
         END DO
      END DO
      CALL SMOOTH  ( ngeosm, geosm, wtlim, ssmi   (1, 1+ltskp), 
     +     LN,     LT-2*ltskp,   smss   (1, 1+ltskp), iret )
      IF  ( iret .NE. 0 )  THEN
         WRITE (*, *)  'Error in ''ssmi'' data'
         STOP
      END IF
      CALL SMOOTH  ( ngeosm, geosm, wtlim, adgeo  (1, 1+ltskp), 
     +     LN,     LT-2*ltskp,   smadge (1, 1+ltskp), iret )
      IF  ( iret .NE. 0 )  THEN
         WRITE (*, *)  'Error in geo data'
         STOP
      END IF
      
      DO  j = 1, ltskp
         DO  i = 1, LN
            smss   (i, j     ) = RMISS
            smss   (i, LT+1-j) = RMISS
            smadge (i, j     ) = RMISS
            smadge (i, LT+1-j) = RMISS
         END DO
      END DO
      


C     Adjust SSMI to fill in missing geo
C     and extend the geo influence into the SSMI region. 
C     Smoothfilled difference adjustment.  next is the j index of the
C     last non-zero-difference lat.; zero-fill the rest of diff.
C     
C     Compute the difference (sets RMISS outside the region of 
C     good data, 40N-40S), and do a constant-rate taper to get 
C     diff in all holes.  As well, after the initial smoothfill
C     pass the taper is increased along columns to force the diff
C     to zero at the outer boundary.  Forcing differences toward 
C     zero as you move away from the TMPI non-missing values
C     solves 
C     the smoosh that smooth-filling the difference tends to
C     cause.
c     
c     Note: following comments for the smoothfill extensions, this 
c     taper extend is computed on unsmoothed values, since there's 
c     a fair amount of smoothing in the taper scheme, and the 
c     initial smoothed difference tended to introduce light precip 
c     abutting zero TMPI precip at the boundaries.
C     All areas outside the TMPI are first set to "missing", the
C     taper fill is done out to the ext limit, then remaining
C     "missing"s are set to 0.  This ensures that the outer B.C.
C     is "missing" during the fill.
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ratio (i, j) .GE. 0. )  THEN
               IF  ( ssmi (i, j) .GE. 0. )  THEN
                  diff   (i, j) = adgeo (i, j) - ssmi (i, j)
               ELSE
                  diff   (i, j) = RMISS
               END IF
            ELSE
               diff   (i, j) = RMISS
            END IF
         END DO
      END DO      
      CALL FILLTA  ( diff, LN, LT, LN, ltskp-next, taper, next, 
     +     diff, iret )
      DO  j = 1,LT
         DO  i = 1, LN
            IF  ( diff (i, j) .EQ. RMISS )  diff (i, j) = 0.
         END DO
      END DO
      
C     Adjust the SSMI by adding diff (but the sum must be .GE. 0).
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ssmi (i, j) .GE. 0. )  THEN
               IF  ( ssmi (i, j) + diff (i, j) .GE. 0. )  THEN
                  adssmi (i, j) = ssmi (i, j) + diff (i, j)
               ELSE
                  adssmi (i, j) = 0.
               END IF
            ELSE
               adssmi (i, j) = RMISS
            END IF
         END DO
      END DO
C     



C     Compute SSM/I sampling error using its own estimated rain just
C     for output. When there are different SSMI sources (and 
C     consequently errors) for land and water plus TOVS (at least 
C     potentially), three sets of constants are provided.
C     
      CALL SAMERR ( 24.,    LN,  LT,     econst,     aconst, 
     +     'ssto', 3,   adssmi, snadj,  ss, ess, iret )



C     Compute AGPI sampling error using its own estimated rain just
C     for output.  Here and throughout, AGPI error is arbitrarily 
C     boosted by 41% (so variance is approx. 2x higher) for loss of 
C     SSM/I calibration data.
C     
      CALL SAMERR ( 24.,    LN, LT,    econst (5), aconst (5), 
     +     'agpi', 1,  adgeo, geon, dum,  egeo, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1+ltskp, LT-ltskp
         DO  i = 1, LN
            IF  ( smss (i, j) .LT. 0. )  THEN
               IF  ( egeo (i, j) .GT. 0. )  egeo (i, j) = 1.41 *
     +              egeo (i, j)
            END IF
         END DO
      END DO
      
      
C     Compute SSM/I and AGPI quality index, and dump out fields.
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, adgeo,
     +     egeo,       qgeo,       iret )
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, adssmi,
     +     ess,        qss,        iret )
      
      
c     Write some interim files
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.egeo'
c      call Write_Binary_File(fname,egeo, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.ess'
c      call Write_Binary_File(fname,ess, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.qss'
c      call Write_Binary_File(fname,qss, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.qgeo'
c      call Write_Binary_File(fname,qgeo, 360, 180)
      

      

C     Compute the straight average of the available rain values 
C     that go into the multi-satellite combination; this is only 
C     used for the rain value in calculating input errors for 
C     the multi-satellite combination.  The order of tests is
C     intended to minimize computation.  A false mrg40ns specifies
C     that geo-AGPI is not combined.
      
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( adssmi (i, j) .GE. 0. )  THEN
               IF  ( adgeo (i, j) .LT. 0. )  THEN
                  avg (i, j) = adssmi (i, j)
               ELSE IF  ( geon (i, j) .GE. GENLIM .AND.
     +                 .NOT. mrg40ns )  THEN
                  avg (i, j) = adgeo (i, j)
               ELSE
                  avg (i, j) = ( adssmi (i, j) + adgeo (i, j) ) * .5
               END IF
            ELSE IF  ( adgeo (i, j) .LT. 0. )  THEN
               avg (i, j) = RMISS
            ELSE
               avg (i, j) = adgeo (i, j)
            END IF
         END DO
      END DO
      
C     Compute SSM/I sampling error using avg rain for multi-satellite
C     combination.  When there are different SSMI sources (and
C     consequently errors) for land and water plus TOVS (at least
C     potentially), three sets of constants are provided.
C     
      CALL SAMERR ( 24.,    LN,  LT,  econst,     aconst, 
     +     'ssto', 3,   avg, snadj,  ss, ess, iret )
      IF  ( iret .NE. 0 )  STOP
      

C     Compute AGPI sampling error using avg rain for multi-satellite
C     combination.
      
      CALL SAMERR ( 24.,    LN, LT,  econst (5), aconst (5), 
     +     'agpi', 1,  avg, geon, dum,  egeo, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1+ltskp, LT-ltskp
         DO  i = 1, LN
            IF  ( smss (i, j) .LT. 0. )  THEN
               IF  ( egeo (i, j) .GT. 0. )  egeo (i, j) = 1.41 *
     +              egeo (i, j)
            END IF
         END DO
      END DO
      

C     Make sure the error is only available where the estimate is
C     available.
C     
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( adssmi (i, j) .LT. 0. )  ess  (i, j) = RMISS
            IF  ( adgeo  (i, j) .LT. 0. )  egeo (i, j) = RMISS
         END DO
      END DO
      
      
C     Form the multi-satellite precip estimate.  In the no-merge case
C     the geo-AGPI is used where available and the combination scheme 
C     is used where the geo-AGPI is not available.  Geo-AGPI 
C     availability is flagged by th number of IR images, but could be 
C     fooled by a day or month with few geo images.  However, in that 
C     case, you might well want to do a combination anyway.  [The code 
C     is implemented in the opposite order.]
C     
C     In the merge case, use a weighted average of AGPI and SSM/I; 
C     this gets important when there are two SSM/I platforms, and is 
C     the scheme for Pathfinder.
C     
      CALL WTCOMB  ( LN, LT, adssmi, ess, adgeo, egeo, sat, esat,
     +     wtss,   wtge,   iret )
C     
      IF  ( .NOT. mrg40ns )  THEN
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( geon (i, j) .GE. GENLIM .AND.
     +              egeo (i, j) .GT. 0. )  THEN
                  sat  (i, j) = adgeo (i, j)
                  esat (i, j) = egeo  (i, j)
                  wtss (i, j) = 0.
                  wtge (i, j) = 1.
               END IF
            END DO
         END DO
      END IF
C     
C     Compute S quality index and dump out fields.
C     
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, sat,
     +     esat,       qsat,       iret )
      
      
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.add'
c      call Write_Binary_File(fname, add, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.rat'
c      call Write_Binary_File(fname, ratio, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.smss'
c      call Write_Binary_File(fname, smss, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.sadge'
c      call Write_Binary_File(fname, smadge, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.adge'
c      call Write_Binary_File(fname, adgeo, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.adss'
c      call Write_Binary_File(fname, adssmi, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.wtge'
c      call Write_Binary_File(fname, wtge, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.wtss'
c      call Write_Binary_File(fname, wtss, 360, 180)
      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.s'
      call Write_Binary_File(fname, sat, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.es'
c      call Write_Binary_File(fname, esat, 360, 180)
c      fname=fbas(1:l_fbas)//'stfsg3.'//yyyymmdd//'.qs'
c      call Write_Binary_File(fname, qsat, 360, 180)
      
      
      WRITE (*, *) 'Done'
      STOP
      END
      

      

