	SUBROUTINE FILLTA  ( arrin,  LN, LT, MXLN, LTSKP, taper, NEXT, 
     +			     arrout, iret )
C************************************************************************
C* FILLTA  Fill holes with a 3x3 taper-smoother for a 2-D real array	*
C*									*
C* Fill holes in a LNxLT field by repetitively applying a 3x3 smoother	*
C* with "taper".  That is, subtract a user-specified "taper" value from	*
C* each value (or add for a negative value) being averaged in at the	*
C* missing points (flagged with the "R*4 missing value", -99999.).	*
C* As well, after the initial smoothfill pass the taper is increased	*
C* along columns (where needed to force the diff to zero at the outer	*
C* boundary.  The north and south boundaries are filled with "missing",	*
C* while the east and west are periodic.  Optionally, only a central	*
C* band (size LT-2*LTSKP) is processed.  Unlike FILLSM.F, FILLTA.F is a	*
C* one-pass operation.  In the smoother, missing values simply give	*
C* more weight to the good values contributing to a point.		*
C*									*
C* FILLTA  ( arrin, LN, LT, MXLN, LTSKP, taper, next, arrout, iret )	*
C*									*
C* Input parameters:							*
C*   arrin	REAL*	Input data (data size LN,LT; physical size	*
C*			MXLN,LT+)					*
C*   LN		INT	Data X size					*
C*   LT		INT	Data Y size					*
C*   MXLN	INT	Array X size (don't need array Y as long as	*
C*			the data start at the "top" of the array)	*
C*   LTSKP	INT	Number of rows to skip in processing		*
C*   taper	REAL	User-specified decrease in arrin as each new	*
C*			grid box is encountered				*
C*   NEXT	INT	Width of taper zone (implicit in LTSKP		*
C*									*
C* Output parameters:							*
C*   arrout	REAL*	Output smoothed data (data size LN,LT; physical	*
C*			size MXLN,LT+)					*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = ismth out of bounds or even	*
C*				    -2 = no data in input array at all	*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 3/12	Adapt FILLSM.F				*
C* G.Huffman/612	 9/12	Add forcing diff to zero at boundary	*
C************************************************************************
	IMPLICIT	NONE
C
C	Args are (ln,lt).
C
	INTEGER*4	MXSM, MS_2, MXX, MXY
	REAL*4		RMISS
	PARAMETER	( MXSM = 3,
     +			  MS_2 = MXSM / 2,
     +			  MXX  = 720,
     +			  MXY  = 360,
     +			  RMISS = -99999. )
C
	INTEGER*4	LN, LT, MXLN,     LTSKP, NEXT, iret
	REAL*4		arrin (MXLN, LT), taper, arrout (MXLN, LT)
C
	REAL*4		wt, xwt, smth  (MXSM, MXSM), augta, dtaper,
     +			aug   (MXX + 2 * MS_2, MXY + 2* MS_2, 2),
     +			augwt (MXX + 2 * MS_2, MXY + 2* MS_2)
	INTEGER*4	i,  ii, j, jj, k, m1, m2, ismth, is_2, nfill, n
	LOGICAL*1	done
	DATA		ismth, smth / 3, 9*1. /
C
C	Check that ismth is odd and in bounds.
C
	IF  ( ismth .LT. 3 .OR. MOD ( ismth, 2 ) .NE. 1 .OR.
     +	      ismth .GT. MXSM )  THEN
	    WRITE (*, *) 'Error: taper routine found ismth (=', 
     +			 ismth, ') <3, even, or >MXSM (=', MXSM, ').'
	    iret = -1
	    RETURN
	END IF
C
C	If there's no data in the input array reflect the input array
C	into the output array with an error.
C
	DO  j = 1, LT
	DO  i = 1, LN
	    IF  ( arrin (i, j) .NE. RMISS )  GO TO 10
	END DO
	END DO
	    DO  j = 1, LT
	    DO  i = 1, LN
		arrout (i, j) = arrin (i, j)
	    END DO
	    END DO
	    WRITE (*, *) 'Error: fill-smooth routine found no data.'
	    iret = -2
	    RETURN
   10	CONTINUE
C
C	Load 2 augmented arrays of data so that indexing the convolution
C	is easy.  The "picture frame" is loaded all the way out to MS_2
C	(rather than is_2) to simplify indexing.  The fill is RMISS
C	(missing) above and below, and periodic left and right.
C
	is_2 = ismth / 2
	DO  j = 1, LT
	DO  i = 1, LN
	    aug (MS_2 + i, MS_2 + j, 1) = arrin (i, j)
	    aug (MS_2 + i, MS_2 + j, 2) = arrin (i, j)
	END DO
	END DO
C
	DO  j = 1, MS_2
	DO  i = 1, LN + 2 * MS_2
	    aug (i, j            , 1) = RMISS
	    aug (i, j + MS_2 + LT, 1) = RMISS
	    aug (i, j            , 2) = RMISS
	    aug (i, j + MS_2 + LT, 2) = RMISS
	END DO
	END DO
C
	DO  j = MS_2 + 1, MS_2 + LT
	DO  i = 1,        MS_2
	    aug (i,             j, 1) = aug (i + LN  , j, 1)
	    aug (i + LN + MS_2, j, 1) = aug (i + MS_2, j, 1)
	END DO
	END DO
C
C	Apply the smoother to originally missing boxes until no boxes 
C	are missing.  [Equivalent to FILLSM.F Section 1.]
C
	nfill = 0
	m1    = 1
	m2    = 2
C
C	Check that there are holes to fill.
C
	DO  j = 1 + LTSKP, LT - LTSKP
	DO  i = 1,         LN
	    IF  ( arrin (i, j) .EQ. RMISS )  GO TO 50
	END DO
	END DO
	done = .TRUE.
	GO TO 55
   50	CONTINUE
	done = .FALSE.
   55	CONTINUE
C
	DO  WHILE ( .NOT. done )
	    nfill = nfill + 1
C
C	    Do the convolution, indexing on aug corner and smth center.
C	    the value being brought in is reduced toward zero first, then
C	    summed.
C
	    DO  j = MS_2 + 1 + LTSKP, MS_2 + LT - LTSKP
	    DO  i = MS_2 + 1,         MS_2 + LN
		wt  = 0.
		xwt = 0.
		IF  ( aug (i, j, m1) .EQ. RMISS )  THEN
		    DO  jj = -is_2, is_2
		    DO  ii = -is_2, is_2
			IF  ( aug (ii + i, jj + j, m1) .NE. RMISS )  THEN
			    IF  ( aug (ii+i, jj+j, m1) .LT. -taper ) THEN
				augta = aug (ii+i, jj+j, m1) + taper
			      ELSE IF  ( aug (ii+i, jj+j, m1) .GT.
     +					 taper )  THEN
				augta = aug (ii+i, jj+j, m1) - taper
			      ELSE
				augta = 0.
			    END IF
			    wt  = wt  + 
     +				  smth (ii + is_2 + 1, jj + is_2 + 1)
			    xwt = xwt + 
     +				  smth (ii + is_2 + 1, jj + is_2 + 1)*
     +		 		  augta
cc	IF  ( i .EQ. 306 )
cc   +	WRITE (*, *) i,j,ii,jj,m1,taper,aug (ii+i, jj+j, m1),wt,xwt
			END IF
		    END DO
		    END DO
		    augwt (i, j) = wt
		    IF  ( wt .EQ. 0. )  THEN
			aug (i, j, m2) = RMISS
		      ELSE
			aug (i, j, m2) = xwt / wt
		    END IF
		END IF
	    END DO
	    END DO
C
C	    Wrap up the smoothing by copying the new m2 values into
C	    m1 and filling the lateral boundaries.  The top and 
C	    bottom boundaries are still "missing".
C
	    DO  j = MS_2 + 1 + LTSKP, MS_2 + LT - LTSKP
	    DO  i = MS_2 + 1,         MS_2 + LN
		aug (i, j, m1) = aug (i, j, m2)
	    END DO
	    END DO
	    DO  j = MS_2 + 1, MS_2 + LT
	    DO  i = 1,        MS_2
		aug (i,             j, m1) = aug (i + LN  , j, m1)
		aug (i + LN + MS_2, j, m1) = aug (i + MS_2, j, m1)
	    END DO
	    END DO
C
C	    Check that there are still holes to fill.
C
	    DO  j = MS_2 + 1 + LTSKP, MS_2 + LT - LTSKP
	    DO  i = MS_2 + 1,         MS_2 + LN
		IF  ( aug (i, j, m1) .EQ. RMISS )  GO TO 150
	    END DO
	    END DO
	    done = .TRUE.
	    GO TO 155
  150	    CONTINUE
	    done = .FALSE.
  155	    CONTINUE
	END DO
	WRITE (*, *)  'It took', nfill, ' passes to fill holes ',
     +		      'in FILLTA'
C
C	Now step to each column (including the lateral boundaries) and 
C	force the taper to go to "taper" at the last real point 
C	(north first, then south) by subtracting a linearly scaled 
C	excess (or adding if the difference is negative) that goes to
C	zero at the last interior point.
C
	DO  i = 1, LN + MS_2 * 2
	    IF  ( aug (i, MS_2+1+LTSKP, m1) .GT. taper )  THEN
		dtaper = aug (i, MS_2+1+LTSKP, m1) - taper
	      ELSE IF  ( aug (i, MS_2+1+LTSKP, m1) .LT. -taper )  THEN
		dtaper = aug (i, MS_2+1+LTSKP, m1) + taper
	      ELSE
		GO TO 200
	    END IF
	    DO  j = 1, next
		aug (i, MS_2+j+LTSKP, m1) = 
     +					aug (i, MS_2+j+LTSKP, m1) -
     +					dtaper * (next - j + 1) / next
	    END DO
  200	    CONTINUE
	END DO
C
	DO  i = 1, LN + MS_2 * 2
	    IF  ( aug (i, MS_2+LT-LTSKP, m1) .GT. taper )  THEN
		dtaper = aug (i, MS_2+LT-LTSKP, m1) - taper
	      ELSE IF  ( aug (i, MS_2+LT-LTSKP, m1) .LT. -taper )  THEN
		dtaper = aug (i, MS_2+LT-LTSKP, m1) + taper
	      ELSE
		GO TO 205
	    END IF
	    DO  j = 1, next
		aug (i, MS_2+LT+1-j-LTSKP, m1) = 
     +					aug (i, MS_2+LT+1-j-LTSKP, m1) - 
     +		    dtaper * (next - j + 1) / next
	    END DO
  205	    CONTINUE
	END DO
C
C	Done; copy the "final" array to the output array.
C
	DO  j = 1, LT
	DO  i = 1, LN
	    arrout (i, j) = aug (MS_2 + i, MS_2 + j, m1)
	END DO
	END DO
C
	iret = 0
	RETURN
	END
