


clear all;

% Read all 1DD CDR files and check precip output for each day
% Note - this is slow as each file must be unzipped and then read...

% add path to directory where some useful functions are stored
path(path,'GPCPcdr_d5/GPCPcdr_d5_TestOutput/');

% get path of directories from options file
% read options file, and match  find name of "OutputPATH", then extract it
lines=textread('OptionsD.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize output file:
oname='GPCP_FullTest_daily.txt';
fid=fopen(oname,'wt');
fprintf(fid,'Results of full test of GPCP 1DD CDR files\n');
fprintf(fid,'Processed: %s\n',datestr(now,0));




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all CDR files
NDaysInMonth=[31,28,31,30,31,30,31,31,30,31,30,31];

tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01oct96'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

Table=[]; nofile=[];
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));

  tests=[]; nofile(k,:)=[yyyy mm zeros(1,31)];
  Ndays=NDaysInMonth(mm); 
  if (mm==2 & mod(yyyy,4)==0), Ndays=29; end;

  % scroll over days
  for dd=1:Ndays,

    % get filename for current day and check if it exists
    fname=sprintf('%s/%04d%02d/gpcp_daily_cdr_v23rB1_y%04d_m%02d_d%02d.nc.gz',fdir,yyyy,mm,yyyy,mm,dd);
    if ~isempty(dir(fname)),

    % read precip in file
    [s w]=unix(['gzip -dcf ' fname ' > tmp.bin']);
    try, data=ncload('tmp.bin');
        tests(dd,:)=[(length(fieldnames(data))-5)~=1 sum(sum(data.precip==data.atts.precip.missing_value))~=0 sum(sum(data.precip==0))~=0];

    % if file does not exist, add to nofile 
    catch,
      nofile(k,dd+2)=1; tests(dd,:)=1;
    end;
  else,
    nofile(k,dd+2)=1; tests(dd,:)=1;
  end;end;

  Table(k,:)=[yyyy mm Ndays sum(tests(:,1:2)~=0) sum(tests(:,3)~=1)];
end; end;

% add to output file
fprintf(fid,'\n\nGPCP 1DD CDR: \n');
fprintf(fid,['YYYY MM  #Days #zero #vars #miss\n']);
fprintf(fid,['%04d %02d   %6d %6d %6d %6d\n'],Table');




% save months that did not process
try, cdr_summary=Table(find(sum(Table(:,5:6)~=0,2)),1:2);
catch, cdr_summary=[];
end;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check for days with no files between first and last day of files
fprintf(fid,'\n\nGPCP 1DD CDR days with no files (that should have files): \n');
fprintf(fid,['YYYY MM ' repmat('%2d ',1,31) '\n'],1:31);
fprintf(fid,['%04d %02d ' repmat('%2d ',1,31) '\n'], nofile');






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% State when files were processed
% this unix code will return all unique days where processing occurred
[s w]=unix(sprintf('ls --full-time %s/*/gpcp_daily_cdr_v13rA1_*gz | awk -F" " ''{print $6}'' |sort|uniq',fdir));

fprintf(fid,'\n\nGPCP 1DD CDR dates of processing: \n');
fprintf(fid,'Note: Files were processed on the following unique days.\n');
fprintf(fid,'%c', w');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Close output file
fclose(fid);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output summary stats to screen
file=textread(oname,'%s','delimiter','\n','whitespace','');
disp(char({'1DD CDR Summary of possible corrupted months:';'';char(file);'';'Suggest reprocessing any potentially corrupt months until all year are OK';'';'Full output saved in:';[pwd '/' oname]}));















