/** GPCP_Daily_Analysis/Main_GPCPcdr_d5_WriteREVNC.c
 *  Global Precipitation Climatology Project (GPCP)
 *
 * NAME
 *   Main_GPCPcdr_d5_WriteREVNC.c
 *
 * PURPOSE
 *   This routine writes the daily GPCP CDR NetCDF file.
 *
 * DESCRIPTION
 *   This is a sample C program to writes the daily GPCP CDR NetCDF file.
 *
 * AUTHOR
 *   Robert Adler, Mathew Sapiano, Jian-Jian Wang
 *   University of Maryland College Park
 *
 * COPYRIGHT
 *   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
 *   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
 *   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
 *   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
 *   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
 *   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
 *   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
 *   SUPPORT TO USERS.
 *
 * HISTORY
 *   1/2017 Initial Version prepared for NCEI
 *
 **/
#include <stdio.h>
#include <time.h>
#include <sys/utsname.h>

#include <sys/stat.h>
#include <openssl/md5.h>
#include <unistd.h>
#include <fcntl.h>

#include "../../OptionsD.h"

int main(int argc, char *argv[])
{
  int    yyyy;
  int    mm;
  int    dd;
  int    yyyy1;
  int    mm1;
  char   fname_cdr[256];
  char   VersNum[6];
  char  *CompName;

  // Establish time variables and structure fo robtaining local time
  time_t current_time_t;
  struct tm *current_time;
  char   current_time_str[256];
  char current_date[128] = {0};

  // establish structure for getting unix system name
  struct utsname unameData;

  // Get day of month
  int DaysOfMonth[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
  int NDays;

  /* Get input arguments */
  yyyy = atoi(argv[1]);
  mm = atoi(argv[2]);
  yyyy1 = yyyy;
  mm1 = mm + 3;
  if (mm1 > 12) {
    mm1 = mm1 - 12;
    yyyy1 = yyyy + 1;
  }

  // Set number of days of month
  NDays=DaysOfMonth[mm-1];

  // deal with leap year (use simple test as 2000 IS a leap year)
  if (yyyy % 4 == 0 && mm == 2) NDays=29;

  /* Set version number - must be 6 digits
   * V22 reflects GPCP V2.2, rXX is release number and can change more often */
  sscanf(argv[3],"%6s",&VersNum);

  /* Obtain current time as seconds elapsed since the Epoch and convert to text */
  current_time_t = time(NULL);
  current_time = localtime(&current_time_t);
  sprintf(current_time_str,"%04d-%02d-%02dT%02d:%02d:%02dZ", 
	  1900+current_time->tm_year, 1+current_time->tm_mon, current_time->tm_mday, 
	  current_time->tm_hour, current_time->tm_min, current_time->tm_sec);

  /* Get Computer name */
  if ( uname(&unameData) == -1) {
    fprintf(stderr,"Can't get system name\n");
    CompName="unknown";
  }
  else {
    CompName=unameData.nodename;
  }

  sprintf(current_date, "%04d%02d%02d",
			1900+current_time->tm_year,
			1+current_time->tm_mon,
			current_time->tm_mday);
  for( dd=0; dd<NDays; dd++ ) {
    /* Set output name - include yyyymm directory*/
    /*sprintf(fname_cdr,"%04d%02d/gpcp_daily_cdr_%s_y%04d_m%02d_d%02d.nc",yyyy,mm,VersNum,yyyy,mm,dd+1);*/
    sprintf(fname_cdr,"%04d%02d/gpcp_%s_daily_d%04d%02d%02d_c%s.nc",yyyy,mm,VersNum,yyyy,mm,dd+1,current_date);
    
    /* Call function to create primary CDR file*/ 
    Write_CDR_nc_daily(OutputPATH,fname_cdr,VersNum,CompName,&current_time_str,yyyy,mm,dd+1);
  }
 
  {
    char data_file[128] = {0};
    char data_dir[128] = {0};
    char data_tar[128] = {0};
    char data_list[128] = {0};
    char command[256] = {0};
    sprintf(data_dir, "%s%04d%02d", OutputPATH, yyyy, mm);
    sprintf(data_tar, "gpcp_%s_daily_d%04d%02d_c%s.tar",
		VersNum,yyyy,mm,current_date);
    sprintf(data_list, "gpcp_%s_daily_*.nc.gz", VersNum);
    sprintf(data_file, "%s/%s", data_dir, data_tar);
    sprintf(command, "cd %s;tar -cvf %s %s", data_dir, data_tar, data_list);
    system(command);

    int size = 0;
    char md5_sum[2*MD5_DIGEST_LENGTH+1] = {0};
    size = get_size(data_file);
    if (get_MD5(data_file, md5_sum) && size) {
      FILE *fp = NULL;
      char info_file[128] = {0};
      sprintf(info_file, "%s.mnf", data_file);
      fp = fopen(info_file, "w");
      if (fp) {
        fprintf(fp, "%s,%s,%d", data_tar, md5_sum, size);
        fclose(fp);
      }
    }
  }
}

int get_size(char *filename)
{
  int size = 0;
  struct stat st;
  stat(filename, &st);
  size = st.st_size;
  return size;
}

int get_MD5(char *filename, char *md5_sum)
{
  int n;
  MD5_CTX c;
  unsigned char out[MD5_DIGEST_LENGTH] = {0};
  char buf[512];
  ssize_t bytes;
  int fdc = open(filename, O_RDONLY);

  if (fdc) {
    MD5_Init(&c);
    bytes=read(fdc, buf, 512);
    while(bytes > 0)
    {
      MD5_Update(&c, buf, bytes);
      bytes=read(fdc, buf, 512);
    }

    MD5_Final(out, &c);

    for(n = 0; n < MD5_DIGEST_LENGTH; n++)
      sprintf(&md5_sum[2*n], "%02x", out[n]);
    md5_sum[2*n+3] = '\0';
    close(fdc);
    return 1;
  } else {
    printf("Unable to open file %s", filename);
    return 0;
  }
}
