#!/bin/sh
#@***h* configure_daily.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   configure_daily.sh
#
# PURPOSE
#   To build Daily GPCP V1.3 CDR
#
# DESCRIPTION
#   Configure script to build GPCP V2.3 CDR
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 1/2017 Initial Version prepared for NCEI
#
#@*****

pdir=`pwd`

##################################################
# establish paths that need to exist
InputPATH=$(echo `grep "InputPATH" OptionsD.h` | awk -F\"  '{print $2}')
InterimPATH=$(echo `grep "InterimPATH" OptionsD.h` | awk -F\"  '{print $2}')
OutputPATH=$(echo `grep "OutputPATH" OptionsD.h` | awk -F\"  '{print $2}')
GPROFPATH=$(echo `grep "GPROFPATH" OptionsD.h` | awk -F\"  '{print $2}')

if [ ! -d $InputPATH ]; then
  echo "Warning: Input path is undefined; please get input data!"
  echo "reluctantly continuing..."
fi
if [ ! -d $InterimPATH ]; then
  echo "Interim path is undefined - creating"
  mkdir $InterimPATH
fi
if [ ! -d $OutputPATH ]; then
  echo "Output path is undefined - creating"
  mkdir $OutputPATH
fi
if [ ! -d $OutputPATH/bin ]; then
  echo "Output path for binary files is undefined - creating"
  mkdir $OutputPATH/bin
fi



# Establish locations for interim files
if [ ! -d "$GPROFPATH" ]; then
  echo "GPROF2004V path is undefined - creating"
  mkdir "$GPROFPATH"
fi
if [ ! -d "${InterimPATH}/CORR_G/" ]; then
  echo "${InterimPATH}/CORR_G path is undefined - creating"
  mkdir "${InterimPATH}/CORR_G/"
fi
if [ ! -d "${InterimPATH}/ADJIR/" ]; then
  echo "${InterimPATH}/ADJIR path is undefined - creating"
  mkdir "${InterimPATH}/ADJIR/"
fi
if [ ! -d "${InterimPATH}/CORR_T/" ]; then
  echo "${InterimPATH}/CORR_T path is undefined - creating"
  mkdir "${InterimPATH}/CORR_T/"
fi
if [ ! -d "${InterimPATH}/FUSE2/" ]; then
  echo "${InterimPATH}/FUSE2 path is undefined - creating"
  mkdir "${InterimPATH}/FUSE2/"
fi




##################################################
# Make codes

cd ${pdir}/GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMI
rm *.o
make -f Makefile

cd ${pdir}/GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMIS
rm *.o
make -f Makefile

cd ${pdir}/GPCPcdr_d4/GPCPcdr_d4_CORR_G
rm *.o
make -f Makefile

cd ${pdir}/GPCPcdr_d4/GPCPcdr_d4_ADJIR
rm *.o
make -f Makefile

cd ${pdir}/GPCPcdr_d4/GPCPcdr_d4_CORR_T
rm *.o
make -f Makefile

cd ${pdir}/GPCPcdr_d4/GPCPcdr_d4_FUSE2
rm *.o
make -f Makefile


##################################################
# return to present directory and exit
cd ${pdir}/
exit



