	PROGRAM GENMISS_1DD
C	
C***********************************************************************
C*  GENMISS_1DD	Creates 1DD files when TOVS is missing		       *
C*								       *
C* This program creates a day of 1DD data when the corresponding day   *
C* of TOVS data is missing.  The 1dd*.p TMPI file is directly          *
C* transferred unchanged to the final stf* 1DD daily file, so the day  *
C* of 1DD data is only the TMPI.  GENMISS_1DD simply increases the     *
C* array size of the TMPI file to match the size of the final 1DD      *
C* file.  This program is necessary since fuse2.10day.new will not     *
C* create a 1DD data file when the daily TOVS data is missing.         *
C* Basically, it was much easier to create this program than to modify *
C* fuse2.10day.new.  						       *
C*								       *
C* genmiss_1dd tmpifile oddfile					       *
C*								       *
C* Input command-line arguments:				       *
C*								       *
C* tmpifile CHAR daily TMPI file from /data1/GPCP_CDR/GPCP_Interim/ADJIR
C* oddfile  CHAR daily 1DD  file to /data1/GPCP_CDR/GPCP_Output/1DD/bin*
C*								       *
C**								       *
C* Log:								       *
C* D.Bolvin/SSAI	06/04	First Version			       *
C* JJ WANG/UMD  	08/17	             			       *
C***********************************************************************
C
	IMPLICIT	NONE
C
C***********************************************************************
C
	INTEGER*4	i, j, iunit, nargs, IARGC, ios
C
	REAL*4		FILL/-99999.0/
	REAL*4		tmpidata(360,80), odddata(360,180)
C
	CHARACTER*256	tmpifile, oddfile
C
	LOGICAL		opn
C
C***********************************************************************
C	Read the command-line arguments.  If no command line arguments 
C	are entered, print out the "usage".  If 1 or > 2 arguments are 
C	entered, print an error message and terminate execution.
C
	nargs = IARGC ()
C
	IF  (nargs .EQ. 0) THEN
	    WRITE (*,*) 'This program creates a 1DD file from a ',
     +			'TMPI file'
     	    WRITE (*,*) 'Usage: <input daily TMPI file> ',
     +			'       <output daily 1DD file> '
     	    STOP
        END IF
C
	IF  (nargs .NE. 2) THEN
	    WRITE (*,*) 'ERROR: NUMBER OF INPUT ARGUMENTS ',nargs, 
     +			      ' INCORRECT; TERMINATING EXECUTION'
     	    STOP
     	END IF
C
	CALL GETARG (1, tmpifile)
	CALL GETARG (2, oddfile)
C
C***********************************************************************
C	Open and read the TMPI file.  If there's an error opening or 
C	reading the file, write an error message and terminate 
C	execution.
C
	opn = .TRUE.
	iunit = 0
C
	DO WHILE (opn)
	    iunit = iunit + 1
	    INQUIRE (iunit, OPENED = opn)
	END DO
C
	OPEN (UNIT = iunit, FILE = tmpifile, FORM = 'UNFORMATTED',
     +	      ACCESS = 'DIRECT', RECL = 115200, STATUS = 'OLD', 
     +	      ERR = 100, IOSTAT = ios)
C
	READ (iunit, REC = 1, ERR = 101, IOSTAT = ios) tmpidata
C
	CLOSE (iunit)
C
C***********************************************************************
C	Initialize the 1DD array with "missing" values and then copy the
C	TMPI data into the 1DD array.
C
	DO i = 1, 360
	    DO j = 1, 180
	    	odddata(i,j) = FILL
	    END DO
	END DO
C
	DO i = 1, 360
	    DO j = 1, 80
	    	odddata(i,j + 50) = tmpidata(i,j)
	    END DO
	END DO
C
C***********************************************************************
C	Open and write the 1DD file.  If there's an error opening or 
C	writing the file, write an error message and terminate 
C	execution.
C
	opn = .TRUE.
	iunit = 0
C
	DO WHILE (opn)
	    iunit = iunit + 1
	    INQUIRE (iunit, OPENED = opn)
	END DO
C
	OPEN (UNIT = iunit, FILE = oddfile, FORM = 'UNFORMATTED',
     +	      ACCESS = 'DIRECT', RECL = 259200, STATUS = 'NEW', 
     +	      ERR = 102, IOSTAT = ios)
C
	WRITE (iunit, REC = 1, ERR = 103, IOSTAT = ios) odddata
C
	CLOSE (iunit)
C
	GO TO 200
C
 100	WRITE (*,*) 'ERROR: UNABLE TO OPEN FILE ', tmpifile, 
     +		    '; ERROR ', ios, '; TERMINATING EXECUTION'
     	STOP
C
 101	WRITE (*,*) 'ERROR: UNABLE TO READ FILE ', tmpifile, 
     +		    '; ERROR ', ios, '; TERMINATING EXECUTION'
     	STOP
C
 102	WRITE (*,*) 'ERROR: UNABLE TO OPEN FILE ', oddfile, 
     +		    '; ERROR ', ios, '; TERMINATING EXECUTION'
     	STOP
C
 103	WRITE (*,*) 'ERROR: UNABLE TO WRITE FILE ', oddfile, 
     +		    '; ERROR ', ios, '; TERMINATING EXECUTION'
     	STOP
C
 200	CONTINUE
C
	STOP
	END

