c     This file contains 3 subroutines that (1) read the 
c     Netcdf file to get the number of scans, (2) read
c     all data in the file and (3) handle errors


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine Readbin_RSS_SSMI_Tb_v6_nscans(input_file, 
     &     nscan_lores, nscan_hires)
c     Code to read number of scans in current NCDC RSS NetCDF4 file
c     This avoids the need for allocatable arrays and allows the use
c     of F77 rather than F90

      implicit none

      character*256   input_file

      INTEGER*4 iunit, l_if
      INTEGER*4 KSAT,IORBIT,NUMSCAN

      integer*4 nscan_lores
      integer*4 nscan_hires

      OPEN(iunit, FILE=input_file, STATUS='OLD',
     &     ACCESS='DIRECT', RECL=3768, FORM='UNFORMATTED')
      READ(iunit,REC=1) KSAT,IORBIT,NUMSCAN
      CLOSE(iunit)

      nscan_lores=NUMSCAN
      nscan_hires=NUMSCAN*2


      end subroutine Readbin_RSS_SSMI_Tb_v6_nscans





cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine Readbin_RSS_SSMI_Tb_v6(input_file, NSCANS, NSCANS2,
     &     satname, itime87_ARR, stdtime_A,
     &     iflags, frcrev, xlat, xlon, ta_lo, ta_hi)



      implicit none

      character*256   input_file
      INTEGER*4 iunit, NSCANS, NSCANS2
      character*3  satname     ! currently 'F13', 'F14', or 'F15'
      integer*4    itime87_ARR(1700)
      integer*2    stdtime_A(1700, 6)  ! YYYY MM DD HH MM SS
      character*24  temptime

      INTEGER*4 KSAT,IORBIT,NUMSCAN 
      CHARACTER*24 BTIME(1700)
      INTEGER*1 IFLAGS(12,1700),ISUR(128,2,1700),IICE(128,2,1700)
      REAL*4 FRCREV(1700),SCLAT(1700),SCLON(1700),SCALT(1700)
      REAL*4 XLAT(128,2,1700), XLON(128,2,1700), THT(128,1700),
     &       AZIMUTH(128,1700), SUNANG(128,1700)
      REAL*4 TA_LO(5,64,1700),TA_HI(6:7,128,2,1700), TB_LO(5,64,1700)


      CHARACTER*1 ABLANK(3756)
      CHARACTER*24 ZTIME
      INTEGER*1 ZFLAGS(12),ZTOIL(128,2)
      INTEGER*4 ILU,I,ISCAN,ZFRCREV,ZSCLAT,ZSCLON,ZSCALT,J,K
      INTEGER*2 ZLAT(128,2), ZLON(128,2), ZTHT(128), ZAZIMUTH(128),
     &          ZSUNANG(128), ZTA_LO(5,64),ZTA_HI(6:7,128,2)





      OPEN(iunit, FILE=input_file, STATUS='OLD',
     &     ACCESS='DIRECT', RECL=3768, FORM='UNFORMATTED')


c     Read header
      READ(iunit,REC=1) KSAT,IORBIT,NUMSCAN,ABLANK



      DO I=1,NUMSCAN
         READ(ILU,REC=I+1)
     &       ISCAN,ZTIME,ZFLAGS,ZFRCREV,ZSCLAT,ZSCLON,ZSCALT,ZLAT,
     &       ZLON,ZTOIL,ZTHT,ZAZIMUTH,ZSUNANG,ZTA_LO,ZTA_HI 
         DO J = 1, 24
            IF (ZTIME(J:J) .EQ. ' ') ZTIME(J:J) = '0'
         END DO

         BTIME(I)      = ZTIME! SCAN TIME
      DO J = 1, 12 
         IFLAGS(J,I)= ZFLAGS(J)! QUALITY FLAGS
      END DO

      IF (ZFLAGS(1).EQ.1) THEN !MISSING SCAN
         FRCREV(I)     = 0     ! FRACTIONAL ORBIT POSITION
         SCLAT(I)      = 0     ! SPACECRAFT LATITUDE  (DEG)
         SCLON(I)      = 0     ! SPACECRAFT LONGITUDE (DEG)
         SCALT(I)      = 0     ! SPACECRAFT ALTITUDE (KM)
         DO J = 1, 128
            THT(J,I)      = 0     ! CELL INCIDENCE ANGLE (DEG)
            AZIMUTH(J,I)  = 0     ! CELL AZIMUTH ANGLE (DEG)
            SUNANG(J,I)   = 0     ! CELL SUN GLITTER ANGLE (DEG)
            DO K = 1, 2
               XLAT(J,K,I)   = 0     ! CELL LATITUDES (DEG)
               XLON(J,K,I)   = 0     ! CELL EAST LONGITUDE (DEG)
               IICE(J,K,I)   = 0     ! SEA-ICE FLAG
               ISUR(J,K,I)   = 0     ! OCEAN, COAST, LAND FLAG
            END DO
         END DO
         DO J = 1, 64
         DO K = 1,  5
            TA_LO(K,J,I)  = 0     ! CELL ANTENNA TEMPERATURES (19V, 19H,
C                                 !                       22V, 37V, 37H)
         END DO
         END DO
         DO J = 1, 128
         DO K = 6,   7
            TA_HI(K,J,1,I) = 0    ! CELL ANTENNA TEMPERATURES (85V,
            TA_HI(K,J,2,I) = 0    !                            85H)
         END DO
         END DO
C
      ELSE
         FRCREV(I)     = 1.E-6*ZFRCREV     ! FRACTIONAL ORBIT POSITION
         SCLAT(I)      = 1.E-3*ZSCLAT      ! SPACECRAFT LATITUDE  (DEG)
         SCLON(I)      = 1.E-3*ZSCLON      ! SPACECRAFT LONGITUDE (DEG)
         SCALT(I)      = 1.E-3*ZSCALT      ! SPACECRAFT ALTITUDE (KM)

         DO J = 1, 128
            THT(J,I)     = 2.E-3*ZTHT(J) + 45. ! CELL INCIDENCE ANGLE (DEG)
            AZIMUTH(J,I) = 0.01*ZAZIMUTH(J) + 180. ! CELL AZIMUTH ANGLE (DEG)
            SUNANG(J,I)  = 0.01*ZSUNANG(J)     ! CELL SUN GLITTER ANGLE (DEG)
            DO K = 1, 2
               XLAT(J,K,I) = 0.01*ZLAT(J,K)     ! CELL LATITUDES (DEG)
               XLON(J,K,I) = 0.01*ZLON(J,K) + 180. ! CELL E LONG (DEG)
               IICE(J,K,I) = INT(ZTOIL(J,K)/4)  ! SEA-ICE FLAG
               ISUR(J,K,I) = ZTOIL(J,K) - 4*INT(ZTOIL(J,K)/4)
C                                               !OCEAN, COAST, LAND FLAG
            END DO
         END DO
         DO J = 1, 64
         DO K = 1,  5
            TA_LO(K,J,I) = 0.01*ZTA_LO(K,J)  ! CELL ANTENNA TEMPERATURES
C                                            ! (19V, 19H, 22V, 37V, 37H)
         END DO
         END DO
         DO J = 1, 128
         DO K = 6,   7
            TA_HI(K,J,1,I) = 0.01*ZTA_HI(K,J,1)  ! CELL ANTENNA (85V,
            TA_HI(K,J,2,I) = 0.01*ZTA_HI(K,J,2)  ! TEMPERATURES  85H)
         END DO
         END DO
      ENDIF

      END DO




c     Get parameter to output that are used in filename
      write (satname, "(A1,I2)") "F", ksat

      do i=1,NUMSCAN
         if ((i<=numscan)) then
            temptime= BTIME(i)
c     set to zero if temptime is ==_FillValue=0
            if (temptime(1:1).EQ."0") then
               stdtime_A(i,1)=-9
               stdtime_A(i,2)=-9
               stdtime_A(i,3)=-9
               stdtime_A(i,4)=-9
               stdtime_A(i,5)=-9
               stdtime_A(i,6)=-9
            else
               read(temptime(1:4),*) stdtime_A(i,1)
               read(temptime(8:9),*) stdtime_A(i,2)
               read(temptime(10:11),*) stdtime_A(i,3)
               read(temptime(12:13),*) stdtime_A(i,4)
               read(temptime(14:15),*) stdtime_A(i,5)
               read(temptime(16:17),*) stdtime_A(i,6)
            end if
         end if
      end do


c     Close file
      CLOSE(iunit)



      end subroutine Readbin_RSS_SSMI_Tb_v6










