c     This file contains 3 subroutines that (1) read the 
c     Netcdf file to get the number of scans, (2) read
c     all data in the file and (3) handle errors


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine ReadNC_RSS_SSMI_Tb_v6_nscans(input_file, 
     &     nscan_lores, nscan_hires)
c     Code to read number of scans in current NCDC RSS NetCDF4 file
c     This avoids the need for allocatable arrays and allows the use
c     of F77 rather than F90

      implicit none

      INCLUDE 'netcdf.inc'

      character*256   input_file
      integer status, ncid

      integer   nscan_lores_id
      integer   nscan_hires_id
      integer*4 nscan_lores
      integer*4 nscan_hires


c     Open file
      status = NF_OPEN(input_file, NF_NOWRITE, ncid)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NSCANS NFOPEN')

c     get dim ids
      status = NF_INQ_DIMID(ncid, 'nscan_lores', nscan_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NSCANS')
      status = NF_INQ_DIMID(ncid, 'nscan_hires', nscan_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NSCANS')

c     Read number of scans - dimension variable
      status = NF_INQ_DIMLEN(ncid, nscan_lores_id, nscan_lores)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NSCANS')
      status = NF_INQ_DIMLEN(ncid, nscan_hires_id, nscan_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NSCANS')

c     Close File
      status = NF_CLOSE(ncid)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NSCANS NFCLOSE')

      end subroutine ReadNC_RSS_SSMI_Tb_v6_nscans




cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine ReadNC_RSS_SSMI_Tb_v6(input_file, NSCANS, NSCANS2,
     &     satname, itime87_ARR, stdtime_A,
     &     iflags, frcrev, xlat, xlon, ta_lo, ta_hi)
c     Code to read NCDC RSS NetCDF4 file
c     This code uses names that reflect those used in
c     READ_SSMI_L1A.F; a FORTRAN code to read RSS 
c     data into a common block of data for GPROF

      implicit none

      INCLUDE 'netcdf.inc'

      character*256 input_file
      character*256 temptxt
      character*20  temptime
      integer status, ncid
      integer i, j, k, idx

c     Define some basic parameters of the file
      integer*4    NSCANS, NSCANS2
      integer    LOPIX, HIPIX, MAX_SCANS
      parameter (LOPIX=64, HIPIX=128, MAX_SCANS=1700)

c     Define numscans (dimension variable)
      integer   nscan_lores_id
      integer   nscan_hires_id
      integer*4 nscan_lores
      integer*4 nscan_hires

c     cccccccccccccccccccccccccccccccccccccccccc
c     Define id variables used for reading
      integer quality_flags_id
c      integer scan_time_lores_id
      integer scan_time_hires_id
      integer scan_time_since87_lores_id
c      integer scan_time_since87_hires_id
c      integer latitude_lores_id
c      integer longitude_lores_id
      integer latitude_hires_id
      integer longitude_hires_id
      integer antenna_temperature_19V_id
      integer antenna_temperature_19H_id
      integer antenna_temperature_22V_id
      integer antenna_temperature_37V_id
      integer antenna_temperature_37H_id
      integer antenna_temperature_85V_id
      integer antenna_temperature_85H_id
c      integer fcdr_brightness_temperature_19V_id
c      integer fcdr_brightness_temperature_19H_id
c      integer fcdr_brightness_temperature_22V_id
c      integer fcdr_brightness_temperature_37V_id
c      integer fcdr_brightness_temperature_37H_id
c      integer fcdr_brightness_temperature_85V_id
c      integer fcdr_brightness_temperature_85H_id
c      integer surface_type_lores_id
c      integer surface_type_hires_id
c      integer sea_ice_hires_id
      integer fractional_orbit_position_id
c      integer spacecraft_latitude_id
c      integer spacecraft_longitude_id
c      integer spacecraft_altitude_id
c      integer earth_incidence_angle_hires_id
c      integer earth_azimuth_angle_hires_id
c      integer earth_sunglitter_angle_hires_id

c     cccccccccccccccccccccccccccccccccccccccccc
c     Define variables to be read
      character*256 platform, time_coverage_start, id
      integer*1     quality_flags(12, NSCANS)
c      character*20  scan_time_lores(NSCANS)
      character*20  scan_time_hires(NSCANS2)
      real*8        scan_time_since87_lores(NSCANS)
c      real*8        scan_time_since87_hires(NSCANS2)
c      integer*2     latitude_lores(LOPIX, NSCANS)
c      integer*2     longitude_lores(LOPIX, NSCANS)
      integer*2     latitude_hires(HIPIX, NSCANS2)
      integer*2     longitude_hires(HIPIX, NSCANS2)
      integer*2     antenna_temperature_19V(LOPIX, NSCANS)
      integer*2     antenna_temperature_19H(LOPIX, NSCANS)
      integer*2     antenna_temperature_22V(LOPIX, NSCANS)
      integer*2     antenna_temperature_37V(LOPIX, NSCANS)
      integer*2     antenna_temperature_37H(LOPIX, NSCANS)
      integer*2     antenna_temperature_85V(HIPIX, NSCANS2)
      integer*2     antenna_temperature_85H(HIPIX, NSCANS2)
c      integer*2     fcdr_brightness_temperature_19V(LOPIX, NSCANS)
c      integer*2     fcdr_brightness_temperature_19H(LOPIX, NSCANS)
c      integer*2     fcdr_brightness_temperature_22V(LOPIX, NSCANS)
c      integer*2     fcdr_brightness_temperature_37V(LOPIX, NSCANS)
c      integer*2     fcdr_brightness_temperature_37H(LOPIX, NSCANS)
c      integer*2     fcdr_brightness_temperature_85V(HIPIX, NSCANS2)
c      integer*2     fcdr_brightness_temperature_85H(HIPIX, NSCANS2)
c      integer*1     surface_type_lores(LOPIX, NSCANS)
c      integer*1     surface_type_hires(HIPIX, NSCANS2)
c      integer*1     sea_ice_hires(HIPIX, NSCANS2)
      real*4        fractional_orbit_position(NSCANS)
c      real*4        spacecraft_latitude(NSCANS)
c      real*4        spacecraft_longitude(NSCANS)
c      real*4        spacecraft_altitude(NSCANS)
c      integer*2     earth_incidence_angle_hires(HIPIX, NSCANS)
c      real*4        earth_azimuth_angle_hires(HIPIX, NSCANS)
c      integer*2     earth_sunglitter_angle_hires(HIPIX, NSCANS)


c     cccccccccccccccccccccccccccccccccccccccccc
c     Define output variables (formerly passed via "common" command)
      logical      usescn(MAX_SCANS)
c      integer*4    ksat
      character*3  satname     ! currently 'F13', 'F14', or 'F15' 
c      integer*4    iorbit
c      integer*4    numscan
c      character*24 btime(MAX_SCANS)
c      integer*4    iyear
c      integer*4    immdd
c      integer*4    iymd
c      integer*4    ihms
      integer*4    itime87_ARR(MAX_SCANS)
      integer*2    stdtime_A(1700,6)
      integer*1    iflags(12, MAX_SCANS)
c      integer*1    isur(HIPIX,2, MAX_SCANS)
c      integer*1    iice(HIPIX,2, MAX_SCANS)
      real*4       frcrev(MAX_SCANS)
c      real*4       sclat(MAX_SCANS)
c      real*4       sclon(MAX_SCANS)
c      real*4       scalt(MAX_SCANS)
      real*4       xlat(HIPIX,2, MAX_SCANS)
      real*4       xlon(HIPIX,2, MAX_SCANS)
c      real*4       that(HIPIX, MAX_SCANS)
c      real*4       azimuth(HIPIX, MAX_SCANS)
c      real*4       sunang(HIPIX, MAX_SCANS)
      real*4       ta_lo(5,LOPIX, MAX_SCANS)
      real*4       ta_hi(2,HIPIX,2, MAX_SCANS)
c      real*4       tb_lo(5,LOPIX, MAX_SCANS)
c      real*4       tb_hi(2,HIPIX,2, MAX_SCANS)


c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Open file
      status = NF_OPEN(input_file, NF_NOWRITE, ncid)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NF_OPEN')

c     get global attributes
      status = NF_GET_ATT_TEXT (ncid, NF_GLOBAL, 'platform', platform)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_ATT_TEXT platform')
      status = NF_GET_ATT_TEXT (ncid, NF_GLOBAL, 'id', id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_ATT_TEXT id')
      status = NF_GET_ATT_TEXT (ncid, NF_GLOBAL, 'time_coverage_start',
     &     time_coverage_start)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_ATT_TEXT time_coverage_start')

c     cccccccccccccccccccccccccccccccccccccccccc
c     Define variable ids
      status = NF_INQ_DIMID(ncid, 'nscan_lores', nscan_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_DIMID nscan_lores')
      status = NF_INQ_DIMID(ncid, 'nscan_hires', nscan_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_DIMID nscan_hires')

      status = NF_INQ_VARID(ncid, 'quality_flags', quality_flags_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID quality_flags')
c      status = NF_INQ_VARID(ncid, 'scan_time_lores',scan_time_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID scan_time_lores')
      status = NF_INQ_VARID(ncid, 'scan_time_hires', scan_time_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID scan_time_hires')
      status = NF_INQ_VARID(ncid, 'scan_time_since87_lores',
     &     scan_time_since87_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID scan_time_since87_lores')
c      status = NF_INQ_VARID(ncid, 'scan_time_since87_hires',
c     &     scan_time_since87_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID scan_time_since87_hires')
c      status = NF_INQ_VARID(ncid, 'latitude_lores', latitude_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID latitude_lores')
c      status = NF_INQ_VARID(ncid, 'longitude_lores',longitude_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID longitude_lores')
      status = NF_INQ_VARID(ncid, 'latitude_hires', latitude_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID latitude_hires')
      status = NF_INQ_VARID(ncid, 'longitude_hires', longitude_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID longitude_hires')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_19V',
     &     antenna_temperature_19V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_19V')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_19H',
     &     antenna_temperature_19H_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_19H')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_22V',
     &     antenna_temperature_22V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_22V')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_37V',
     &     antenna_temperature_37V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_37V')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_37H',
     &     antenna_temperature_37H_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_37H')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_85V',
     &     antenna_temperature_85V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_85V')
      status = NF_INQ_VARID(ncid, 'antenna_temperature_85H',
     &     antenna_temperature_85H_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID antenna_temperature_85H')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_19V',
c     &     fcdr_brightness_temperature_19V_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_19V')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_19H',
c     &     fcdr_brightness_temperature_19H_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_19H')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_22V',
c     &     fcdr_brightness_temperature_22V_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_22V')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_37V',
c     &     fcdr_brightness_temperature_37V_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_37V')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_37H',
c     &     fcdr_brightness_temperature_37H_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_37H')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_85V',
c     &     fcdr_brightness_temperature_85V_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_85V')
c      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_85H',
c     &     fcdr_brightness_temperature_85H_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID fcdr_brightness_temperature_85H')
c      status = NF_INQ_VARID(ncid, 'surface_type_lores',
c     &     surface_type_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID surface_type_lores')
c      status = NF_INQ_VARID(ncid, 'surface_type_hires',
c     &     surface_type_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID surface_type_hires')
c      status = NF_INQ_VARID(ncid, 'sea_ice_hires', sea_ice_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID sea_ice_hires')
      status = NF_INQ_VARID(ncid, 'fractional_orbit_position',
     &     fractional_orbit_position_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fractional_orbit_position')
c      status = NF_INQ_VARID(ncid, 'spacecraft_latitude',
c     &     spacecraft_latitude_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID spacecraft_latitude')
c      status = NF_INQ_VARID(ncid, 'spacecraft_longitude',
c     &     spacecraft_longitude_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID spacecraft_longitude')
c      status = NF_INQ_VARID(ncid, 'spacecraft_altitude',
c     &     spacecraft_altitude_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID spacecraft_altitude')
c      status = NF_INQ_VARID(ncid, 'earth_incidence_angle_hires',
c     &     earth_incidence_angle_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_incidence_angle_hires')
c      status = NF_INQ_VARID(ncid, 'earth_azimuth_angle_hires',
c     &     earth_azimuth_angle_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_azimuth_angle_hires')
c      status = NF_INQ_VARID(ncid, 'earth_sunglitter_angle_hires',
c     &     earth_sunglitter_angle_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_sunglitter_angle_hires')

c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Read data

c     Read number of scans - dimension variable
      status = NF_INQ_DIMLEN(ncid, nscan_lores_id, nscan_lores)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_DIMLEN nscan_lores')
      status = NF_INQ_DIMLEN(ncid, nscan_hires_id, nscan_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_DIMLEN nscan_hires')
c     Check number of scans is less than MAX_SCANS
      if (nscan_lores>NSCANS) then
         write(*,*) "Too many scans in file. Aborting..."
         stop
      end if

c     read rest of data
      status = NF_GET_VAR_INT1(ncid, quality_flags_id, quality_flags)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT1 quality_flags')
c      status = NF_GET_VAR_TEXT(ncid, scan_time_lores_id,
c     &     scan_time_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_TEXT scan_time_lores')
      status = NF_GET_VAR_TEXT(ncid, scan_time_hires_id,
     &     scan_time_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_TEXT scan_time_hires')
      status = NF_GET_VAR_DOUBLE(ncid, scan_time_since87_lores_id,
     &     scan_time_since87_lores)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_DOUBLE scan_time_since87_lores')
c      status = NF_GET_VAR_DOUBLE(ncid, scan_time_since87_hires_id,
c     &     scan_time_since87_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_DOUBLE scan_time_since87_hires')
c      status = NF_GET_VAR_INT2(ncid, latitude_lores_id, latitude_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 latitude_lores')
c      status = NF_GET_VAR_INT2(ncid,longitude_lores_id,longitude_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 longitude_lores')
      status = NF_GET_VAR_INT2(ncid, latitude_hires_id, latitude_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 latitude_hires')
      status = NF_GET_VAR_INT2(ncid, longitude_hires_id,
     &     longitude_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 longitude_hires')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_19V_id,
     &    antenna_temperature_19V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_19V')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_19H_id,
     &     antenna_temperature_19H)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_19H')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_22V_id,
     &     antenna_temperature_22V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_22V')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_37V_id,
     &     antenna_temperature_37V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_37V')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_37H_id,
     &     antenna_temperature_37H)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_37H')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_85V_id,
     &     antenna_temperature_85V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_85V')
      status = NF_GET_VAR_INT2(ncid, antenna_temperature_85H_id,
     &     antenna_temperature_85H)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 antenna_temperature_85H')
c      status = NF_GET_VAR_INT2(ncid, 
c     &     fcdr_brightness_temperature_19V_id, 
c     &     fcdr_brightness_temperature_19V)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_19V')
c      status = NF_GET_VAR_INT2(ncid,
c     &     fcdr_brightness_temperature_19H_id, 
c     &     fcdr_brightness_temperature_19H)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_19H')
c      status = NF_GET_VAR_INT2(ncid,
c     &     fcdr_brightness_temperature_22V_id, 
c     &     fcdr_brightness_temperature_22V)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_22V')
c      status = NF_GET_VAR_INT2(ncid,
c     &     fcdr_brightness_temperature_37V_id, 
c     &     fcdr_brightness_temperature_37V)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_37V')
c      status = NF_GET_VAR_INT2(ncid,
c     &     fcdr_brightness_temperature_37H_id, 
c     &     fcdr_brightness_temperature_37H)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_37H')
c      status = NF_GET_VAR_INT2(ncid,
c     &     fcdr_brightness_temperature_85V_id, 
c     &     fcdr_brightness_temperature_85V)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_85V')
c      status = NF_GET_VAR_INT2(ncid, 
c     &     fcdr_brightness_temperature_85H_id, 
c     &     fcdr_brightness_temperature_85H)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 fcdr_brightness_temperature_85H')
c      status = NF_GET_VAR_INT1(ncid, surface_type_lores_id,
c     &     surface_type_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT1 surface_type_lores')
c      status = NF_GET_VAR_INT1(ncid, surface_type_hires_id,
c     &     surface_type_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT1 surface_type_hires')
c      status = NF_GET_VAR_INT1(ncid, sea_ice_hires_id, sea_ice_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT1 sea_ice_hires')
      status = NF_GET_VAR_REAL(ncid, fractional_orbit_position_id,
     &     fractional_orbit_position)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fractional_orbit_position')
c      status = NF_GET_VAR_REAL(ncid, spacecraft_latitude_id,
c     &     spacecraft_latitude)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL spacecraft_latitude')
c      status = NF_GET_VAR_REAL(ncid, spacecraft_longitude_id,
c     &     spacecraft_longitude)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL spacecraft_longitude')
c      status = NF_GET_VAR_REAL(ncid, spacecraft_altitude_id, 
c     &     spacecraft_altitude)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL spacecraft_altitude')
c      status = NF_GET_VAR_INT2(ncid, earth_incidence_angle_hires_id,
c     &     earth_incidence_angle_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 earth_incidence_angle_hires')
c      status = NF_GET_VAR_REAL(ncid, earth_azimuth_angle_hires_id,
c     &     earth_azimuth_angle_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL earth_azimuth_angle_hires')
c      status = NF_GET_VAR_INT2(ncid, earth_sunglitter_angle_hires_id,
c     &     earth_sunglitter_angle_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 earth_sunglitter_angle_hires_id')


c     Close File
      status = NF_CLOSE(ncid)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NF_CLOSE')



c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Copy required output data

c     Convert text from GlobAtts to integers
      idx=index(platform,'F')
c      read(platform((idx+1):(idx+2)),*) ksat
      satname=platform(idx:(idx+2))
c      idx=index(id,'_R')
c      read(id((idx+2):(idx+6)),*) iorbit
c      read(time_coverage_start(1:4),*) iyear
c      temptxt=time_coverage_start(6:7)//time_coverage_start(9:10)
c      read(temptxt,*) immdd
c      temptxt=time_coverage_start(1:4)//time_coverage_start(6:7)//
c     &     time_coverage_start(9:10)
c      read(temptxt,*) iymd
c      temptxt=time_coverage_start(12:13)//time_coverage_start(15:16)//
c     &     time_coverage_start(18:19)
c      read(temptxt,*) ihms

      do i=1,MAX_SCANS
         if ((i<=nscan_lores)) then
            usescn=.TRUE.
            itime87_ARR(i)=int(scan_time_since87_lores(i))

c           Get scan times from hires vector so as to get b-scan times
            temptime=scan_time_hires(i*2)
c           set to zero if temptime is ==_FillValue=0
            if (temptime(1:1).EQ."0") then
               stdtime_A(i,1)=-9
               stdtime_A(i,2)=-9
               stdtime_A(i,3)=-9
               stdtime_A(i,4)=-9
               stdtime_A(i,5)=-9
               stdtime_A(i,6)=-9
c               btime(i)="00000000000000000.000000"
            else
               read(temptime(1:4),*) stdtime_A(i,1)
               read(temptime(6:7),*) stdtime_A(i,2)
               read(temptime(9:10),*) stdtime_A(i,3)
               read(temptime(12:13),*) stdtime_A(i,4)
               read(temptime(15:16),*) stdtime_A(i,5)
               read(temptime(18:19),*) stdtime_A(i,6)
c               btime(i)=temptime(1:4)//'000'//temptime(6:7)//
c     &              temptime(9:10)//temptime(12:13)//temptime(15:16)//
c     &              temptime(18:19)//'.000000'
            end if

            frcrev(i)=fractional_orbit_position(i)
c            sclat(i)=spacecraft_latitude(i)
c            if (spacecraft_longitude(i)<0) then
c              sclon(i)=spacecraft_longitude(i)+360
c            else
c              sclon(i)=spacecraft_longitude(i)
c            end if
c            scalt(i)=spacecraft_altitude(i)
            do j=1,12
               iflags(j,i)=quality_flags(i,j)
            end do
            do j=1,LOPIX
               ta_lo(1,j,i)=real(antenna_temperature_19V(j,i))*.01
               ta_lo(2,j,i)=real(antenna_temperature_19H(j,i))*.01
               ta_lo(3,j,i)=real(antenna_temperature_22V(j,i))*.01
               ta_lo(4,j,i)=real(antenna_temperature_37V(j,i))*.01
               ta_lo(5,j,i)=real(antenna_temperature_37H(j,i))*.01
c               tb_lo(1,j,i)=
c     &              real(fcdr_brightness_temperature_19V(j,i))*.01
c               tb_lo(2,j,i)=
c     &              real(fcdr_brightness_temperature_19H(j,i))*.01
c               tb_lo(3,j,i)=real(
c     &              fcdr_brightness_temperature_22V(j,i))*.01
c               tb_lo(4,j,i)=real(
c     &              fcdr_brightness_temperature_37V(j,i))*.01
c               tb_lo(5,j,i)=real(
c     &              fcdr_brightness_temperature_37H(j,i))*.01
            end do
            do j=1,HIPIX
c               that(j,i)=real(earth_incidence_angle_hires(j,i))*.01
c               azimuth(j,i)= earth_azimuth_angle_hires(j,i)
c               sunang(j,i)=real(earth_sunglitter_angle_hires(j,i))*.01
               do k=1,2
c                  isur(j,k,i)=surface_type_hires(j,(j-1)*2+k)
c                  iice(j,k,i)=sea_ice_hires(j,(i-1)*2+k)
                  xlat(j,k,i)=latitude_hires(j,(i-1)*2+k)*.01
                  xlon(j,k,i)=longitude_hires(j,(i-1)*2+k)*.01
                  if (xlon(j,k,i) .lt. 0) then
                     xlon(j,k,i)=360+xlon(j,k,i)
                  end if
                  ta_hi(1,j,k,i)=
     &                 real(antenna_temperature_85V(j,(i-1)*2+k))*.01
                  ta_hi(2,j,k,i)=
     &                 real(antenna_temperature_85H(j,(i-1)*2+k))*.01
c                  tb_hi(1,j,k,i)=real(fcdr_brightness_temperature_85V
c     &                 (j,(i-1)*2+k))*.01
c                  tb_hi(2,j,k,i)=real(fcdr_brightness_temperature_85H
c     &                 (j,(i-1)*2+k))*.01
               end do
            end do
         else
            usescn=.FALSE.
c            btime(i)='                        '
            itime87_ARR(i)=-999
            frcrev(i)=-999.9
c            sclat(i)=-999.9
c            sclon(i)=-999.9
c            scalt(i)=-999.9
            do j=1,12
               iflags(j,i)=-9
            end do
            do j=1,LOPIX
               do k=1,5
                  ta_lo(k,j,i)=-999.9
c                  tb_lo(k,j,i)=-999.9
               end do
            end do
            do j=1,HIPIX
c               that(j,i)=-999.9
c               azimuth(j,i)=-999.9
c               sunang(j,i)=-999.9
               do k=1,2
c                  isur(j,k,i)=-9
c                  iice(j,k,i)=-9
                  xlat(j,k,i)=-999.9
                  xlon(j,k,i)=-999.9
                  ta_hi(1,j,k,i)=-999.9
                  ta_hi(2,j,k,i)=-999.9
c                  tb_hi(1,j,k,i)=-999.9
c                  tb_hi(2,j,k,i)=-999.9
               end do
            end do
         end if
      end do

      end subroutine ReadNC_RSS_SSMI_Tb_v6



      SUBROUTINE HANDLE_ERR(status, errmessage)
      character*256 errmessage
      INTEGER status

      IF (STATUS .NE. NF_NOERR) THEN
         PRINT *, NF_STRERROR(STATUS), ':', errmessage
         STOP "Stopping due to NetCDF errors in ReadNC_RSS_SSMI_Tb_v6"
      ENDIF
      END








