c     This file contains 5 subroutines that (1) read the 
c     Netcdf file to get the number of scans, (2) read
c     all data in the file, (3) handle errors, (4) read
c     v7->v6 conversion coeffs and (5) apply v7->v6 conversion


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine ReadNC_RSS_SSMIS_Tb_v6_nscans(input_file, nscan)
c     Code to read number of scans in current NCDC RSS NetCDF4 file
c     This avoids the need for allocatable arrays and allows the use
c     of F77 rather than F90

      implicit none

      INCLUDE 'netcdf.inc'

      character*256   input_file
      integer status, ncid

      integer   nscan_id
      integer nscan

c     Open file
      status = NF_OPEN(input_file, NF_NOWRITE, ncid)
      IF (STATUS .NE. NF_NOERR) THEN
         PRINT *, NF_STRERROR(STATUS), ': NSCANS NFOPEN'
         PRINT *, "Skipping file - error reading Nscans (may be empty)"
         nscan=0; return
      ENDIF

c     get dim id
      status = NF_INQ_DIMID(ncid, 'scan_number', nscan_id)
      IF (STATUS .NE. NF_NOERR) THEN
         PRINT *, NF_STRERROR(STATUS), ': NSCANS READ'
         PRINT *, "Skipping file - error reading Nscans (may be empty)"
         nscan=0; return
      ENDIF

c     Read number of scans - dimension variable
      status = NF_INQ_DIMLEN(ncid, nscan_id, nscan)
      IF (STATUS .NE. NF_NOERR) THEN
         PRINT *, NF_STRERROR(STATUS), ': NSCANS DIMLEN'
         PRINT *, "Skipping file - error reading Nscans (may be empty)"
         nscan=0; return
      ENDIF

c     Close File
      status = NF_CLOSE(ncid)
      IF (STATUS .NE. NF_NOERR) THEN
         PRINT *, NF_STRERROR(STATUS), ': NSCANS NFCLOSE'
         PRINT *, "Skipping file - error reading Nscans (may be empty)"
         nscan=0; return
      ENDIF

      end subroutine ReadNC_RSS_SSMIS_Tb_v6_nscans




cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine ReadNC_RSS_SSMIS_Tb_v6(input_file, iver, nscans,
     &     sat_name, stdtime_A, btime, frcrev, itime87_ARR, startdate, 
     &     TB_LO, TB_HI, XLAT_LO, XLON_LO, XLAT_HI, 
     &     XLON_HI, iscn_flag, ical_flag, ical_flag_hi, lnd, lnd_hi)

c     Code to read NCDC RSS NetCDF4 file
c     This code uses names that reflect those used in
c     READ_L1B_FILE.F; a FORTRAN code to read RSS SSMIS
c     data into a common block of data for GPROF
c
c   List of output values
C*  btime        CHAR   array of scan times (YYYYJJJMMDDHHMMSS.SSSSSS) *
C*  tb_lo        REAL   low-frequency brightness temperature array     *
C*  tb_hi        REAL   high-frequency brightness temperature array    *
C*  xlat_lo      REAL   low-frequency (19/22 and 37) latitude array    *
C*  xlon_lo      REAL   low-frequency (19/22 and 37) longitude array   *
C*  xlat_hi      REAL   high-frequency (91) latitude array             *
C*  xlon_hi      REAL   high-frequency (91) longitude array            *
C*  iscn_flag    INT    quality flag #1 array                          *
C*  ical_flag    INT    quality flag #2 array                          *
C*  ical_flag_hi INT    quality flag #3 array                          *
C*  lnd          INT    surface flag, low-resolution                   *
C*  lnd_hi       INT    surface flag, high-resolution                  *



      implicit none

      include 'parameters.inc'
      INCLUDE 'netcdf.inc'

      character*256 input_file
      character*256 temptxt
      character*20  temptime
      integer status, ncid, iver
      integer i, j, k, idx, N

      integer tmpdate(6), DaysOfYear(12), DaysOfYearL(12)
      real*8  targsecs, tmpsecs, tmpdays
      data DaysOfYear  /31,28,31,30,31,30,31,31,30,31,30,31/
      data DaysOfYearL /31,29,31,30,31,30,31,31,30,31,30,31/


c     Define some basic parameters of the file
      integer    nscans
      integer    LOPIX, HIPIX
      parameter (LOPIX=90, HIPIX=180)


c     cccccccccccccccccccccccccccccccccccccccccc
c     Define id variables used for reading
      integer iorbit_id
      integer scan_time_hires_id
      integer orbit_position_id
c      integer sc_lat_id
c      integer sc_lon_id
c      integer sc_alt_id
      integer iscn_flag_id
      integer ical_flag_lores_id
      integer ical_flag_hires_id
      integer latitude_hires_id
      integer longitude_hires_id
c      integer earth_incidence_angle_hires_id
c      integer earth_azimuth_angle_hires_id
c      integer sun_glitter_angle_hires_id
      integer land_flag_hires_id
c      integer ice_flag_hires_id
      integer fcdr_brightness_temperature_92V_id
      integer fcdr_brightness_temperature_92H_id
      integer latitude_lores_id
      integer longitude_lores_id
c      integer earth_incidence_angle_lores_id
c      integer earth_azimuth_angle_lores_id
c      integer sun_glitter_angle_lores_id
      integer land_flag_lores_id
c      integer ice_flag_lores_id
      integer fcdr_brightness_temperature_19V_id
      integer fcdr_brightness_temperature_19H_id
      integer fcdr_brightness_temperature_22V_id
      integer fcdr_brightness_temperature_37V_id
      integer fcdr_brightness_temperature_37H_id



c     cccccccccccccccccccccccccccccccccccccccccc
c     Define variables to be read
      character*256 platform, time_coverage_start, id
      integer   iorbit
      real*8    scan_time_hires(nscans)
      real*8    orbit_position(nscans)
c      real*4    sc_lat(nscans)
c      real*4    sc_lon(nscans)
c      real*4    sc_alt(nscans)
      integer*1 iscn_flag_in(nscans, 11)
      integer*1 ical_flag_lores(nscans, 4)
      integer*1 ical_flag_hires(nscans, 4)
      integer*2 latitude_hires(HIPIX, nscans)
      integer*2 longitude_hires(HIPIX, nscans)
c      integer*2 earth_incidence_angle_hires(HIPIX, nscans)
c      integer*2 earth_azimuth_angle_hires(HIPIX, nscans)
c      integer*2 sun_glitter_angle_hires(HIPIX, nscans)
      integer*1 land_flag_hires(HIPIX, nscans)
c      integer*1 ice_flag_hires(HIPIX, nscans)
      real*4    fcdr_brightness_temperature_92V(HIPIX, nscans)
      real*4    fcdr_brightness_temperature_92H(HIPIX, nscans)
      integer*2 latitude_lores(LOPIX, nscans)
      integer*2 longitude_lores(LOPIX, nscans)
c      integer*2 earth_incidence_angle_lores(LOPIX, nscans)
c      integer*2 earth_azimuth_angle_lores(LOPIX, nscans)
c      integer*2 sun_glitter_angle_lores(LOPIX, nscans)
      integer*1 land_flag_lores(LOPIX, nscans)
c      integer*2 ice_flag_lores(LOPIX, nscans)
      real*4    fcdr_brightness_temperature_19V(LOPIX, nscans)
      real*4    fcdr_brightness_temperature_19H(LOPIX, nscans)
      real*4    fcdr_brightness_temperature_22V(LOPIX, nscans)
      real*4    fcdr_brightness_temperature_37V(LOPIX, nscans)
      real*4    fcdr_brightness_temperature_37H(LOPIX, nscans)


c     cccccccccccccccccccccccccccccccccccccccccc
c     Define output variables
      integer      startdate(6)
      CHARACTER*24 BTIME(MAX_SCANS), ctime
      character*3  sat_name
      integer*2    stdtime_A(MAX_SCANS,6)
      integer      itime87_ARR(MAX_SCANS)
      real*8       frcrev(MAX_SCANS)
      real*4       tb_lo(5, LOPIX,MAX_SCANS)
      real*4       tb_hi(2, HIPIX,MAX_SCANS)
      real*4       xlat_lo(LOPIX,MAX_SCANS)
      real*4       xlon_lo(LOPIX,MAX_SCANS)
      real*4       xlat_hi(HIPIX,MAX_SCANS)
      real*4       xlon_hi(HIPIX,MAX_SCANS)
      integer*4    iscn_flag(MAX_SCANS)
      integer*4    ical_flag(MAX_SCANS)
      integer*4    ical_flag_hi(MAX_SCANS)
      integer*4    lnd(LOPIX,MAX_SCANS)
      integer*4    lnd_hi(HIPIX,MAX_SCANS)


c     If data is v7, then convert to v6
      REAL(4) acoef(3,2,4)
      IF (iver.EQ.1) CALL READ_v7_TO_v6_CONVERSION_TABLE(acoef)



c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Open file
      status = NF_OPEN(input_file, NF_NOWRITE, ncid)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NF_OPEN')

c     get global attributes
      status = NF_GET_ATT_TEXT (ncid, NF_GLOBAL, 'platform', platform)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_ATT_TEXT platform')
      status = NF_GET_ATT_TEXT (ncid, NF_GLOBAL, 'id', id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_ATT_TEXT id')
      status = NF_GET_ATT_TEXT (ncid, NF_GLOBAL, 'time_coverage_start',
     &     time_coverage_start)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_ATT_TEXT time_coverage_start')

c     cccccccccccccccccccccccccccccccccccccccccc
c     Define variable ids
      status = NF_INQ_VARID(ncid, 'iorbit', iorbit_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID iorbit')
      status = NF_INQ_VARID(ncid, 'scan_time_hires', scan_time_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID scan_time_hires')
      status = NF_INQ_VARID(ncid, 'orbit_position', orbit_position_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID orbit_position')
c      status = NF_INQ_VARID(ncid, 'sc_lat', sc_lat_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID sc_lat')
c      status = NF_INQ_VARID(ncid, 'sc_lon', sc_lon_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID sc_lon')
c      status = NF_INQ_VARID(ncid, 'sc_alt', sc_alt_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID sc_alt')
      status = NF_INQ_VARID(ncid, 'iscn_flag', iscn_flag_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID iscn_flag')
      status = NF_INQ_VARID(ncid, 'ical_flag_lores', ical_flag_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID ical_flag_lores')
      status = NF_INQ_VARID(ncid, 'ical_flag_hires', ical_flag_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID ical_flag_hires')
      status = NF_INQ_VARID(ncid, 'latitude_hires', latitude_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID latitude_hires')
      status = NF_INQ_VARID(ncid, 'longitude_hires', longitude_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID longitude_hires')
c      status = NF_INQ_VARID(ncid, 'earth_incidence_angle_hires', earth_incidence_angle_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_incidence_angle_hires')
c      status = NF_INQ_VARID(ncid, 'earth_azimuth_angle_hires',earth_azimuth_angle_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_azimuth_angle_hires')
c      status = NF_INQ_VARID(ncid, 'sun_glitter_angle_hires',sun_glitter_angle_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID sun_glitter_angle_hires')
      status = NF_INQ_VARID(ncid, 'land_flag_hires', land_flag_hires_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID land_flag_hires')
c      status = NF_INQ_VARID(ncid, 'ice_flag_hires', ice_flag_hires_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID ice_flag_hires')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_92V',
     &     fcdr_brightness_temperature_92V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_92V')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_92H',
     &     fcdr_brightness_temperature_92H_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_92H')
      status = NF_INQ_VARID(ncid, 'latitude_lores', latitude_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID latitude_lores')
      status = NF_INQ_VARID(ncid, 'longitude_lores', longitude_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID longitude_lores')
c      status = NF_INQ_VARID(ncid, 'earth_incidence_angle_lores',earth_incidence_angle_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_incidence_angle_lores')
c      status = NF_INQ_VARID(ncid, 'earth_azimuth_angle_lores',earth_azimuth_angle_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID earth_azimuth_angle_lores')
c      status = NF_INQ_VARID(ncid, 'sun_glitter_angle_lores',sun_glitter_angle_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID sun_glitter_angle_lores')
      status = NF_INQ_VARID(ncid, 'land_flag_lores', land_flag_lores_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID land_flag_lores')
c      status = NF_INQ_VARID(ncid, 'ice_flag_lores', ice_flag_lores_id)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_INQ_VARID ice_flag_lores')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_19V',
     &     fcdr_brightness_temperature_19V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_19V')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_19H',
     &     fcdr_brightness_temperature_19H_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_19H')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_22V',
     &     fcdr_brightness_temperature_22V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_22V')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_37V',
     &     fcdr_brightness_temperature_37V_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_37V')
      status = NF_INQ_VARID(ncid, 'fcdr_brightness_temperature_37H',
     &     fcdr_brightness_temperature_37H_id)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_INQ_VARID fcdr_brightness_temperature_37H')








c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Read data
      status = NF_GET_VAR_INT(ncid, iorbit_id, iorbit)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VAR_INT iorbit')
      status = NF_GET_VAR_DOUBLE(ncid, scan_time_hires_id,
     &     scan_time_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_DOUBLE scan_time_hires')
      status = NF_GET_VAR_DOUBLE(ncid, orbit_position_id,
     &     orbit_position)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_DOUBLE orbit_position')
c      status = NF_GET_VAR_REAL(ncid, sc_lat_id, sc_lat)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL sc_lat')
c      status = NF_GET_VAR_REAL(ncid, sc_lon_id, sc_lon)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL sc_lon')
c      status = NF_GET_VAR_REAL(ncid, sc_alt_id, sc_alt)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_REAL sc_alt')
      status = NF_GET_VAR_INT1(ncid, iscn_flag_id, iscn_flag_in)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT1 iscn_flag')
      status = NF_GET_VAR_INT1(ncid, ical_flag_lores_id,
     &     ical_flag_lores)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT1 ical_flag_lores')
      status = NF_GET_VAR_INT1(ncid, ical_flag_hires_id,
     &     ical_flag_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT1 ical_flag_hires')
      status = NF_GET_VAR_INT2(ncid, latitude_hires_id,latitude_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 latitude_hires')
      status = NF_GET_VAR_INT2(ncid, longitude_hires_id,
     &     longitude_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 longitude_hires')
c      status = NF_GET_VAR_INT2(ncid, earth_incidence_angle_hires_id,
c     &      earth_incidence_angle_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 earth_incidence_angle_hires')
c      status = NF_GET_VAR_INT2(ncid, earth_azimuth_angle_hires_id,
c     &      earth_azimuth_angle_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 earth_azimuth_angle_hires')
c      status = NF_GET_VAR_INT2(ncid, sun_glitter_angle_hires_id,
c     &     sun_glitter_angle_hires)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 sun_glitter_angle_hires')
c      status = NF_GET_VAR_INT1(ncid, quality_flags_id, quality_flags)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT1 quality_flags')
      status = NF_GET_VAR_INT1(ncid, land_flag_hires_id, 
     &      land_flag_hires)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT1 land_flag_hires')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_92V_id,
     &     fcdr_brightness_temperature_92V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_92V')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_92H_id,
     &     fcdr_brightness_temperature_92H)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_92H')
      status = NF_GET_VAR_INT2(ncid, latitude_lores_id, latitude_lores)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 latitude_lores')
      status = NF_GET_VAR_INT2(ncid, longitude_lores_id,
     &     longitude_lores)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_INT2 longitude_lores')
c      status = NF_GET_VAR_INT2(ncid, earth_incidence_angle_lores_id,
c     &       earth_incidence_angle_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 earth_incidence_angle_lores')
c      status = NF_GET_VAR_INT2(ncid, earth_azimuth_angle_lores_id,
c     &      earth_azimuth_angle_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 earth_azimuth_angle_lores')
c      status = NF_GET_VAR_INT2(ncid, sun_glitter_angle_lores_id,
c     &       sun_glitter_angle_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 sun_glitter_angle_lores')
c      status = NF_GET_VAR_INT1(ncid, quality_flags_id, quality_flags)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT1 quality_flags')
c      status = NF_GET_VAR_INT2(ncid, ice_flag_lores_id, ice_flag_lores)
c      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
c     &     'NF_GET_VARA_INT2 ice_flag_lores')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_19V_id,
     &     fcdr_brightness_temperature_19V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_19V')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_19H_id,
     &     fcdr_brightness_temperature_19H)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_19H')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_22V_id,
     &     fcdr_brightness_temperature_22V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_22V')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_37V_id,
     &     fcdr_brightness_temperature_37V)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_37V')
      status = NF_GET_VAR_REAL(ncid,fcdr_brightness_temperature_37H_id,
     &     fcdr_brightness_temperature_37H)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,
     &     'NF_GET_VARA_REAL fcdr_brightness_temperature_37H')

c     Close File
      status = NF_CLOSE(ncid)
      if (status .NE. NF_NOERR) call HANDLE_ERR(status,'NF_CLOSE')




c     cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     Copy required output data

c     Convert text from GlobAtts to integers

c     Get sat_name
      idx=index(platform,'F')
      sat_name=platform(idx:(idx+2))

c     Get start date as integers
      read(time_coverage_start(1:4),*)   startdate(1)
      read(time_coverage_start(6:7),*)   startdate(2)
      read(time_coverage_start(9:10),*)  startdate(3)
      read(time_coverage_start(12:13),*) startdate(4)
      read(time_coverage_start(15:16),*) startdate(5)
      read(time_coverage_start(18:19),*) startdate(6)
c      print *, startdate

c     iorbit is set to zero in SSMIS files (!), so estimate it from frcrev
      iorbit=0
      N=0
      do i=1,nscans
         if (orbit_position(i) .GT. 0) then
            iorbit=iorbit+int(orbit_position(i))
            N=N+1
         endif
      end do
      iorbit=iorbit/N



c     Loop over scans...
      do i=1,MAX_SCANS
         if ((i<=nscans) .and. (scan_time_hires(i)>10000000)) then
            frcrev(i)=orbit_position(i)-real(iorbit)

c           Create stdtime_A
c           scan_time_hires is UTC "seconds since 2000-01-01 00:00:00"
c           Note: UTC seconds includes leap seconds, 
c           Get year (note: 2000 was a leap year, 1900 was not)
c           Dates are refernced to 1/1/2000, so start seconds at 86400
            targsecs=0
            tmpdate(1)=2000
            do j=1,30
               tmpsecs=targsecs+365*86400
               if (mod(tmpdate(1),4) .EQ. 0)
     &              tmpsecs=tmpsecs+86400
               if (tmpsecs .GE. scan_time_hires(i)) exit
               tmpdate(1)=tmpdate(1)+1
               targsecs=tmpsecs
            end do


c           Get the month of year
            tmpsecs=scan_time_hires(i)-targsecs
            do j=1,12
               tmpdays=DaysofYear(j)*86400
               if (mod(tmpdate(1),4) .EQ. 0) 
     &              tmpdays=DaysofYearL(j)*86400
               if (tmpsecs .LT. tmpdays) exit
               tmpsecs=tmpsecs-tmpdays
            end do
            tmpdate(2)=j
c           Calculate days from remaining seconds
            tmpdate(3)=1+tmpsecs/86400
c           Calculate hours, mins, secs from residual
            tmpsecs=tmpsecs-(tmpdate(3)-1)*86400
            tmpdate(4)=tmpsecs/3600
            tmpdate(5)=(tmpsecs-tmpdate(4)*3600)/60
            tmpdate(6)=tmpsecs-tmpdate(4)*3600-tmpdate(5)*60

c           Copy result into stdtime_A
            stdtime_A(i,1)=tmpdate(1)
            stdtime_A(i,2)=tmpdate(2)
            stdtime_A(i,3)=tmpdate(3)
            stdtime_A(i,4)=tmpdate(4)
            stdtime_A(i,5)=tmpdate(5)
            stdtime_A(i,6)=tmpdate(6)          

c           Calculate itime87_ARR as secs since 1987-01-01
c           UTC seconds for 2000-01-01 = 946684800
c           UTC seconds for 1987-01-01 = 536457600
               itime87_ARR(i)=int(scan_time_hires(i))+
     &              946684800-536457600

c           Now write out btime from stdtime
            write (btime(i), 
     &              "(I0.4,A3,I0.2,I0.2,I0.2,I0.2,I0.2,A7)") 
     &              tmpdate(1),"000",tmpdate(2),tmpdate(3),tmpdate(4),
     &              tmpdate(5),tmpdate(6),".000000"

c           Convert flags to single number; zero implies good scan
            iscn_flag(i)=0
            do j=1,11
               iscn_flag(i)=iscn_flag(i)+
     &              int(iscn_flag_in(i,j))+127
            end do
            ical_flag(i)=0
            ical_flag_hi(i)=0
            do j=1,4
               ical_flag(i)=ical_flag(i)+ 
     &              int(ical_flag_lores(i,j))+127
               ical_flag_hi(i)=ical_flag_hi(i)+
     &              int(ical_flag_hires(i,j))+127
            end do

            do j=1,LOPIX
               tb_lo(1,j,i)=fcdr_brightness_temperature_19V(j,i)
               tb_lo(2,j,i)=fcdr_brightness_temperature_19H(j,i)
               tb_lo(3,j,i)=fcdr_brightness_temperature_22V(j,i)
               tb_lo(4,j,i)=fcdr_brightness_temperature_37V(j,i)
               tb_lo(5,j,i)=fcdr_brightness_temperature_37H(j,i)
               xlat_lo(j,i)=latitude_lores(j,i)*.01
               xlon_lo(j,i)=longitude_lores(j,i)*.01
               if (xlon_lo(j,i) .lt. 0) xlon_lo(j,i)=360+ xlon_lo(j,i)
               lnd(j,i)=land_flag_lores(j,i)
            end do

c           Convert lo-res Tb from V7 to V6 calibration
c           Note: this is not done for the hi-res channels
            IF (iver.EQ.1) CALL convert_v7_to_v6(acoef, tb_lo(:,j,i))

            do j=1,HIPIX
               tb_hi(1,j,i)=fcdr_brightness_temperature_92V(j,i)
               tb_hi(2,j,i)=fcdr_brightness_temperature_92H(j,i)
               xlat_hi(j,i)=latitude_hires(j,i)*.01
               xlon_hi(j,i)=longitude_hires(j,i)*.01
               if (xlon_hi(j,i) .lt. 0) xlon_hi(j,i)=360+ xlon_hi(j,i)
               lnd_hi(j,i)=land_flag_hires(j,i)
            end do

         else
            ctime='                        '
            iscn_flag(i)=-9
            ical_flag(i)=-9
            ical_flag_hi(i)=-9
            do j=1,LOPIX
               tb_lo(1,j,i)=-999.9
               tb_lo(2,j,i)=-999.9
               tb_lo(3,j,i)=-999.9
               tb_lo(4,j,i)=-999.9
               tb_lo(5,j,i)=-999.9
               xlat_lo(j,i)=-999.9
               xlon_lo(j,i)=-999.9
               lnd(j,i)=-9
            end do
            do j=1,HIPIX
               tb_hi(1,j,i)=-999.9
               tb_hi(2,j,i)=-999.9
               xlat_hi(j,i)=-999.9
               xlon_hi(j,i)=-999.9
               lnd_hi(j,i)=-9
            end do
         end if

         btime(i)=ctime
      end do

      end subroutine ReadNC_RSS_SSMIS_Tb_v6







      SUBROUTINE HANDLE_ERR(status, errmessage)
      character*256 errmessage
      INTEGER status

      IF (STATUS .NE. NF_NOERR) THEN
         PRINT *, NF_STRERROR(STATUS), ':', errmessage
         STOP "Stopping due to NetCDF errors in ReadNC_RSS_SSMIS_Tb_v6"
      ENDIF
      END







      SUBROUTINE READ_v7_to_v6_conversion_table(acoef)
      IMPLICIT NONE
      SAVE
C
C     EJN 1/5/2012 - Correct a bug in RSS-supplied code.  The read
C     should be an implied DO loop, not a DO k = 1,3/END DO.
C
      INTEGER(4) ifreq, ipol, istart, k, iunit_coef
      REAL(4) acoef(3,2,4)
      LOGICAL in_use
C
      DATA istart/1/
C
      IF (istart.EQ.1) THEN  !table only needs be READ in during first call
         istart = 0
C
        in_use = .TRUE.
        iunit_coef  = 2
        DO  WHILE ( in_use )
            iunit_coef = iunit_coef + 1
            INQUIRE ( iunit_coef, OPENED=in_use )
        END DO
C        OPEN(3,file='v7_to_v6_conversion_table.txt')
         OPEN(iunit_coef,file='GPCPcdr_d2/GPCPcdr_d2_GPROFv2004_SSMIS'//
     &       '/v7_to_v6_conversion_table.txt')
         DO ifreq = 1,4
            DO ipol= 1,2
C                 READ(3,3001) (acoef(k,ipol,ifreq), k=1,3)
                  READ(iunit_coef,3001) (acoef(k,ipol,ifreq), k=1,3)
 3001             FORMAT(3f10.6)
            END DO
         END DO
C        CLOSE(3)
         CLOSE(iunit_coef)
      ENDIF
C
      RETURN
      END
C
C
      SUBROUTINE convert_v7_to_v6(acoef, tb)
      IMPLICIT NONE
C
      REAL(4) acoef(3,2,4)
      REAL(4) tb(5),tbx(5)
      INTEGER(4) j
C
      DO j = 1, 5
         tbx(j) = tb(j)
      END DO
C
      tb(1) = acoef(1,1,1) + acoef(2,1,1)*tbx(1) + acoef(3,1,1)*tbx(2)
      tb(2) = acoef(1,2,1) + acoef(2,2,1)*tbx(1) + acoef(3,2,1)*tbx(2)
      tb(3) = acoef(1,1,2) + acoef(2,1,2)*tbx(3)
      tb(4) = acoef(1,1,3) + acoef(2,1,3)*tbx(4) + acoef(3,1,3)*tbx(5)
      tb(5) = acoef(1,2,3) + acoef(2,2,3)*tbx(4) + acoef(3,2,3)*tbx(5)
C
C     tb(1:2) = acoef(1,:,1) + acoef(2,:,1)*tbx(1) + acoef(3,:,1)*tbx(2)
C     tb(  3) = acoef(1,1,2) + acoef(2,1,2)*tbx(3)
C     tb(4:5) = acoef(1,:,3) + acoef(2,:,3)*tbx(4) + acoef(3,:,3)*tbx(5)
C
      RETURN
      END






      SUBROUTINE convert_hi_v7_to_v6(acoef, tb)
      IMPLICIT NONE
C
      REAL(4) acoef(3,2,4)
      REAL(4) tb(2),tbx(2)
      INTEGER(4) j
C
      DO j = 1, 2
         tbx(j) = tb(j)
      END DO
C
      tb(1) = acoef(1,1,4) + acoef(2,1,4)*tbx(1) + acoef(3,1,4)*tbx(2)
      tb(2) = acoef(1,2,4) + acoef(2,2,4)*tbx(1) + acoef(3,2,4)*tbx(2)
C
C     tb(1:2) = acoef(1,:,4) + acoef(2,:,4)*tbx(1) + acoef(3,:,4)*tbx(2)
C
      RETURN
      END





